@@ -20,11 +20,11 @@
  */
 
 import * as tests from "@gnu-taler/web-util/testing";
-import { AmlExchangeBackend } from "../types.js";
 import {
   CasesUI as TestedComponent,
 } from "./Cases.js";
 import { AmountString } from "@gnu-taler/taler-util";
+import { AmlExchangeBackend } from "../utils/types.js";
 
 export default {
   title: "cases",
