@@ -1657,7 +1657,7 @@ export function mergeDiscoveryAggregate(
   newPolicies: PolicyMetaInfo[],
   oldAgg: AggregatedPolicyMetaInfo[],
 ): AggregatedPolicyMetaInfo[] {
-  const aggregatedPolicies: AggregatedPolicyMetaInfo[] = [...oldAgg] ?? [];
+  const aggregatedPolicies: AggregatedPolicyMetaInfo[] = [...oldAgg];
   const polHashToIndex: Record<string, number> = {};
   for (const pol of newPolicies) {
     const oldIndex = polHashToIndex[pol.policy_hash];
