@@ -1,6 +1,6 @@
 /*
  This file is part of GNU Taler
- (C) 2022 Taler Systems S.A.
+ (C) 2022-2024 Taler Systems S.A.
 
  GNU Taler is free software; you can redistribute it and/or modify it under the
  terms of the GNU General Public License as published by the Free Software
@@ -18,7 +18,10 @@ import { TalerError } from "@gnu-taler/taler-util";
 import { useCashouts } from "../../hooks/circuit.js";
 import { Props, State } from "./index.js";
 
-export function useComponentState({ account, onSelected }: Props): State {
+export function useComponentState({
+  account,
+  routeCashoutDetails,
+}: Props): State {
   const result = useCashouts(account);
   if (!result) {
     return {
@@ -35,14 +38,14 @@ export function useComponentState({ account, onSelected }: Props): State {
   if (result.type === "fail") {
     return {
       status: "failed",
-      error: result
-    }
+      error: result,
+    };
   }
 
   return {
     status: "ready",
     error: undefined,
     cashouts: result.body.cashouts,
-    onSelected,
+    routeCashoutDetails,
   };
 }
