@@ -1,6 +1,6 @@
 /*
  This file is part of GNU Taler
- (C) 2022 Taler Systems S.A.
+ (C) 2022-2024 Taler Systems S.A.
 
  GNU Taler is free software; you can redistribute it and/or modify it under the
  terms of the GNU General Public License as published by the Free Software
@@ -19,14 +19,13 @@
  * @author Sebastian Javier Marchano (sebasjm)
  */
 import { useState } from "preact/hooks";
-// import { cancelPendingRequest } from "./backend";
 
 export interface Options {
   slowTolerance: number;
 }
 
 export interface AsyncOperationApi<T> {
-  request: (...a: any) => void;
+  request: (...a: Array<unknown>) => void;
   cancel: () => void;
   data: T | undefined;
   isSlow: boolean;
@@ -35,15 +34,15 @@ export interface AsyncOperationApi<T> {
 }
 
 export function useAsync<T>(
-  fn?: (...args: any) => Promise<T>,
+  fn?: (...args: Array<unknown>) => Promise<T>,
   { slowTolerance: tooLong }: Options = { slowTolerance: 1000 },
 ): AsyncOperationApi<T> {
   const [data, setData] = useState<T | undefined>(undefined);
   const [isLoading, setLoading] = useState<boolean>(false);
-  const [error, setError] = useState<any>(undefined);
+  const [error, setError] = useState<string | undefined>(undefined);
   const [isSlow, setSlow] = useState(false);
 
-  const request = async (...args: any) => {
+  const request = async (...args: Array<unknown>) => {
     if (!fn) return;
     setLoading(true);
     const handler = setTimeout(() => {
@@ -54,7 +53,11 @@ export function useAsync<T>(
       const result = await fn(...args);
       setData(result);
     } catch (error) {
-      setError(error);
+      if (error instanceof Error) {
+        setError(error.message);
+      } else {
+        setError(`Unknown error: ${error}`);
+      }
     }
     setLoading(false);
     setSlow(false);
