@@ -1,6 +1,6 @@
 /*
  This file is part of GNU Taler
- (C) 2022 Taler Systems S.A.
+ (C) 2022-2024 Taler Systems S.A.
 
  GNU Taler is free software; you can redistribute it and/or modify it under the
  terms of the GNU General Public License as published by the Free Software
@@ -26,7 +26,7 @@ export type ValueOrFunction<T> = T | ((p: T) => T);
 
 const calculateRootPath = () => {
   const rootPath =
-    typeof window !== undefined
+    typeof window !== "undefined"
       ? window.location.origin + window.location.pathname
       : "/";
   return rootPath;
