@@ -1,6 +1,6 @@
 /*
  This file is part of GNU Taler
- (C) 2022 Taler Systems S.A.
+ (C) 2022-2024 Taler Systems S.A.
 
  GNU Taler is free software; you can redistribute it and/or modify it under the
  terms of the GNU General Public License as published by the Free Software
@@ -16,8 +16,12 @@
 
 import App from "./components/app.js";
 import { h, render } from "preact";
-import "./scss/main.css"
+import "./scss/main.css";
 
 const app = document.getElementById("app");
 
-render(<App />, app as any);
+if (app) {
+  render(<App />, app);
+} else {
+  console.error("HTML element with id 'app' not found.");
+}
