@@ -17,6 +17,7 @@ import { h, VNode } from "preact";
 import { Router, route, Route } from "preact-router";
 import InstanceCreatePage from "./paths/admin/create/index.js";
 import InstanceListPage from "./paths/admin/list/index.js";
+import { InstancePaths } from "./InstanceRoutes.js";
 
 export enum AdminPaths {
   list_instances = "/instances",
@@ -42,7 +43,7 @@ export function AdminRoutes(): VNode {
         component={InstanceCreatePage}
         onBack={() => route(AdminPaths.list_instances)}
         onConfirm={() => {
-          // route(AdminPaths.list_instances);
+          route(InstancePaths.bank_list);
         }}
 
         // onError={(error: any) => {
