@@ -18,12 +18,12 @@
  *
  * @author Sebastian Javier Marchano (sebasjm)
  */
-import { HttpStatusCode } from "@gnu-taler/taler-util";
+import { HttpStatusCode, TranslatedString } from "@gnu-taler/taler-util";
 import { ErrorType, useTranslationContext } from "@gnu-taler/web-util/browser";
 import { createHashHistory } from "history";
 import { Fragment, VNode, h } from "preact";
 import { Route, Router, route } from "preact-router";
-import { useState } from "preact/hooks";
+import { useEffect, useErrorBoundary, useState } from "preact/hooks";
 import { InstanceRoutes } from "./InstanceRoutes.js";
 import {
   NotConnectedAppMenu,
@@ -54,7 +54,6 @@ export function ApplicationReadyRoutes(): VNode {
     updateToken(token)
     setUnauthorized(false)
   }
-
   const result = useBackendInstancesTestForAdmin();
 
   const clearTokenAndGoToRoot = () => {
