@@ -19,64 +19,62 @@
  * @author Sebastian Javier Marchano (sebasjm)
  */
 
+import { TranslatedString } from "@gnu-taler/taler-util";
 import {
-  useTranslationContext,
-  HttpError,
   ErrorType,
+  HttpError,
+  useTranslationContext
 } from "@gnu-taler/web-util/browser";
 import { format } from "date-fns";
-import { Fragment, FunctionComponent, h, VNode } from "preact";
-import { Route, route, Router } from "preact-router";
-import { useCallback, useEffect, useMemo, useState } from "preact/hooks";
+import { Fragment, FunctionComponent, VNode, h } from "preact";
+import { Route, Router, route } from "preact-router";
+import { useEffect, useErrorBoundary, useMemo, useState } from "preact/hooks";
 import { Loading } from "./components/exception/loading.js";
 import { Menu, NotificationCard } from "./components/menu/index.js";
-import { useBackendContext } from "./context/backend.js";
 import { InstanceContextProvider } from "./context/instance.js";
+import { LoginToken, MerchantBackend } from "./declaration.js";
+import { useInstanceBankAccounts } from "./hooks/bank.js";
 import {
   useBackendDefaultToken,
   useBackendInstanceToken,
   useSimpleLocalStorage,
 } from "./hooks/index.js";
 import { useInstanceKYCDetails } from "./hooks/instance.js";
+import { dateFormatForSettings, useSettings } from "./hooks/useSettings.js";
 import InstanceCreatePage from "./paths/admin/create/index.js";
 import InstanceListPage from "./paths/admin/list/index.js";
-import TokenPage from "./paths/instance/token/index.js";
+import BankAccountCreatePage from "./paths/instance/accounts/create/index.js";
+import BankAccountListPage from "./paths/instance/accounts/list/index.js";
+import BankAccountUpdatePage from "./paths/instance/accounts/update/index.js";
 import ListKYCPage from "./paths/instance/kyc/list/index.js";
 import OrderCreatePage from "./paths/instance/orders/create/index.js";
 import OrderDetailsPage from "./paths/instance/orders/details/index.js";
 import OrderListPage from "./paths/instance/orders/list/index.js";
-import ProductCreatePage from "./paths/instance/products/create/index.js";
-import ProductListPage from "./paths/instance/products/list/index.js";
-import ProductUpdatePage from "./paths/instance/products/update/index.js";
-import BankAccountCreatePage from "./paths/instance/accounts/create/index.js";
-import BankAccountListPage from "./paths/instance/accounts/list/index.js";
-import BankAccountUpdatePage from "./paths/instance/accounts/update/index.js";
-import ReservesCreatePage from "./paths/instance/reserves/create/index.js";
-import ReservesDetailsPage from "./paths/instance/reserves/details/index.js";
-import ReservesListPage from "./paths/instance/reserves/list/index.js";
-import TemplateCreatePage from "./paths/instance/templates/create/index.js";
-import TemplateUsePage from "./paths/instance/templates/use/index.js";
-import TemplateQrPage from "./paths/instance/templates/qr/index.js";
-import TemplateListPage from "./paths/instance/templates/list/index.js";
-import TemplateUpdatePage from "./paths/instance/templates/update/index.js";
-import WebhookCreatePage from "./paths/instance/webhooks/create/index.js";
-import WebhookListPage from "./paths/instance/webhooks/list/index.js";
-import WebhookUpdatePage from "./paths/instance/webhooks/update/index.js";
 import ValidatorCreatePage from "./paths/instance/otp_devices/create/index.js";
 import ValidatorListPage from "./paths/instance/otp_devices/list/index.js";
 import ValidatorUpdatePage from "./paths/instance/otp_devices/update/index.js";
+import ProductCreatePage from "./paths/instance/products/create/index.js";
+import ProductListPage from "./paths/instance/products/list/index.js";
+import ProductUpdatePage from "./paths/instance/products/update/index.js";
+import TemplateCreatePage from "./paths/instance/templates/create/index.js";
+import TemplateListPage from "./paths/instance/templates/list/index.js";
+import TemplateQrPage from "./paths/instance/templates/qr/index.js";
+import TemplateUpdatePage from "./paths/instance/templates/update/index.js";
+import TemplateUsePage from "./paths/instance/templates/use/index.js";
+import TokenPage from "./paths/instance/token/index.js";
 import TransferCreatePage from "./paths/instance/transfers/create/index.js";
 import TransferListPage from "./paths/instance/transfers/list/index.js";
 import InstanceUpdatePage, {
   AdminUpdate as InstanceAdminUpdatePage,
   Props as InstanceUpdatePageProps,
 } from "./paths/instance/update/index.js";
+import WebhookCreatePage from "./paths/instance/webhooks/create/index.js";
+import WebhookListPage from "./paths/instance/webhooks/list/index.js";
+import WebhookUpdatePage from "./paths/instance/webhooks/update/index.js";
 import { LoginPage } from "./paths/login/index.js";
 import NotFoundPage from "./paths/notfound/index.js";
-import { Notification } from "./utils/types.js";
-import { LoginToken, MerchantBackend } from "./declaration.js";
 import { Settings } from "./paths/settings/index.js";
-import { dateFormatForSettings, useSettings } from "./hooks/useSettings.js";
+import { Notification } from "./utils/types.js";
 
 export enum InstancePaths {
   error = "/error",
@@ -151,7 +149,7 @@ export function InstanceRoutes({
   const [token, updateToken] = useBackendInstanceToken(id);
   const { i18n } = useTranslationContext();
 
-  type GlobalNotifState = (Notification & { to: string }) | undefined;
+  type GlobalNotifState = (Notification & { to: string | undefined }) | undefined;
   const [globalNotification, setGlobalNotification] =
     useState<GlobalNotifState>(undefined);
 
@@ -163,15 +161,17 @@ export function InstanceRoutes({
     }
     onLoginPass()
   };
-  // const updateLoginStatus = (url: string, token?: string) => {
-  //   changeToken(token);
-  // };
+
+  const [error] = useErrorBoundary();
 
   const value = useMemo(
     () => ({ id, token, admin, changeToken }),
     [id, token, admin],
   );
 
+  const instance = useInstanceBankAccounts();
+  const accounts = !instance.ok ? undefined : instance.data.accounts;
+
   function ServerErrorRedirectTo(to: InstancePaths | AdminPaths) {
     return function ServerErrorRedirectToImpl(
       error: HttpError<MerchantBackend.ErrorDetail>,
@@ -229,7 +229,7 @@ export function InstanceRoutes({
             <InstanceCreatePage
               forceId="default"
               onConfirm={() => {
-                route(InstancePaths.order_list);
+                route(InstancePaths.bank_list);
               }}
             />
           </Fragment>
@@ -249,6 +249,28 @@ export function InstanceRoutes({
     updateDefaultToken(undefined)
   };
 
+  if (accounts !== undefined && !admin && accounts.length < 1) {
+    return <InstanceContextProvider value={value}>
+      <Menu
+        instance={id}
+        admin={admin}
+        onShowSettings={() => {
+          route(InstancePaths.interface)
+        }}
+        path={path}
+        onLogout={clearTokenAndGoToRoot}
+        setInstanceName={setInstanceName}
+        isPasswordOk={defaultToken !== undefined}
+      />
+      <NotificationCard notification={{
+        type: "INFO",
+        message: i18n.str`You need to associate a bank account to receive revenue.`,
+        description: i18n.str`Without this the merchant backend will refuse to create new orders.`
+      }} />
+      <BankAccountCreatePage onConfirm={() => { }} />
+    </InstanceContextProvider>
+  }
+
   return (
     <InstanceContextProvider value={value}>
       <Menu
@@ -264,6 +286,15 @@ export function InstanceRoutes({
       />
       <KycBanner />
       <NotificationCard notification={globalNotification} />
+      {error &&
+        <NotificationCard notification={{
+          message: "Internal error, please repot",
+          type: "ERROR",
+          description: <pre>
+            {(error instanceof Error ? error.stack : String(error)) as TranslatedString}
+          </pre>
+        }} />
+      }
 
       <Router
         onChange={(e) => {
@@ -633,42 +664,6 @@ export function InstanceRoutes({
           }}
         />
 
-        {/**
-         * reserves pages
-         */}
-        <Route
-          path={InstancePaths.reserves_list}
-          component={ReservesListPage}
-          onUnauthorized={LoginPageAccessDenied}
-          onNotFound={IfAdminCreateDefaultOr(NotFoundPage)}
-          onLoadError={ServerErrorRedirectTo(InstancePaths.settings)}
-          onSelect={(id: string) => {
-            route(InstancePaths.reserves_details.replace(":rid", id));
-          }}
-          onCreate={() => {
-            route(InstancePaths.reserves_new);
-          }}
-        />
-        <Route
-          path={InstancePaths.reserves_details}
-          component={ReservesDetailsPage}
-          onUnauthorized={LoginPageAccessDenied}
-          onLoadError={ServerErrorRedirectTo(InstancePaths.reserves_list)}
-          onNotFound={IfAdminCreateDefaultOr(NotFoundPage)}
-          onBack={() => {
-            route(InstancePaths.reserves_list);
-          }}
-        />
-        <Route
-          path={InstancePaths.reserves_new}
-          component={ReservesCreatePage}
-          onConfirm={() => {
-            route(InstancePaths.reserves_list);
-          }}
-          onBack={() => {
-            route(InstancePaths.reserves_list);
-          }}
-        />
         <Route path={InstancePaths.kyc} component={ListKYCPage} />
         <Route path={InstancePaths.interface} component={Settings} />
         {/**
