@@ -158,14 +158,6 @@ export function Sidebar({
                   </span>
                 </a>
               </li>
-              <li>
-                <a href={"/reserves"} class="has-icon">
-                  <span class="icon">
-                    <i class="mdi mdi-cash" />
-                  </span>
-                  <span class="menu-item-label">Reserves</span>
-                </a>
-              </li>
               <li>
                 <a href={"/webhooks"} class="has-icon">
                   <span class="icon">
@@ -215,14 +207,6 @@ export function Sidebar({
               </span>
             </a>
           </li>
-          <li>
-            <div>
-              <span style={{ width: "3rem" }} class="icon">
-                <i class="mdi mdi-currency-eur" />
-              </span>
-              <span class="menu-item-label">{config.currency}</span>
-            </div>
-          </li>
           <li>
             <div>
               <span style={{ width: "3rem" }} class="icon">
