@@ -21,12 +21,9 @@
 
 import { createContext } from "preact";
 import { useContext } from "preact/hooks";
+import { MerchantBackend } from "../declaration.js";
 
-interface Type {
-  currency: string;
-  version: string;
-}
-const Context = createContext<Type>(null!);
+const Context = createContext<MerchantBackend.VersionResponse>(null!);
 
 export const ConfigContextProvider = Context.Provider;
-export const useConfigContext = (): Type => useContext(Context);
+export const useConfigContext = (): MerchantBackend.VersionResponse => useContext(Context);
