@@ -132,8 +132,6 @@ interface useBackendInstanceRequestType {
     options?: RequestOptions,
   ) => Promise<HttpResponseOk<T>>;
   fetcher: <T>(endpoint: string) => Promise<HttpResponseOk<T>>;
-  reserveDetailFetcher: <T>(endpoint: string) => Promise<HttpResponseOk<T>>;
-  rewardsDetailFetcher: <T>(endpoint: string) => Promise<HttpResponseOk<T>>;
   multiFetcher: <T>(params: [url: string[]]) => Promise<HttpResponseOk<T>[]>;
   orderFetcher: <T>(
     params: [endpoint: string,
@@ -285,7 +283,7 @@ export function useBackendInstanceRequest(): useBackendInstanceRequestType {
 
   const { baseUrl, token: loginToken } = !admin
     ? { baseUrl: rootBackendUrl, token: rootToken }
-    : { baseUrl: `${rootBackendUrl}/instances/${id}`, token: instanceToken };
+    : { baseUrl: rootBackendUrl, token: instanceToken };
 
   const token = loginToken?.token;
 
@@ -353,34 +351,6 @@ export function useBackendInstanceRequest(): useBackendInstanceRequestType {
     [baseUrl, token],
   );
 
-  const reserveDetailFetcher = useCallback(
-    function reserveDetailFetcherImpl<T>(
-      endpoint: string,
-    ): Promise<HttpResponseOk<T>> {
-      return requestHandler<T>(baseUrl, endpoint, {
-        params: {
-          rewards: "yes",
-        },
-        token,
-      });
-    },
-    [baseUrl, token],
-  );
-
-  const rewardsDetailFetcher = useCallback(
-    function rewardsDetailFetcherImpl<T>(
-      endpoint: string,
-    ): Promise<HttpResponseOk<T>> {
-      return requestHandler<T>(baseUrl, endpoint, {
-        params: {
-          pickups: "yes",
-        },
-        token,
-      });
-    },
-    [baseUrl, token],
-  );
-
   const transferFetcher = useCallback(
     function transferFetcherImpl<T>(
       args: [endpoint: string,
@@ -468,8 +438,6 @@ export function useBackendInstanceRequest(): useBackendInstanceRequestType {
     fetcher,
     multiFetcher,
     orderFetcher,
-    reserveDetailFetcher,
-    rewardsDetailFetcher,
     transferFetcher,
     templateFetcher,
     webhookFetcher,
