@@ -67,11 +67,11 @@ export function useBackendDefaultToken(
 ): [LoginToken | undefined, ((d: LoginToken | undefined) => void)] {
   const { update: setToken, value: tokenMap, reset } = useLocalStorage(TOKENS_KEY, {})
 
-  const tokenOfDefaultInstance = tokenMap["default"]
-  const clearCache = useMatchMutate()
-  useEffect(() => {
-    clearCache()
-  }, [tokenOfDefaultInstance])
+  // const tokenOfDefaultInstance = tokenMap["default"]
+  // const clearCache = useMatchMutate()
+  // useEffect(() => {
+  //   clearCache()
+  // }, [tokenOfDefaultInstance])
 
   function updateToken(
     value: (LoginToken | undefined)
