@@ -32,13 +32,16 @@ import { MerchantBackend } from "../../../declaration.js";
 import { INSTANCE_ID_REGEX } from "../../../utils/constants.js";
 import { undefinedIfEmpty } from "../../../utils/table.js";
 import { SetTokenNewInstanceModal } from "../../../components/modal/index.js";
+import { Duration } from "@gnu-taler/taler-util";
 
-export type Entity = MerchantBackend.Instances.InstanceConfigurationMessage & {
+export type Entity = Omit<Omit<MerchantBackend.Instances.InstanceConfigurationMessage, "default_pay_delay">, "default_wire_transfer_delay"> & {
   auth_token?: string;
+  default_pay_delay: Duration,
+  default_wire_transfer_delay: Duration,
 };
 
 interface Props {
-  onCreate: (d: Entity) => Promise<void>;
+  onCreate: (d: MerchantBackend.Instances.InstanceConfigurationMessage) => Promise<void>;
   onBack?: () => void;
   forceId?: string;
 }
@@ -49,8 +52,8 @@ function with_defaults(id?: string): Partial<Entity> {
     // accounts: [],
     user_type: "business",
     use_stefan: true,
-    default_pay_delay: { d_us: 2 * 1000 * 60 * 60 * 1000 }, // two hours
-    default_wire_transfer_delay: { d_us: 1000 * 2 * 60 * 60 * 24 * 1000 }, // two days
+    default_pay_delay: { d_ms: 2 * 60 * 60 * 1000 }, // two hours
+    default_wire_transfer_delay: { d_ms: 2 * 60 * 60 * 24 * 1000 }, // two days
   };
 }
 
@@ -88,9 +91,9 @@ export function CreatePage({ onCreate, onBack, forceId }: Props): VNode {
     default_pay_delay: !value.default_pay_delay
       ? i18n.str`required`
       : !!value.default_wire_transfer_delay &&
-        value.default_wire_transfer_delay.d_us !== "forever" &&
-        value.default_pay_delay.d_us !== "forever" &&
-        value.default_pay_delay.d_us > value.default_wire_transfer_delay.d_us ?
+        value.default_wire_transfer_delay.d_ms !== "forever" &&
+        value.default_pay_delay.d_ms !== "forever" &&
+        value.default_pay_delay.d_ms > value.default_wire_transfer_delay.d_ms ?
         i18n.str`pay delay can't be greater than wire transfer delay` : undefined,
     default_wire_transfer_delay: !value.default_wire_transfer_delay
       ? i18n.str`required`
@@ -115,16 +118,23 @@ export function CreatePage({ onCreate, onBack, forceId }: Props): VNode {
 
   const submit = (): Promise<void> => {
     // use conversion instead of this
-    const newToken = value.auth_token;
-    value.auth_token = undefined;
-    value.auth = newToken === null || newToken === undefined
+    const newValue = structuredClone(value);
+
+    const newToken = newValue.auth_token;
+    newValue.auth_token = undefined;
+    newValue.auth = newToken === null || newToken === undefined
       ? { method: "external" }
       : { method: "token", token: `secret-token:${newToken}` };
-    if (!value.address) value.address = {};
-    if (!value.jurisdiction) value.jurisdiction = {};
+    if (!newValue.address) newValue.address = {};
+    if (!newValue.jurisdiction) newValue.jurisdiction = {};
     // remove above use conversion
     // schema.validateSync(value, { abortEarly: false })
-    return onCreate(value as Entity);
+    newValue.default_pay_delay = Duration.toTalerProtocolDuration(newValue.default_pay_delay!) as any
+    newValue.default_wire_transfer_delay = Duration.toTalerProtocolDuration(newValue.default_wire_transfer_delay!) as any
+    // delete value.default_pay_delay;
+    // delete value.default_wire_transfer_delay;
+
+    return onCreate(newValue as any as MerchantBackend.Instances.InstanceConfigurationMessage);
   };
 
   function updateToken(token: string | null) {
@@ -174,54 +184,54 @@ export function CreatePage({ onCreate, onBack, forceId }: Props): VNode {
             </FormProvider>
 
             <div class="level">
-            <div class="level-item has-text-centered">
-              <h1 class="title">
-                <button
-                  class={
-                    !isTokenSet
-                      ? "button is-danger has-tooltip-bottom"
-                      : !value.auth_token
-                        ? "button has-tooltip-bottom"
-                        : "button is-info has-tooltip-bottom"
-                  }
-                  data-tooltip={i18n.str`change authorization configuration`}
-                  onClick={() => updateIsTokenDialogActive(true)}
-                >
-                  <div class="icon is-centered">
-                    <i class="mdi mdi-lock-reset" />
-                  </div>
-                  <span>
-                    <i18n.Translate>Set access token</i18n.Translate>
-                  </span>
-                </button>
-              </h1>
+              <div class="level-item has-text-centered">
+                <h1 class="title">
+                  <button
+                    class={
+                      !isTokenSet
+                        ? "button is-danger has-tooltip-bottom"
+                        : !value.auth_token
+                          ? "button has-tooltip-bottom"
+                          : "button is-info has-tooltip-bottom"
+                    }
+                    data-tooltip={i18n.str`change authorization configuration`}
+                    onClick={() => updateIsTokenDialogActive(true)}
+                  >
+                    <div class="icon is-centered">
+                      <i class="mdi mdi-lock-reset" />
+                    </div>
+                    <span>
+                      <i18n.Translate>Set access token</i18n.Translate>
+                    </span>
+                  </button>
+                </h1>
+              </div>
             </div>
-          </div>
-          <div class="level">
-            <div class="level-item has-text-centered">
-              {!isTokenSet ? (
-                <p class="is-size-6">
-                  <i18n.Translate>
-                    Access token is not yet configured. This instance can't be
-                    created.
-                  </i18n.Translate>
-                </p>
-              ) : value.auth_token === undefined ? (
-                <p class="is-size-6">
-                  <i18n.Translate>
-                    No access token. Authorization must be handled externally.
-                  </i18n.Translate>
-                </p>
-              ) : (
-                <p class="is-size-6">
-                  <i18n.Translate>
-                    Access token is set. Authorization is handled by the
-                    merchant backend.
-                  </i18n.Translate>
-                </p>
-              )}
+            <div class="level">
+              <div class="level-item has-text-centered">
+                {!isTokenSet ? (
+                  <p class="is-size-6">
+                    <i18n.Translate>
+                      Access token is not yet configured. This instance can't be
+                      created.
+                    </i18n.Translate>
+                  </p>
+                ) : value.auth_token === undefined ? (
+                  <p class="is-size-6">
+                    <i18n.Translate>
+                      No access token. Authorization must be handled externally.
+                    </i18n.Translate>
+                  </p>
+                ) : (
+                  <p class="is-size-6">
+                    <i18n.Translate>
+                      Access token is set. Authorization is handled by the
+                      merchant backend.
+                    </i18n.Translate>
+                  </p>
+                )}
+              </div>
             </div>
-          </div>
             <div class="buttons is-right mt-5">
               {onBack && (
                 <button class="button" onClick={onBack}>
