@@ -124,27 +124,6 @@ export const InstanceSchema = yup.object().shape({
 export const InstanceUpdateSchema = InstanceSchema.clone().omit(["id"]);
 export const InstanceCreateSchema = InstanceSchema.clone();
 
-export const AuthorizeRewardSchema = yup.object().shape({
-  justification: yup.string().required(),
-  amount: yup
-    .string()
-    .required()
-    .test("amount", "the amount is not valid", currencyWithAmountIsValid)
-    .test("amount_positive", "the amount is not valid", currencyGreaterThan0),
-  next_url: yup.string().required(),
-});
-
-const stringIsValidJSON = (value?: string) => {
-  const p = value?.trim();
-  if (!p) return true;
-  try {
-    JSON.parse(p);
-    return true;
-  } catch {
-    return false;
-  }
-};
-
 export const OrderCreateSchema = yup.object().shape({
   pricing: yup
     .object()
