@@ -26,9 +26,9 @@
  */
 import {
   AccountAddDetails,
+  AccountRestriction,
   AmountJson,
   Amounts,
-  TalerCorebankApiClient,
   Configuration,
   CoreApiResponse,
   Duration,
@@ -36,6 +36,7 @@ import {
   Logger,
   MerchantInstanceConfig,
   PartialMerchantInstanceConfig,
+  TalerCorebankApiClient,
   TalerError,
   WalletNotification,
   createEddsaKeyPair,
@@ -43,9 +44,9 @@ import {
   encodeCrock,
   hash,
   j2s,
+  openPromise,
   parsePaytoUri,
   stringToBytes,
-  AccountRestriction,
 } from "@gnu-taler/taler-util";
 import {
   HttpRequestLibrary,
@@ -57,7 +58,6 @@ import {
   WalletCoreRequestType,
   WalletCoreResponseType,
   WalletOperations,
-  openPromise,
 } from "@gnu-taler/taler-wallet-core";
 import {
   RemoteWallet,
@@ -73,7 +73,6 @@ import * as http from "http";
 import * as net from "node:net";
 import * as path from "path";
 import * as readline from "readline";
-import { URL } from "url";
 import { CoinConfig } from "./denomStructures.js";
 
 const logger = new Logger("harness.ts");
@@ -358,7 +357,7 @@ export class GlobalTestState {
     logName: string,
     env: { [index: string]: string | undefined } = process.env,
   ): ProcessWrapper {
-    logger.trace(
+    logger.info(
       `spawning process (${logName}): ${shellescape([command, ...args])}`,
     );
     const proc = spawn(command, args, {
@@ -371,7 +370,7 @@ export class GlobalTestState {
     });
     proc.on("exit", (code, signal) => {
       if (code == 0 && signal == null) {
-        logger.info(`process ${logName} exited with success`);
+        logger.trace(`process ${logName} exited with success`);
       } else {
         logger.warn(`process ${logName} exited ${j2s({ code, signal })}`);
       }
@@ -413,6 +412,17 @@ export class GlobalTestState {
       }
     }
   }
+
+  /**
+   * Log that the test arrived a certain step.
+   *
+   * The step name should be unique across the whole
+   */
+  logStep(stepName: string): void {
+    // Now we just log, later we may report the steps that were done
+    // to easily see where the test hangs.
+    console.info(`STEP: ${stepName}`);
+  }
 }
 
 export function shouldLingerInTest(): boolean {
@@ -595,6 +605,11 @@ export interface HarnessExchangeBankAccount {
 
   debitRestrictions?: AccountRestriction[];
   creditRestrictions?: AccountRestriction[];
+
+  /**
+   * If set, the harness will not automatically configure the wire fee for this account.
+   */
+  skipWireFeeCreation?: boolean;
 }
 
 /**
@@ -729,7 +744,7 @@ export class FakebankService
 }
 
 /**
- * Implementation of the bank service using the "taler-fakebank-run" tool.
+ * Implementation of the bank service using the libeufin-bank implementation.
  */
 export class LibeufinBankService
   extends BankServiceBase
@@ -766,21 +781,14 @@ export class LibeufinBankService
     config.setString("libeufin-bank", "serve", "tcp");
     config.setString(
       "libeufin-bank",
-      "DEFAULT_CUSTOMER_DEBT_LIMIT",
-      `${bc.currency}:500`,
-    );
-    config.setString(
-      "libeufin-bank",
-      "DEFAULT_ADMIN_DEBT_LIMIT",
-      `${bc.currency}:999999`,
+      "DEFAULT_DEBT_LIMIT",
+      `${bc.currency}:100`,
     );
     config.setString(
       "libeufin-bank",
       "registration_bonus",
       `${bc.currency}:100`,
     );
-    config.setString("libeufin-bank", "registration_bonus_enabled", `yes`);
-    config.setString("libeufin-bank", "max_auth_token_duration", "1h");
     const cfgFilename = testDir + "/bank.conf";
     config.write(cfgFilename, { excludeDefaults: true });
 
@@ -804,7 +812,7 @@ export class LibeufinBankService
         .required(),
       httpPort: config.getNumber("libeufin-bank", "port").required(),
       maxDebt: config
-        .getString("libeufin-bank", "DEFAULT_CUSTOMER_DEBT_LIMIT")
+        .getString("libeufin-bank", "DEFAULT_DEBT_LIMIT")
         .required(),
     };
     return new FakebankService(gc, bc, cfgFilename);
@@ -850,10 +858,16 @@ export class LibeufinBankService
 
     await sh(
       this.globalTestState,
-      "libeufin-bank-dbinit",
+      "libeufin-bank-passwd",
       `libeufin-bank passwd -c "${this.configFile}" admin adminpw`,
     );
 
+    await sh(
+      this.globalTestState,
+      "libeufin-bank-edit-account",
+      `libeufin-bank edit-account -c "${this.configFile}" admin --debit_threshold=${this.bankConfig.currency}:1000`,
+    );
+
     this.proc = this.globalTestState.spawnService(
       "libeufin-bank",
       ["serve", "-c", this.configFile],
@@ -890,6 +904,7 @@ export interface ExchangeConfig {
   httpPort: number;
   database: string;
   overrideTestDir?: string;
+  overrideWireFee?: string;
 }
 
 export interface ExchangeServiceInterface {
@@ -930,11 +945,11 @@ export class ExchangeService implements ExchangeServiceInterface {
 
   private exchangeBankAccounts: HarnessExchangeBankAccount[] = [];
 
-  setTimetravel(t: number | undefined): void {
+  setTimetravel(tMs: number | undefined): void {
     if (this.isRunning()) {
       throw Error("can't set time travel while the exchange is running");
     }
-    this.currentTimetravelOffsetMs = t;
+    this.currentTimetravelOffsetMs = tMs;
   }
 
   private get timetravelArg(): string | undefined {
@@ -980,29 +995,24 @@ export class ExchangeService implements ExchangeServiceInterface {
       this.globalState,
       `exchange-${this.name}-aggregator-once`,
       "taler-exchange-aggregator",
-      [...timetravelArgArr, "-c", this.configFilename, "-t"],
+      [...timetravelArgArr, "-c", this.configFilename, "-t", "-y", "-LINFO"],
     );
   }
 
   async runAggregatorOnce() {
-    try {
-      await runCommand(
-        this.globalState,
-        `exchange-${this.name}-aggregator-once`,
-        "taler-exchange-aggregator",
-        [...this.timetravelArgArr, "-c", this.configFilename, "-t", "-y"],
-      );
-    } catch (e) {
-      logger.info(
-        "running aggregator with KYC off didn't work, might be old version, running again",
-      );
-      await runCommand(
-        this.globalState,
-        `exchange-${this.name}-aggregator-once`,
-        "taler-exchange-aggregator",
-        [...this.timetravelArgArr, "-c", this.configFilename, "-t"],
-      );
-    }
+    await runCommand(
+      this.globalState,
+      `exchange-${this.name}-aggregator-once`,
+      "taler-exchange-aggregator",
+      [
+        ...this.timetravelArgArr,
+        "-c",
+        this.configFilename,
+        "-t",
+        "-y",
+        "-LINFO",
+      ],
+    );
   }
 
   async runTransferOnce() {
@@ -1014,6 +1024,31 @@ export class ExchangeService implements ExchangeServiceInterface {
     );
   }
 
+  async runTransferOnceWithTimetravel(opts: {
+    timetravelMicroseconds: number;
+  }) {
+    let timetravelArgArr = [];
+    timetravelArgArr.push(`--timetravel=${opts.timetravelMicroseconds}`);
+    await runCommand(
+      this.globalState,
+      `exchange-${this.name}-transfer-once`,
+      "taler-exchange-transfer",
+      [...timetravelArgArr, "-c", this.configFilename, "-t"],
+    );
+  }
+
+  /**
+   * Run the taler-exchange-expire command once in test mode.
+   */
+  async runExpireOnce() {
+    await runCommand(
+      this.globalState,
+      `exchange-${this.name}-expire-once`,
+      "taler-exchange-expire",
+      [...this.timetravelArgArr, "-c", this.configFilename, "-t"],
+    );
+  }
+
   changeConfig(f: (config: Configuration) => void) {
     const config = Configuration.load(this.configFilename);
     f(config);
@@ -1057,6 +1092,9 @@ export class ExchangeService implements ExchangeServiceInterface {
     config.setString("taler-exchange-secmod-eddsa", "lookahead_sign", "20 s");
     config.setString("taler-exchange-secmod-rsa", "lookahead_sign", "20 s");
 
+    // FIXME: Remove once the exchange default config properly ships this.
+    config.setString("exchange", "EXPIRE_IDLE_SLEEP_INTERVAL", "1 s");
+
     const exchangeMasterKey = createEddsaKeyPair();
 
     config.setString(
@@ -1214,6 +1252,15 @@ export class ExchangeService implements ExchangeServiceInterface {
     }
   }
 
+  async stopAggregator(): Promise<void> {
+    const agg = this.exchangeAggregatorProc;
+    if (agg) {
+      agg.proc.kill("SIGTERM");
+      await agg.wait();
+      this.exchangeAggregatorProc = undefined;
+    }
+  }
+
   async startWirewatch(): Promise<void> {
     const wirewatch = this.exchangeWirewatchProc;
     if (wirewatch) {
@@ -1288,7 +1335,6 @@ export class ExchangeService implements ExchangeServiceInterface {
       if (!p) {
         throw Error(`invalid payto uri in exchange config: ${paytoUri}`);
       }
-      accountTargetTypes.add(p?.targetType);
       const optArgs: string[] = [];
       if (acct.conversionUrl != null) {
         optArgs.push("conversion-url", acct.conversionUrl);
@@ -1307,31 +1353,44 @@ export class ExchangeService implements ExchangeServiceInterface {
           "upload",
         ],
       );
+
+      const accTargetType = p.targetType;
+
+      const covered = accountTargetTypes.has(p.targetType);
+      if (!covered && !acct.skipWireFeeCreation) {
+        const year = new Date().getFullYear();
+
+        for (let i = year; i < year + 5; i++) {
+          const wireFee =
+            this.exchangeConfig.overrideWireFee ??
+            `${this.exchangeConfig.currency}:0.01`;
+          await runCommand(
+            this.globalState,
+            "exchange-offline",
+            "taler-exchange-offline",
+            [
+              "-c",
+              this.configFilename,
+              "wire-fee",
+              // Year
+              `${i}`,
+              // Wire method
+              accTargetType,
+              // Wire fee
+              wireFee,
+              // Closing fee
+              `${this.exchangeConfig.currency}:0.01`,
+              "upload",
+            ],
+          );
+          accountTargetTypes.add(accTargetType);
+        }
+      }
     }
 
-    const year = new Date().getFullYear();
-    for (const accTargetType of accountTargetTypes.values()) {
-      for (let i = year; i < year + 5; i++) {
-        await runCommand(
-          this.globalState,
-          "exchange-offline",
-          "taler-exchange-offline",
-          [
-            "-c",
-            this.configFilename,
-            "wire-fee",
-            // Year
-            `${i}`,
-            // Wire method
-            accTargetType,
-            // Wire fee
-            `${this.exchangeConfig.currency}:0.01`,
-            // Closing fee
-            `${this.exchangeConfig.currency}:0.01`,
-            "upload",
-          ],
-        );
-      }
+    const wireTypeConfigured: Set<string> = new Set();
+
+    for (const acct of this.exchangeBankAccounts) {
     }
 
     await runCommand(
@@ -1871,16 +1930,39 @@ function tryUnixConnect(socketPath: string): Promise<void> {
 
 export interface WalletServiceOptions {
   useInMemoryDb?: boolean;
+  /**
+   * Use a particular DB path instead of the default one in the
+   * test environment.
+   */
+  overrideDbPath?: string;
   name: string;
 }
 
+/**
+ * A wallet service that listens on a unix domain socket for commands.
+ */
 export class WalletService {
   walletProc: ProcessWrapper | undefined;
 
+  private internalDbPath: string;
+
   constructor(
     private globalState: GlobalTestState,
     private opts: WalletServiceOptions,
-  ) {}
+  ) {
+    if (this.opts.overrideDbPath) {
+      this.internalDbPath = this.opts.overrideDbPath;
+    } else {
+      if (this.opts.useInMemoryDb) {
+        this.internalDbPath = ":memory:";
+      } else {
+        this.internalDbPath = path.join(
+          this.globalState.testDir,
+          `walletdb-${this.opts.name}.sqlite3`,
+        );
+      }
+    }
+  }
 
   get socketPath() {
     const unixPath = path.join(
@@ -1891,10 +1973,7 @@ export class WalletService {
   }
 
   get dbPath() {
-    return path.join(
-      this.globalState.testDir,
-      `walletdb-${this.opts.name}.json`,
-    );
+    return this.internalDbPath;
   }
 
   async stop(): Promise<void> {
@@ -1905,27 +1984,19 @@ export class WalletService {
   }
 
   async start(): Promise<void> {
-    let dbPath: string;
-    if (this.opts.useInMemoryDb) {
-      dbPath = ":memory:";
-    } else {
-      dbPath = path.join(
-        this.globalState.testDir,
-        `walletdb-${this.opts.name}.json`,
-      );
-    }
     const unixPath = this.socketPath;
     this.walletProc = this.globalState.spawnService(
       "taler-wallet-cli",
       [
         "--wallet-db",
-        dbPath,
+        this.dbPath,
         "-LTRACE", // FIXME: Make this configurable?
         "--no-throttle", // FIXME: Optionally do throttling for some tests?
         "advanced",
         "serve",
         "--unix-path",
         unixPath,
+        "--no-init",
       ],
       `wallet-${this.opts.name}`,
     );
@@ -1953,6 +2024,7 @@ export class WalletService {
 }
 
 export interface WalletClientArgs {
+  name?: string;
   unixPath: string;
   onNotification?(n: WalletNotification): void;
 }
@@ -1995,6 +2067,7 @@ export class WalletClient {
     const waiter = this.waiter;
     const walletClient = this;
     const w = await createRemoteWallet({
+      name: this.args.name,
       socketFilename: this.args.unixPath,
       notificationHandler(n) {
         if (walletClient.args.onNotification) {
@@ -2106,7 +2179,7 @@ export class WalletCli {
     return this._client;
   }
 
-  async runUntilDone(args: { maxRetries?: number } = {}): Promise<void> {
+  async runUntilDone(args: {} = {}): Promise<void> {
     await runCommand(
       this.globalTestState,
       `wallet-${this.name}`,
@@ -2119,7 +2192,6 @@ export class WalletCli {
         "--wallet-db",
         this.dbfile,
         "run-until-done",
-        ...(args.maxRetries ? ["--max-retries", `${args.maxRetries}`] : []),
       ],
     );
   }
@@ -2180,7 +2252,7 @@ export function generateRandomPayto(label: string): string {
     return `payto://iban/SANDBOXX/${generateRandomTestIban(
       label,
     )}?receiver-name=${label}`;
-  return `payto://x-taler-bank/localhost/${label}`;
+  return `payto://x-taler-bank/localhost/${label}?receiver-name=${label}`;
 }
 
 function waitMs(ms: number): Promise<void> {
