@@ -25,14 +25,15 @@
  */
 import {
   AmountString,
-  TalerCorebankApiClient,
   ConfirmPayResultType,
   Duration,
   Logger,
   MerchantApiClient,
   MerchantContractTerms,
   NotificationType,
+  PartialWalletRunConfig,
   PreparePayResultType,
+  TalerCorebankApiClient,
   TransactionMajorState,
   WalletNotification,
 } from "@gnu-taler/taler-util";
@@ -385,7 +386,7 @@ export async function createSimpleTestkudosEnvironmentV2(
 
   const { walletClient, walletService } = await createWalletDaemonWithClient(
     t,
-    { name: "wallet" },
+    { name: "wallet", persistent: true },
   );
 
   console.log("setup done!");
@@ -405,6 +406,8 @@ export interface CreateWalletArgs {
   handleNotification?(wn: WalletNotification): void;
   name: string;
   persistent?: boolean;
+  overrideDbPath?: string;
+  config?: PartialWalletRunConfig;
 }
 
 export async function createWalletDaemonWithClient(
@@ -414,22 +417,39 @@ export async function createWalletDaemonWithClient(
   const walletService = new WalletService(t, {
     name: args.name,
     useInMemoryDb: !args.persistent,
+    overrideDbPath: args.overrideDbPath,
   });
   await walletService.start();
   await walletService.pingUntilAvailable();
 
+  const observabilityEventFile = t.testDir + `/wallet-${args.name}-notifs.log`;
+
+  const onNotif = (notif: WalletNotification) => {
+    if (observabilityEventFile) {
+      fs.appendFileSync(
+        observabilityEventFile,
+        new Date().toISOString() + " " + JSON.stringify(notif) + "\n",
+      );
+    }
+    if (args.handleNotification) {
+      args.handleNotification(notif);
+    }
+  };
+
   const walletClient = new WalletClient({
+    name: args.name,
     unixPath: walletService.socketPath,
-    onNotification(n) {
-      console.log(`got ${args.name} notification`, n);
-      if (args.handleNotification) {
-        args.handleNotification(n);
-      }
-    },
+    onNotification: onNotif,
   });
   await walletClient.connect();
+  const defaultRunConfig = {
+    testing: {
+      skipDefaults: true,
+      emitObservabilityEvents: !!process.env["TALER_TEST_OBSERVABILITY"],
+    },
+  } satisfies PartialWalletRunConfig;
   await walletClient.client.call(WalletApiOperation.InitWallet, {
-    skipDefaults: true,
+    config: args.config ?? defaultRunConfig,
   });
 
   return { walletClient, walletService };
@@ -675,7 +695,7 @@ export async function makeTestPaymentV2(
   );
 
   const r2 = await walletClient.call(WalletApiOperation.ConfirmPay, {
-    proposalId: preparePayResult.proposalId,
+    transactionId: preparePayResult.transactionId,
   });
 
   t.assertTrue(r2.type === ConfirmPayResultType.Done);
