@@ -20,24 +20,19 @@
 import {
   AmountString,
   Amounts,
+  BalancesResponse,
   Configuration,
   Duration,
   HttpStatusCode,
   Logger,
-  MerchantApiClient,
   MerchantInstanceConfig,
   RegisterAccountRequest,
-  TalerCoreBankHttpClient,
-  TalerCorebankApiClient,
-  TalerError,
-  TestForApi,
-  addPaytoQueryParams,
+  TransactionsResponse,
   decodeCrock,
   generateIban,
   j2s,
   rsaBlind,
   setGlobalLogLevelFromString,
-  setPrintHttpRequestAsCurl,
 } from "@gnu-taler/taler-util";
 import { clk } from "@gnu-taler/taler-util/clk";
 import {
@@ -47,12 +42,14 @@ import {
 import {
   CryptoDispatcher,
   SynchronousCryptoWorkerFactoryPlain,
+  WalletApiOperation,
+} from "@gnu-taler/taler-wallet-core";
+import {
   downloadExchangeInfo,
   topupReserveWithDemobank,
-} from "@gnu-taler/taler-wallet-core";
+} from "@gnu-taler/taler-wallet-core/dbless";
 import { deepStrictEqual } from "assert";
 import fs from "fs";
-import { createRandomTestUser, createTestForBankCore } from "http-client/bank-core.js";
 import os from "os";
 import path from "path";
 import { runBench1 } from "./bench1.js";
@@ -62,9 +59,14 @@ import { runEnvFull } from "./env-full.js";
 import { runEnv1 } from "./env1.js";
 import {
   GlobalTestState,
+  WalletClient,
   delayMs,
   runTestWithState,
 } from "./harness/harness.js";
+import {
+  createSimpleTestkudosEnvironmentV2,
+  createWalletDaemonWithClient,
+} from "./harness/helpers.js";
 import { getTestInfo, runTests } from "./integrationtests/testrunner.js";
 import { lintExchangeDeployment } from "./lint.js";
 
@@ -77,8 +79,9 @@ process.on("unhandledRejection", (error: any) => {
 });
 
 declare const __VERSION__: string;
+declare const __GIT_HASH__: string;
 function printVersion(): void {
-  console.log(__VERSION__);
+  console.log(`${__VERSION__} ${__GIT_HASH__}`);
   process.exit(0);
 }
 
@@ -181,9 +184,165 @@ advancedCli
     await runTestWithState(testState, runEnv1, "env1", true);
   });
 
-const sandcastleCli = testingCli.subcommand("sandcastleArgs", "sandcastle", {
-  help: "Subcommands for handling GNU Taler sandcastle deployments.",
-});
+async function doDbChecks(
+  t: GlobalTestState,
+  walletClient: WalletClient,
+  indir: string,
+): Promise<void> {
+  // Check that balance didn't break
+  const balPath = `${indir}/wallet-balances.json`;
+  const expectedBal: BalancesResponse = JSON.parse(
+    fs.readFileSync(balPath, { encoding: "utf8" }),
+  ) as BalancesResponse;
+  const actualBal = await walletClient.call(WalletApiOperation.GetBalances, {});
+  t.assertDeepEqual(actualBal.balances.length, expectedBal.balances.length);
+
+  // Check that transactions didn't break
+  const txnPath = `${indir}/wallet-transactions.json`;
+  const expectedTxn: TransactionsResponse = JSON.parse(
+    fs.readFileSync(txnPath, { encoding: "utf8" }),
+  ) as TransactionsResponse;
+  const actualTxn = await walletClient.call(
+    WalletApiOperation.GetTransactions,
+    { includeRefreshes: true },
+  );
+  t.assertDeepEqual(
+    actualTxn.transactions.length,
+    expectedTxn.transactions.length,
+  );
+}
+
+advancedCli
+  .subcommand("walletDbcheck", "wallet-dbcheck", {
+    help: "Check a wallet database (used for migration testing).",
+  })
+  .requiredArgument("indir", clk.STRING)
+  .action(async (args) => {
+    const indir = args.walletDbcheck.indir;
+    if (!fs.existsSync(indir)) {
+      throw Error("directory to be checked does not exist");
+    }
+
+    const testRootDir = fs.mkdtempSync(path.join(os.tmpdir(), "taler-dbchk-"));
+    const t: GlobalTestState = new GlobalTestState({
+      testDir: testRootDir,
+    });
+    const origWalletDbPath = `${indir}/wallet-db.sqlite3`;
+    const testWalletDbPath = `${testRootDir}/wallet-testdb.sqlite3`;
+    fs.cpSync(origWalletDbPath, testWalletDbPath);
+    if (!fs.existsSync(origWalletDbPath)) {
+      throw new Error("wallet db to be checked does not exist");
+    }
+    const { walletClient, walletService } = await createWalletDaemonWithClient(
+      t,
+      { name: "wallet-loaded", overrideDbPath: testWalletDbPath },
+    );
+
+    await walletService.pingUntilAvailable();
+
+    // Do DB checks with the DB we loaded.
+    await doDbChecks(t, walletClient, indir);
+
+    const {
+      walletClient: freshWalletClient,
+      walletService: freshWalletService,
+    } = await createWalletDaemonWithClient(t, {
+      name: "wallet-fresh",
+      persistent: false,
+    });
+
+    await freshWalletService.pingUntilAvailable();
+
+    // Check that we can still import the backup JSON.
+
+    const backupPath = `${indir}/wallet-backup.json`;
+    const backupData = JSON.parse(
+      fs.readFileSync(backupPath, { encoding: "utf8" }),
+    );
+    await freshWalletClient.call(WalletApiOperation.ImportDb, {
+      dump: backupData,
+    });
+
+    // Repeat same checks with wallet that we restored from backup
+    // instead of from the DB file.
+    await doDbChecks(t, freshWalletClient, indir);
+
+    await t.shutdown();
+  });
+
+advancedCli
+  .subcommand("walletDbgen", "wallet-dbgen", {
+    help: "Generate a wallet test database (to be used for migration testing).",
+  })
+  .requiredArgument("outdir", clk.STRING)
+  .action(async (args) => {
+    const outdir = args.walletDbgen.outdir;
+    if (fs.existsSync(outdir)) {
+      throw new Error("outdir already exists, please delete first");
+    }
+    fs.mkdirSync(outdir, {
+      recursive: true,
+    });
+
+    const testRootDir = fs.mkdtempSync(path.join(os.tmpdir(), "taler-dbgen-"));
+    console.log(`generating data in ${testRootDir}`);
+    const t = new GlobalTestState({
+      testDir: testRootDir,
+    });
+    const { walletClient, walletService, bank, exchange, merchant } =
+      await createSimpleTestkudosEnvironmentV2(t);
+    await walletClient.call(WalletApiOperation.RunIntegrationTestV2, {
+      amountToSpend: "TESTKUDOS:5" as AmountString,
+      amountToWithdraw: "TESTKUDOS:10" as AmountString,
+      corebankApiBaseUrl: bank.corebankApiBaseUrl,
+      exchangeBaseUrl: exchange.baseUrl,
+      merchantBaseUrl: merchant.makeInstanceBaseUrl(),
+    });
+    await walletClient.call(
+      WalletApiOperation.TestingWaitTransactionsFinal,
+      {},
+    );
+
+    const transactionsJson = await walletClient.call(
+      WalletApiOperation.GetTransactions,
+      {
+        includeRefreshes: true,
+      },
+    );
+
+    const balancesJson = await walletClient.call(
+      WalletApiOperation.GetBalances,
+      {},
+    );
+
+    const backupJson = await walletClient.call(WalletApiOperation.ExportDb, {});
+
+    const versionJson = await walletClient.call(
+      WalletApiOperation.GetVersion,
+      {},
+    );
+
+    await walletService.stop();
+
+    await t.shutdown();
+
+    console.log(`generated data in ${testRootDir}`);
+
+    fs.copyFileSync(walletService.dbPath, `${outdir}/wallet-db.sqlite3`);
+    fs.writeFileSync(
+      `${outdir}/wallet-transactions.json`,
+      j2s(transactionsJson),
+    );
+    fs.writeFileSync(`${outdir}/wallet-balances.json`, j2s(balancesJson));
+    fs.writeFileSync(`${outdir}/wallet-backup.json`, j2s(backupJson));
+    fs.writeFileSync(`${outdir}/wallet-version.json`, j2s(versionJson));
+    fs.writeFileSync(
+      `${outdir}/meta.json`,
+      j2s({
+        timestamp: new Date(),
+      }),
+    );
+  });
 
 const configCli = testingCli.subcommand("configArgs", "config", {
   help: "Subcommands for handling the Taler configuration.",
@@ -222,101 +381,6 @@ const deploymentCli = testingCli.subcommand("deploymentArgs", "deployment", {
   help: "Subcommands for handling GNU Taler deployments.",
 });
 
-deploymentCli
-  .subcommand("tipTopup", "tip-topup")
-  .requiredOption("merchantBaseUrl", ["--merchant-url"], clk.STRING)
-  .requiredOption("exchangeBaseUrl", ["--exchange-url"], clk.STRING)
-  .requiredOption("merchantApikey", ["--merchant-apikey"], clk.STRING)
-  .requiredOption("bankAccessUrl", ["--bank-access-url"], clk.STRING)
-  .requiredOption("bankAccount", ["--bank-account"], clk.STRING)
-  .requiredOption("bankPassword", ["--bank-password"], clk.STRING)
-  .requiredOption("wireMethod", ["--wire-method"], clk.STRING)
-  .requiredOption("amount", ["--amount"], clk.AMOUNT)
-  .action(async (args) => {
-    const amount = args.tipTopup.amount;
-
-    const merchantClient = new MerchantApiClient(
-      args.tipTopup.merchantBaseUrl,
-      {
-        method: "token",
-        token: args.tipTopup.merchantApikey,
-      },
-    );
-
-    const res = await merchantClient.getPrivateInstanceInfo();
-    console.log(res);
-
-    const tipReserveResp = await merchantClient.createTippingReserve({
-      exchange_url: args.tipTopup.exchangeBaseUrl,
-      initial_balance: amount,
-      wire_method: args.tipTopup.wireMethod,
-    });
-
-    console.log(tipReserveResp);
-
-    const bankAccessApiClient = new TalerCorebankApiClient(
-      args.tipTopup.bankAccessUrl,
-      {
-        auth: {
-          username: args.tipTopup.bankAccount,
-          password: args.tipTopup.bankPassword,
-        },
-      },
-    );
-
-    const paytoUri = addPaytoQueryParams(tipReserveResp.accounts[0].payto_uri, {
-      message: `tip-reserve ${tipReserveResp.reserve_pub}`,
-    });
-
-    console.log("payto URI:", paytoUri);
-
-    const transactions = await bankAccessApiClient.getTransactions(
-      args.tipTopup.bankAccount,
-    );
-    console.log("transactions:", j2s(transactions));
-
-    await bankAccessApiClient.createTransaction(args.tipTopup.bankAccount, {
-      amount,
-      paytoUri,
-    });
-  });
-
-deploymentCli
-  .subcommand("tipCleanup", "tip-cleanup")
-  .requiredOption("merchantBaseUrl", ["--merchant-url"], clk.STRING)
-  .requiredOption("merchantApikey", ["--merchant-apikey"], clk.STRING)
-  .flag("dryRun", ["--dry-run"])
-  .action(async (args) => {
-    const merchantClient = new MerchantApiClient(
-      args.tipCleanup.merchantBaseUrl,
-      {
-        method: "token",
-        token: args.tipCleanup.merchantApikey,
-      },
-    );
-
-    const res = await merchantClient.getPrivateInstanceInfo();
-    console.log(res);
-
-    const tipRes = await merchantClient.getPrivateTipReserves();
-    console.log(tipRes);
-
-    for (const reserve of tipRes.reserves) {
-      if (Amounts.isZero(reserve.exchange_initial_amount)) {
-        if (args.tipCleanup.dryRun) {
-          logger.info(`dry run, would purge reserve ${reserve}`);
-        } else {
-          await merchantClient.deleteTippingReserve({
-            reservePub: reserve.reserve_pub,
-            purge: true,
-          });
-        }
-      }
-    }
-
-    // FIXME: Now delete reserves that are not filled yet
-  });
-
 deploymentCli
   .subcommand("testTalerdotnetDemo", "test-demodottalerdotnet")
   .action(async (args) => {
@@ -402,25 +466,6 @@ deploymentCli
     console.log("reserve status", reserveStatusResp.status);
   });
 
-deploymentCli
-  .subcommand("tipStatus", "tip-status")
-  .requiredOption("merchantBaseUrl", ["--merchant-url"], clk.STRING)
-  .requiredOption("merchantApikey", ["--merchant-apikey"], clk.STRING)
-  .action(async (args) => {
-    const merchantClient = new MerchantApiClient(
-      args.tipStatus.merchantBaseUrl,
-      {
-        method: "token",
-        token: args.tipStatus.merchantApikey,
-      },
-    );
-
-    const res = await merchantClient.getPrivateInstanceInfo();
-
-    const tipRes = await merchantClient.getPrivateTipReserves();
-    console.log(j2s(tipRes));
-  });
-
 deploymentCli
   .subcommand("lintExchange", "lint-exchange", {
     help: "Run checks on the exchange deployment.",
@@ -637,7 +682,7 @@ deploymentCli
       username: accountLogin,
       is_public: !!args.provisionBankAccount.public,
       is_taler_exchange: !!args.provisionBankAccount.exchange,
-      internal_payto_uri: args.provisionBankAccount.internalPayto,
+      payto_uri: args.provisionBankAccount.internalPayto,
     };
     const resp = await httpLib.fetch(url.href, {
       method: "POST",
@@ -657,96 +702,6 @@ deploymentCli
     process.exit(2);
   });
 
-
-type TestResult = { testName: string, caseName: string, result: "skiped" | "ok" | "fail", error?: any }
-
-async function getTestSummary<T extends object>(filter: string | undefined, ...apis: Array<TestForApi<T>>) {
-  const regex = !filter ? undefined : new RegExp(filter)
-  const apiState = await Promise.all(apis.flatMap(api => Object.entries(api).flatMap(([testName, casesMap]) => {
-    return Object.entries(casesMap).map(async ([caseName, caseFunc]): Promise<TestResult> => {
-      if (!caseFunc) {
-        return { testName, caseName, result: "skiped" as const }
-      }
-      if (regex && !regex.test(`${testName}:${caseName}`)) {
-        return { testName, caseName, result: "skiped" as const }
-      }
-      return caseFunc()
-        .then(r => ({ testName, caseName, result: "ok" as const }))
-        .catch(error => ({ testName, caseName, result: "fail" as const, error }))
-    })
-  })))
-
-  return apiState.reduce((prev, testResult) => {
-    if (testResult.result === "ok") {
-      prev.ok.push(testResult)
-    }
-    if (testResult.result === "skiped") {
-      prev.skiped.push(testResult)
-    }
-    if (testResult.result === "fail") {
-      prev.fail.push(testResult)
-    }
-    return prev
-  }, { "ok": [] as TestResult[], "skiped": [] as TestResult[], "fail": [] as TestResult[] })
-}
-
-deploymentCli
-  .subcommand("testBankAPI", "test-bank-api", {
-    help: "test api compatibility.",
-  })
-  .requiredArgument("corebankApiBaseUrl", clk.STRING)
-  .maybeOption("adminPwd", ["--admin-password"], clk.STRING)
-  .maybeOption("filter", ["--filter"], clk.STRING)
-  .flag("showCurl", ["--show-curl"])
-  .action(async (args) => {
-    const httpLib = createPlatformHttpLib();
-    const api = new TalerCoreBankHttpClient(args.testBankAPI.corebankApiBaseUrl, httpLib);
-
-    process.stdout.write("config: ");
-    const config = await api.getConfig()
-    if (!api.isCompatible(config.body.version)) {
-      console.log("fail")
-      return;
-    } else {
-      console.log("ok")
-    }
-
-    if (!args.testBankAPI.adminPwd) {
-      console.log("no admin password, exit")
-      return;
-    }
-
-    const resp = await api.getAuthenticationAPI("admin").createAccessToken(args.testBankAPI.adminPwd, {
-      scope: "readwrite"
-    })
-
-    if (args.testBankAPI.showCurl) {
-      setPrintHttpRequestAsCurl(true)
-    }
-
-    if (resp.type === "fail") {
-      console.log("wrong admin password")
-      return;
-    }
-
-    const bankCore = createTestForBankCore(api, resp.body.access_token)
-
-    const summary = await getTestSummary(args.testBankAPI.filter, bankCore)
-
-    console.log("successful tests:", summary.ok.length)
-    console.log("uncompleted tests:", summary.skiped.length)
-    summary.skiped.forEach((testResult) => {
-      if (testResult.result === "skiped") {
-        console.log("    ", testResult.testName, testResult.caseName)
-      }
-    })
-    console.log("failed tests:", summary.fail.length)
-    summary.fail.forEach((testResult, i) => {
-      console.log(i, ")", testResult)
-    })
-  });
-
-
 deploymentCli
   .subcommand("coincfg", "gen-coin-config", {
     help: "Generate a coin/denomination configuration for the exchange.",
@@ -757,6 +712,7 @@ deploymentCli
   .requiredOption("maxAmount", ["--max-amount"], clk.STRING, {
     help: "Largest denomination",
   })
+  .flag("noFees", ["--no-fees"])
   .action(async (args) => {
     let out = "";
 
@@ -783,7 +739,11 @@ deploymentCli
       out += `DURATION_SPEND = 2 years\n`;
       out += `DURATION_LEGAL = 6 years\n`;
       out += `FEE_WITHDRAW = ${currency}:0\n`;
-      out += `FEE_DEPOSIT = ${Amounts.stringify(min)}\n`;
+      if (args.coincfg.noFees) {
+        out += `FEE_DEPOSIT = ${currency}:0\n`;
+      } else {
+        out += `FEE_DEPOSIT = ${Amounts.stringify(min)}\n`;
+      }
       out += `FEE_REFRESH = ${currency}:0\n`;
       out += `FEE_REFUND = ${currency}:0\n`;
       out += `RSA_KEYSIZE = 2048\n`;
