@@ -28,21 +28,26 @@ import {
   shouldLingerInTest,
 } from "../harness/harness.js";
 import { getSharedTestDir } from "../harness/helpers.js";
+import { runAgeRestrictionsDepositTest } from "./test-age-restrictions-deposit.js";
 import { runAgeRestrictionsMerchantTest } from "./test-age-restrictions-merchant.js";
 import { runAgeRestrictionsMixedMerchantTest } from "./test-age-restrictions-mixed-merchant.js";
 import { runAgeRestrictionsPeerTest } from "./test-age-restrictions-peer.js";
 import { runBankApiTest } from "./test-bank-api.js";
 import { runClaimLoopTest } from "./test-claim-loop.js";
 import { runClauseSchnorrTest } from "./test-clause-schnorr.js";
+import { runCurrencyScopeTest } from "./test-currency-scope.js";
+import { runDenomLostTest } from "./test-denom-lost.js";
 import { runDenomUnofferedTest } from "./test-denom-unoffered.js";
 import { runDepositTest } from "./test-deposit.js";
 import { runExchangeDepositTest } from "./test-exchange-deposit.js";
+import { runExchangeManagementFaultTest } from "./test-exchange-management-fault.js";
 import { runExchangeManagementTest } from "./test-exchange-management.js";
 import { runExchangePurseTest } from "./test-exchange-purse.js";
 import { runExchangeTimetravelTest } from "./test-exchange-timetravel.js";
 import { runFeeRegressionTest } from "./test-fee-regression.js";
 import { runForcedSelectionTest } from "./test-forced-selection.js";
 import { runKycTest } from "./test-kyc.js";
+import { runLibeufinBankTest } from "./test-libeufin-bank.js";
 import { runMerchantExchangeConfusionTest } from "./test-merchant-exchange-confusion.js";
 import { runMerchantInstancesDeleteTest } from "./test-merchant-instances-delete.js";
 import { runMerchantInstancesUrlsTest } from "./test-merchant-instances-urls.js";
@@ -50,9 +55,12 @@ import { runMerchantInstancesTest } from "./test-merchant-instances.js";
 import { runMerchantLongpollingTest } from "./test-merchant-longpolling.js";
 import { runMerchantRefundApiTest } from "./test-merchant-refund-api.js";
 import { runMerchantSpecPublicOrdersTest } from "./test-merchant-spec-public-orders.js";
+import { runMultiExchangeTest } from "./test-multiexchange.js";
+import { runOtpTest } from "./test-otp.js";
 import { runPayPaidTest } from "./test-pay-paid.js";
 import { runPaymentAbortTest } from "./test-payment-abort.js";
 import { runPaymentClaimTest } from "./test-payment-claim.js";
+import { runPaymentDeletedTest } from "./test-payment-deleted.js";
 import { runPaymentExpiredTest } from "./test-payment-expired.js";
 import { runPaymentFaultTest } from "./test-payment-fault.js";
 import { runPaymentForgettableTest } from "./test-payment-forgettable.js";
@@ -76,26 +84,32 @@ import { runSimplePaymentTest } from "./test-simple-payment.js";
 import { runStoredBackupsTest } from "./test-stored-backups.js";
 import { runTimetravelAutorefreshTest } from "./test-timetravel-autorefresh.js";
 import { runTimetravelWithdrawTest } from "./test-timetravel-withdraw.js";
-import { runTippingTest } from "./test-tipping.js";
 import { runTermOfServiceFormatTest } from "./test-tos-format.js";
 import { runWalletBackupBasicTest } from "./test-wallet-backup-basic.js";
 import { runWalletBackupDoublespendTest } from "./test-wallet-backup-doublespend.js";
+import { runWalletBalanceNotificationsTest } from "./test-wallet-balance-notifications.js";
+import { runWalletBalanceZeroTest } from "./test-wallet-balance-zero.js";
 import { runWalletBalanceTest } from "./test-wallet-balance.js";
+import { runWalletCliTerminationTest } from "./test-wallet-cli-termination.js";
+import { runWalletConfigTest } from "./test-wallet-config.js";
 import { runWalletCryptoWorkerTest } from "./test-wallet-cryptoworker.js";
 import { runWalletDblessTest } from "./test-wallet-dbless.js";
+import { runWalletDd48Test } from "./test-wallet-dd48.js";
+import { runWalletDevExperimentsTest } from "./test-wallet-dev-experiments.js";
+import { runWalletGenDbTest } from "./test-wallet-gendb.js";
+import { runWalletInsufficientBalanceTest } from "./test-wallet-insufficient-balance.js";
 import { runWalletNotificationsTest } from "./test-wallet-notifications.js";
+import { runWalletObservabilityTest } from "./test-wallet-observability.js";
+import { runWalletRefreshTest } from "./test-wallet-refresh.js";
+import { runWalletWirefeesTest } from "./test-wallet-wirefees.js";
 import { runWallettestingTest } from "./test-wallettesting.js";
 import { runWithdrawalAbortBankTest } from "./test-withdrawal-abort-bank.js";
 import { runWithdrawalBankIntegratedTest } from "./test-withdrawal-bank-integrated.js";
+import { runWithdrawalConversionTest } from "./test-withdrawal-conversion.js";
 import { runWithdrawalFakebankTest } from "./test-withdrawal-fakebank.js";
 import { runWithdrawalFeesTest } from "./test-withdrawal-fees.js";
 import { runWithdrawalHugeTest } from "./test-withdrawal-huge.js";
 import { runWithdrawalManualTest } from "./test-withdrawal-manual.js";
-import { runWalletGenDbTest } from "./test-wallet-gendb.js";
-import { runLibeufinBankTest } from "./test-libeufin-bank.js";
-import { runMultiExchangeTest } from "./test-multiexchange.js";
-import { runAgeRestrictionsDepositTest } from "./test-age-restrictions-deposit.js";
-import { runWithdrawalConversionTest } from "./test-withdrawal-conversion.js";
 
 /**
  * Test runner.
@@ -123,7 +137,7 @@ const allTests: TestMainFunction[] = [
   runDenomUnofferedTest,
   runDepositTest,
   runSimplePaymentTest,
-  runExchangeManagementTest,
+  runExchangeManagementFaultTest,
   runExchangeTimetravelTest,
   runFeeRegressionTest,
   runForcedSelectionTest,
@@ -163,7 +177,6 @@ const allTests: TestMainFunction[] = [
   runWithdrawalManualTest,
   runTimetravelAutorefreshTest,
   runTimetravelWithdrawTest,
-  runTippingTest,
   runWalletBackupBasicTest,
   runWalletBackupDoublespendTest,
   runWalletNotificationsTest,
@@ -171,6 +184,7 @@ const allTests: TestMainFunction[] = [
   runWalletDblessTest,
   runWallettestingTest,
   runWithdrawalAbortBankTest,
+  // runWithdrawalNotifyBeforeTxTest,
   runWithdrawalBankIntegratedTest,
   runWithdrawalFakebankTest,
   runWithdrawalFeesTest,
@@ -181,6 +195,21 @@ const allTests: TestMainFunction[] = [
   runPaymentExpiredTest,
   runWalletGenDbTest,
   runLibeufinBankTest,
+  runPaymentDeletedTest,
+  runWalletDd48Test,
+  runCurrencyScopeTest,
+  runWalletRefreshTest,
+  runWalletCliTerminationTest,
+  runOtpTest,
+  runWalletBalanceNotificationsTest,
+  runExchangeManagementTest,
+  runWalletConfigTest,
+  runWalletObservabilityTest,
+  runWalletDevExperimentsTest,
+  runWalletBalanceZeroTest,
+  runWalletInsufficientBalanceTest,
+  runWalletWirefeesTest,
+  runDenomLostTest,
 ];
 
 export interface TestRunSpec {
