@@ -22,12 +22,12 @@
  * Imports.
  */
 import {
-  buildCodecForObject,
-  codecForString,
-  codecForNumber,
   Codec,
   Context,
   DecodingError,
+  buildCodecForObject,
+  codecForNumber,
+  codecForString,
   renderContext,
 } from "./codec.js";
 import { CurrencySpecification } from "./index.js";
@@ -51,9 +51,9 @@ export const amountFractionalLength = 8;
 export const amountMaxValue = 2 ** 52;
 
 /**
- * Separator character between interger and fractional
+ * Separator character between integer and fractional
  */
-export const FRAC_SEPARATOR = "."
+export const FRAC_SEPARATOR = ".";
 
 /**
  * Non-negative financial amount.  Fractional values are expressed as multiples
@@ -368,9 +368,16 @@ export class Amounts {
     return a.value === 0 && a.fraction === 0;
   }
 
+  /**
+   * Check whether a string is a valid currency for a Taler amount.
+   */
+  static isCurrency(s: string): boolean {
+    return /^[a-zA-Z]{1,11}$/.test(s);
+  }
+
   /**
    * Parse an amount like 'EUR:20.5' for 20 Euros and 50 ct.
-   * 
+   *
    * Currency name size limit is 11 of ASCII letters
    * Fraction size limit is 8
    */
@@ -379,7 +386,7 @@ export class Amounts {
     if (!res) {
       return undefined;
     }
-    const tail = res[3] || (FRAC_SEPARATOR + "0");
+    const tail = res[3] || FRAC_SEPARATOR + "0";
     if (tail.length > amountFractionalLength + 1) {
       return undefined;
     }
@@ -442,7 +449,7 @@ export class Amounts {
   static mult(a: AmountLike, n: number): Result {
     a = this.jsonifyAmount(a);
     if (!Number.isInteger(n)) {
-      throw Error("amount can only be multipied by an integer");
+      throw Error("amount can only be multiplied by an integer");
     }
     if (n < 0) {
       throw Error("amount can only be multiplied by a positive integer");
@@ -501,12 +508,16 @@ export class Amounts {
     return `${a.currency}:${s}` as AmountString;
   }
 
-  static isSameCurrency(a1: AmountLike, a2: AmountLike): boolean {
+  static amountHasSameCurrency(a1: AmountLike, a2: AmountLike): boolean {
     const x1 = this.jsonifyAmount(a1);
     const x2 = this.jsonifyAmount(a2);
     return x1.currency.toUpperCase() === x2.currency.toUpperCase();
   }
 
+  static isSameCurrency(curr1: string, curr2: string): boolean {
+    return curr1.toLowerCase() === curr2.toLowerCase();
+  }
+
   static stringifyValue(a: AmountLike, minFractional = 0): string {
     const aJ = Amounts.jsonifyAmount(a);
     const av = aJ.value + Math.floor(aJ.fraction / amountFractionalBase);
@@ -550,58 +561,76 @@ export class Amounts {
     return amountFractionalLength - i + 1;
   }
 
-  
-  static stringifyValueWithSpec(value: AmountJson, spec: CurrencySpecification): { currency: string, normal: string, small?: string } {
-    const strValue = Amounts.stringifyValue(value)
-    const pos = strValue.indexOf(FRAC_SEPARATOR)
+  static stringifyValueWithSpec(
+    value: AmountJson,
+    spec: CurrencySpecification,
+  ): { currency: string; normal: string; small?: string } {
+    const strValue = Amounts.stringifyValue(value);
+    const pos = strValue.indexOf(FRAC_SEPARATOR);
     const originalPosition = pos < 0 ? strValue.length : pos;
-  
-    let currency = value.currency
-    const names = Object.keys(spec.alt_unit_names)
-    let FRAC_POS_NEW_POSITION = originalPosition
+
+    let currency = value.currency;
+    const names = Object.keys(spec.alt_unit_names);
+    let FRAC_POS_NEW_POSITION = originalPosition;
     //find symbol
     //FIXME: this should be based on a cache to speed up
     if (names.length > 0) {
-      let unitIndex: string = "0" //default entry by DD51
-      names.forEach(index => {
-        const i = Number.parseInt(index, 10)
+      let unitIndex: string = "0"; //default entry by DD51
+      names.forEach((index) => {
+        const i = Number.parseInt(index, 10);
         if (Number.isNaN(i)) return; //skip
         if (originalPosition - i <= 0) return; //too big
         if (originalPosition - i < FRAC_POS_NEW_POSITION) {
           FRAC_POS_NEW_POSITION = originalPosition - i;
-          unitIndex = index
+          unitIndex = index;
         }
-      })
-      currency = spec.alt_unit_names[unitIndex]
+      });
+      currency = spec.alt_unit_names[unitIndex];
     }
-  
+
     if (originalPosition === FRAC_POS_NEW_POSITION) {
-      const { normal, small } = splitNormalAndSmall(strValue, originalPosition, spec)
-      return { currency, normal, small }
+      const { normal, small } = splitNormalAndSmall(
+        strValue,
+        originalPosition,
+        spec,
+      );
+      return { currency, normal, small };
     }
-  
-    const intPart = strValue.substring(0, originalPosition)
-    const fracPArt = strValue.substring(originalPosition + 1)
+
+    const intPart = strValue.substring(0, originalPosition);
+    const fracPArt = strValue.substring(originalPosition + 1);
     //indexSize is always smaller than originalPosition
-    const newValue = intPart.substring(0, FRAC_POS_NEW_POSITION) + FRAC_SEPARATOR + intPart.substring(FRAC_POS_NEW_POSITION)  + fracPArt
-    const { normal, small } = splitNormalAndSmall(newValue, FRAC_POS_NEW_POSITION, spec)
-    return { currency, normal, small }
+    const newValue =
+      intPart.substring(0, FRAC_POS_NEW_POSITION) +
+      FRAC_SEPARATOR +
+      intPart.substring(FRAC_POS_NEW_POSITION) +
+      fracPArt;
+    const { normal, small } = splitNormalAndSmall(
+      newValue,
+      FRAC_POS_NEW_POSITION,
+      spec,
+    );
+    return { currency, normal, small };
   }
-  
-  
 }
 
-function splitNormalAndSmall(decimal: string, fracSeparatorIndex: number, spec: CurrencySpecification): { normal: string, small?: string } {
+function splitNormalAndSmall(
+  decimal: string,
+  fracSeparatorIndex: number,
+  spec: CurrencySpecification,
+): { normal: string; small?: string } {
   let normal: string;
   let small: string | undefined;
-  if (decimal.length - fracSeparatorIndex - 1 > spec.num_fractional_normal_digits) {
-    const limit = fracSeparatorIndex + spec.num_fractional_normal_digits + 1
-    normal = decimal.substring(0, limit)
-    small = decimal.substring(limit)
+  if (
+    decimal.length - fracSeparatorIndex - 1 >
+    spec.num_fractional_normal_digits
+  ) {
+    const limit = fracSeparatorIndex + spec.num_fractional_normal_digits + 1;
+    normal = decimal.substring(0, limit);
+    small = decimal.substring(limit);
   } else {
-    normal = decimal
-    small = undefined
+    normal = decimal;
+    small = undefined;
   }
-  return { normal, small }
+  return { normal, small };
 }
-
