@@ -77,7 +77,7 @@ export interface PaytoUriBitcoin extends PaytoUriGeneric {
 
 const paytoPfx = "payto://";
 
-export type PaytoType = "iban" | "bitcoin" | "x-taler-bank"
+export type PaytoType = "iban" | "bitcoin" | "x-taler-bank";
 
 export function buildPayto(
   type: "iban",
@@ -111,7 +111,7 @@ export function buildPayto(
       return result;
     }
     case "iban": {
-      const uppercased = first.toUpperCase()
+      const uppercased = first.toUpperCase();
       const result: PaytoUriIBAN = {
         isKnown: true,
         targetType: "iban",
