@@ -844,7 +844,8 @@ export function hashDenomPub(pub: DenominationPubKey): Uint8Array {
     return hash(uint8ArrayBuf);
   } else {
     throw Error(
-      `unsupported cipher (${(pub as DenominationPubKey).cipher
+      `unsupported cipher (${
+        (pub as DenominationPubKey).cipher
       }), unable to hash`,
     );
   }
@@ -993,6 +994,7 @@ export enum TalerSignaturePurpose {
   WALLET_ACCOUNT_MERGE = 1214,
   WALLET_PURSE_ECONTRACT = 1216,
   WALLET_PURSE_DELETE = 1220,
+  WALLET_COIN_HISTORY = 1209,
   EXCHANGE_CONFIRM_RECOUP = 1039,
   EXCHANGE_CONFIRM_RECOUP_REFRESH = 1041,
   TALER_SIGNATURE_AML_DECISION = 1350,
@@ -1022,7 +1024,7 @@ export enum WalletAccountMergeFlags {
 export class SignaturePurposeBuilder {
   private chunks: Uint8Array[] = [];
 
-  constructor(private purposeNum: number) { }
+  constructor(private purposeNum: number) {}
 
   put(bytes: Uint8Array): SignaturePurposeBuilder {
     this.chunks.push(Uint8Array.from(bytes));
@@ -1433,7 +1435,7 @@ export namespace AgeRestriction {
 }
 
 // FIXME: make it a branded type!
-type EncryptionNonce = FlavorP<Uint8Array, "EncryptionNonce", 24>;
+export type EncryptionNonce = FlavorP<Uint8Array, "EncryptionNonce", 24>;
 
 async function deriveKey(
   keySeed: OpaqueData,
