@@ -27,9 +27,9 @@
 
 import { Amounts, codecForAmountString } from "./amounts.js";
 import {
+  Codec,
   buildCodecForObject,
   buildCodecForUnion,
-  Codec,
   codecForAny,
   codecForBoolean,
   codecForConstNumber,
@@ -38,20 +38,21 @@ import {
   codecForMap,
   codecForNumber,
   codecForString,
+  codecForStringURL,
   codecOptional,
 } from "./codec.js";
 import { strcmp } from "./helpers.js";
 import {
   CurrencySpecification,
   codecForCurrencySpecificiation,
+  codecForEither,
 } from "./index.js";
-import { AgeCommitmentProof, Edx25519PublicKeyEnc } from "./taler-crypto.js";
+import { Edx25519PublicKeyEnc } from "./taler-crypto.js";
 import {
-  codecForAbsoluteTime,
-  codecForDuration,
-  codecForTimestamp,
   TalerProtocolDuration,
   TalerProtocolTimestamp,
+  codecForDuration,
+  codecForTimestamp,
 } from "./time.js";
 
 /**
@@ -976,6 +977,8 @@ export class CheckPaymentResponse {
  * Response from the bank.
  */
 export class WithdrawOperationStatusResponse {
+  status: "selected" | "aborted" | "confirmed" | "pending";
+
   selection_done: boolean;
 
   transfer_done: boolean;
@@ -1227,9 +1230,42 @@ export interface MerchantOrderStatusUnpaid {
  * POST {talerBankIntegrationApi}/withdrawal-operation/{wopid}
  */
 export interface BankWithdrawalOperationPostResponse {
+  // Current status of the operation
+  // pending: the operation is pending parameters selection (exchange and reserve public key)
+  // selected: the operations has been selected and is pending confirmation
+  // aborted: the operation has been aborted
+  // confirmed: the transfer has been confirmed and registered by the bank
+  status: "selected" | "aborted" | "confirmed" | "pending";
+
+  // URL that the user needs to navigate to in order to
+  // complete some final confirmation (e.g. 2FA).
+  //
+  // Only applicable when status is selected or pending.
+  // It may contain withdrawal operation id
+  confirm_transfer_url?: string;
+
+  // Deprecated field use status instead
+  // The transfer has been confirmed and registered by the bank.
+  // Does not guarantee that the funds have arrived at the exchange already.
   transfer_done: boolean;
 }
 
+export const codeForBankWithdrawalOperationPostResponse =
+  (): Codec<BankWithdrawalOperationPostResponse> =>
+    buildCodecForObject<BankWithdrawalOperationPostResponse>()
+      .property(
+        "status",
+        codecForEither(
+          codecForConstString("selected"),
+          codecForConstString("confirmed"),
+          codecForConstString("aborted"),
+          codecForConstString("pending"),
+        ),
+      )
+      .property("confirm_transfer_url", codecOptional(codecForString()))
+      .property("transfer_done", codecForBoolean())
+      .build("BankWithdrawalOperationPostResponse");
+
 export type DenominationPubKey = RsaDenominationPubKey | CsDenominationPubKey;
 
 export interface RsaDenominationPubKey {
@@ -1539,6 +1575,15 @@ export const codecForCheckPaymentResponse = (): Codec<CheckPaymentResponse> =>
 export const codecForWithdrawOperationStatusResponse =
   (): Codec<WithdrawOperationStatusResponse> =>
     buildCodecForObject<WithdrawOperationStatusResponse>()
+      .property(
+        "status",
+        codecForEither(
+          codecForConstString("selected"),
+          codecForConstString("confirmed"),
+          codecForConstString("aborted"),
+          codecForConstString("pending"),
+        ),
+      )
       .property("selection_done", codecForBoolean())
       .property("transfer_done", codecForBoolean())
       .property("aborted", codecForBoolean())
@@ -2217,7 +2262,7 @@ export interface ExchangeBatchDepositRequest {
   // Date until which the merchant can issue a refund to the customer via the
   // exchange, to be omitted if refunds are not allowed.
   //
-  // THIS FIELD WILL BE DEPRICATED, once the refund mechanism becomes a
+  // THIS FIELD WILL BE DEPRECATED, once the refund mechanism becomes a
   // policy via extension.
   refund_deadline?: TalerProtocolTimestamp;
 
@@ -2372,7 +2417,7 @@ export interface ExchangeWireAccount {
 
 export const codecForExchangeWireAccount = (): Codec<ExchangeWireAccount> =>
   buildCodecForObject<ExchangeWireAccount>()
-    .property("conversion_url", codecOptional(codecForString()))
+    .property("conversion_url", codecOptional(codecForStringURL()))
     .property("credit_restrictions", codecForList(codecForAny()))
     .property("debit_restrictions", codecForList(codecForAny()))
     .property("master_sig", codecForString())
