@@ -14,11 +14,23 @@
  GNU Taler; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
  */
 
+/**
+ * @fileoverview
+ * Construction and parsing of taler:// URIs.
+ * Specification: https://lsd.gnunet.org/lsd0006/
+ */
+
+/**
+ * Imports.
+ */
 import { Codec, Context, DecodingError, renderContext } from "./codec.js";
 import { canonicalizeBaseUrl } from "./helpers.js";
 import { AmountString } from "./taler-types.js";
-import { URLSearchParams, URL } from "./url.js";
+import { URL, URLSearchParams } from "./url.js";
 
+/**
+ * A parsed taler URI.
+ */
 export type TalerUri =
   | PayUriResult
   | PayTemplateUriResult
@@ -27,11 +39,8 @@ export type TalerUri =
   | PayPushUriResult
   | BackupRestoreUri
   | RefundUriResult
-  | RewardUriResult
   | WithdrawUriResult
-  | ExchangeUri
-  | WithdrawExchangeUri
-  | AuditorUri;
+  | WithdrawExchangeUri;
 
 declare const __action_str: unique symbol;
 export type TalerActionString = string & { [__action_str]: true };
@@ -63,11 +72,16 @@ export interface PayUriResult {
   noncePriv?: string;
 }
 
+export type TemplateParams = {
+  amount?: string;
+  summary?: string;
+};
+
 export interface PayTemplateUriResult {
   type: TalerUriAction.PayTemplate;
   merchantBaseUrl: string;
   templateId: string;
-  templateParams: Record<string, string>;
+  templateParams: TemplateParams;
 }
 
 export interface WithdrawUriResult {
@@ -82,24 +96,6 @@ export interface RefundUriResult {
   orderId: string;
 }
 
-export interface RewardUriResult {
-  type: TalerUriAction.Reward;
-  merchantBaseUrl: string;
-  merchantRewardId: string;
-}
-
-export interface ExchangeUri {
-  type: TalerUriAction.Exchange;
-  exchangeBaseUrl: string;
-  exchangePub: string;
-}
-
-export interface AuditorUri {
-  type: TalerUriAction.Auditor;
-  auditorBaseUrl: string;
-  auditorPub: string;
-}
-
 export interface PayPushUriResult {
   type: TalerUriAction.PayPush;
   exchangeBaseUrl: string;
@@ -126,7 +122,7 @@ export interface BackupRestoreUri {
 export interface WithdrawExchangeUri {
   type: TalerUriAction.WithdrawExchange;
   exchangeBaseUrl: string;
-  exchangePub: string;
+  exchangePub?: string;
   amount?: AmountString;
 }
 
@@ -181,20 +177,13 @@ export enum TalerUriType {
   Unknown = "unknown",
 }
 
-const talerActionPayPull = "pay-pull";
-const talerActionPayPush = "pay-push";
-const talerActionPayTemplate = "pay-template";
-
 export enum TalerUriAction {
   Pay = "pay",
   Withdraw = "withdraw",
   Refund = "refund",
-  Reward = "reward",
   PayPull = "pay-pull",
   PayPush = "pay-push",
   PayTemplate = "pay-template",
-  Exchange = "exchange",
-  Auditor = "auditor",
   Restore = "restore",
   DevExperiment = "dev-experiment",
   WithdrawExchange = "withdraw-exchange",
@@ -234,11 +223,8 @@ const parsers: { [A in TalerUriAction]: Parser } = {
   [TalerUriAction.PayTemplate]: parsePayTemplateUri,
   [TalerUriAction.Restore]: parseRestoreUri,
   [TalerUriAction.Refund]: parseRefundUri,
-  [TalerUriAction.Reward]: parseRewardUri,
   [TalerUriAction.Withdraw]: parseWithdrawUri,
   [TalerUriAction.DevExperiment]: parseDevExperimentUri,
-  [TalerUriAction.Exchange]: parseExchangeUri,
-  [TalerUriAction.Auditor]: parseAuditorUri,
   [TalerUriAction.WithdrawExchange]: parseWithdrawExchangeUri,
 };
 
@@ -277,21 +263,12 @@ export function stringifyTalerUri(uri: TalerUri): string {
     case TalerUriAction.Refund: {
       return stringifyRefundUri(uri);
     }
-    case TalerUriAction.Reward: {
-      return stringifyRewardUri(uri);
-    }
     case TalerUriAction.Withdraw: {
       return stringifyWithdrawUri(uri);
     }
-    case TalerUriAction.Exchange: {
-      return stringifyExchangeUri(uri);
-    }
     case TalerUriAction.WithdrawExchange: {
       return stringifyWithdrawExchange(uri);
     }
-    case TalerUriAction.Auditor: {
-      return stringifyAuditorUri(uri);
-    }
   }
 }
 
@@ -332,7 +309,7 @@ export function parsePayUri(s: string): PayUriResult | undefined {
 export function parsePayTemplateUri(
   uriString: string,
 ): PayTemplateUriResult | undefined {
-  const pi = parseProtoInfo(uriString, talerActionPayTemplate);
+  const pi = parseProtoInfo(uriString, TalerUriAction.PayTemplate);
   if (!pi) {
     return undefined;
   }
@@ -366,7 +343,7 @@ export function parsePayTemplateUri(
 }
 
 export function parsePayPushUri(s: string): PayPushUriResult | undefined {
-  const pi = parseProtoInfo(s, talerActionPayPush);
+  const pi = parseProtoInfo(s, TalerUriAction.PayPush);
   if (!pi) {
     return undefined;
   }
@@ -391,7 +368,7 @@ export function parsePayPushUri(s: string): PayPushUriResult | undefined {
 }
 
 export function parsePayPullUri(s: string): PayPullUriResult | undefined {
-  const pi = parseProtoInfo(s, talerActionPayPull);
+  const pi = parseProtoInfo(s, TalerUriAction.PayPull);
   if (!pi) {
     return undefined;
   }
@@ -415,60 +392,6 @@ export function parsePayPullUri(s: string): PayPullUriResult | undefined {
   };
 }
 
-/**
- * Parse a taler[+http]://reward URI.
- * Return undefined if not passed a valid URI.
- */
-export function parseRewardUri(s: string): RewardUriResult | undefined {
-  const pi = parseProtoInfo(s, "reward");
-  if (!pi) {
-    return undefined;
-  }
-  const c = pi?.rest.split("?");
-  const parts = c[0].split("/");
-  if (parts.length < 2) {
-    return undefined;
-  }
-  const host = parts[0].toLowerCase();
-  const rewardId = parts[parts.length - 1];
-  const pathSegments = parts.slice(1, parts.length - 1);
-  const hostAndSegments = [host, ...pathSegments].join("/");
-  const merchantBaseUrl = canonicalizeBaseUrl(
-    `${pi.innerProto}://${hostAndSegments}/`,
-  );
-
-  return {
-    type: TalerUriAction.Reward,
-    merchantBaseUrl,
-    merchantRewardId: rewardId,
-  };
-}
-
-export function parseExchangeUri(s: string): ExchangeUri | undefined {
-  const pi = parseProtoInfo(s, "exchange");
-  if (!pi) {
-    return undefined;
-  }
-  const c = pi?.rest.split("?");
-  const parts = c[0].split("/");
-  if (parts.length < 2) {
-    return undefined;
-  }
-  const host = parts[0].toLowerCase();
-  const exchangePub = parts[parts.length - 1];
-  const pathSegments = parts.slice(1, parts.length - 1);
-  const hostAndSegments = [host, ...pathSegments].join("/");
-  const exchangeBaseUrl = canonicalizeBaseUrl(
-    `${pi.innerProto}://${hostAndSegments}/`,
-  );
-
-  return {
-    type: TalerUriAction.Exchange,
-    exchangeBaseUrl,
-    exchangePub,
-  };
-}
-
 export function parseWithdrawExchangeUri(
   s: string,
 ): WithdrawExchangeUri | undefined {
@@ -478,11 +401,11 @@ export function parseWithdrawExchangeUri(
   }
   const c = pi?.rest.split("?");
   const parts = c[0].split("/");
-  if (parts.length < 2) {
+  if (parts.length < 1) {
     return undefined;
   }
   const host = parts[0].toLowerCase();
-  const exchangePub = parts[parts.length - 1];
+  const exchangePub = parts.length > 1 ? parts[parts.length - 1] : undefined;
   const pathSegments = parts.slice(1, parts.length - 1);
   const hostAndSegments = [host, ...pathSegments].join("/");
   const exchangeBaseUrl = canonicalizeBaseUrl(
@@ -494,36 +417,11 @@ export function parseWithdrawExchangeUri(
   return {
     type: TalerUriAction.WithdrawExchange,
     exchangeBaseUrl,
-    exchangePub,
+    exchangePub: exchangePub != "" ? exchangePub : undefined,
     amount,
   };
 }
 
-export function parseAuditorUri(s: string): AuditorUri | undefined {
-  const pi = parseProtoInfo(s, "auditor");
-  if (!pi) {
-    return undefined;
-  }
-  const c = pi?.rest.split("?");
-  const parts = c[0].split("/");
-  if (parts.length < 2) {
-    return undefined;
-  }
-  const host = parts[0].toLowerCase();
-  const auditorPub = parts[parts.length - 1];
-  const pathSegments = parts.slice(1, parts.length - 1);
-  const hostAndSegments = [host, ...pathSegments].join("/");
-  const auditorBaseUrl = canonicalizeBaseUrl(
-    `${pi.innerProto}://${hostAndSegments}/`,
-  );
-
-  return {
-    type: TalerUriAction.Auditor,
-    auditorBaseUrl,
-    auditorPub,
-  };
-}
-
 /**
  * Parse a taler[+http]://refund URI.
  * Return undefined if not passed a valid URI.
@@ -598,56 +496,6 @@ export function parseRestoreUri(uri: string): BackupRestoreUri | undefined {
 //  To string functions
 // ================================================
 
-/**
- * @deprecated use stringifyRecoveryUri
- */
-export function constructRecoveryUri(args: {
-  walletRootPriv: string;
-  providers: string[];
-}): string {
-  return stringifyRestoreUri(args);
-}
-
-/**
- * @deprecated stringifyPayPullUri
- */
-export function constructPayPullUri(args: {
-  exchangeBaseUrl: string;
-  contractPriv: string;
-}): string {
-  return stringifyPayPullUri(args);
-}
-
-/**
- * @deprecated use stringifyPayPushUri
- */
-export function constructPayPushUri(args: {
-  exchangeBaseUrl: string;
-  contractPriv: string;
-}): string {
-  return stringifyPayPushUri(args);
-}
-
-/**
- *
- * @deprecated use stringifyPayUri
- */
-export function constructPayUri(
-  merchantBaseUrl: string,
-  orderId: string,
-  sessionId: string,
-  claimToken?: string,
-  noncePriv?: string,
-): string {
-  return stringifyPayUri({
-    merchantBaseUrl,
-    orderId,
-    sessionId,
-    claimToken,
-    noncePriv,
-  });
-}
-
 export function stringifyPayUri({
   merchantBaseUrl,
   orderId,
@@ -697,7 +545,7 @@ export function stringifyWithdrawExchange({
   const { proto, path, query } = getUrlInfo(exchangeBaseUrl, {
     a: amount,
   });
-  return `${proto}://withdraw-exchange/${path}${exchangePub}${query}`;
+  return `${proto}://withdraw-exchange/${path}${exchangePub ?? ""}${query}`;
 }
 
 export function stringifyDevExperimentUri({
@@ -714,35 +562,13 @@ export function stringifyPayTemplateUri({
   const { proto, path, query } = getUrlInfo(merchantBaseUrl, templateParams);
   return `${proto}://pay-template/${path}${templateId}${query}`;
 }
+
 export function stringifyRefundUri({
   merchantBaseUrl,
   orderId,
 }: Omit<RefundUriResult, "type">): string {
   const { proto, path } = getUrlInfo(merchantBaseUrl);
-  return `${proto}://refund/${path}${orderId}`;
-}
-export function stringifyRewardUri({
-  merchantBaseUrl,
-  merchantRewardId,
-}: Omit<RewardUriResult, "type">): string {
-  const { proto, path } = getUrlInfo(merchantBaseUrl);
-  return `${proto}://reward/${path}${merchantRewardId}`;
-}
-
-export function stringifyExchangeUri({
-  exchangeBaseUrl,
-  exchangePub,
-}: Omit<ExchangeUri, "type">): string {
-  const { proto, path } = getUrlInfo(exchangeBaseUrl);
-  return `${proto}://exchange/${path}${exchangePub}`;
-}
-
-export function stringifyAuditorUri({
-  auditorBaseUrl,
-  auditorPub,
-}: Omit<AuditorUri, "type">): string {
-  const { proto, path } = getUrlInfo(auditorBaseUrl);
-  return `${proto}://auditor/${path}${auditorPub}`;
+  return `${proto}://refund/${path}${orderId}/`;
 }
 
 export function stringifyWithdrawUri({
@@ -757,10 +583,6 @@ export function stringifyWithdrawUri({
  * Use baseUrl to defined http or https
  * create path using host+port+pathname
  * use params to create a query parameter string or empty
- *
- * @param baseUrl
- * @param params
- * @returns
  */
 function getUrlInfo(
   baseUrl: string,
