@@ -24,42 +24,50 @@
 /**
  * Imports.
  */
-import { TalerPreciseTimestamp, TalerProtocolTimestamp } from "./time.js";
+import {
+  Codec,
+  buildCodecForObject,
+  codecForAny,
+  codecForBoolean,
+  codecForConstString,
+  codecForEither,
+  codecForList,
+  codecForString,
+  codecOptional,
+} from "./codec.js";
 import {
   AmountString,
-  Product,
   InternationalizedString,
   MerchantInfo,
   codecForInternationalizedString,
   codecForMerchantInfo,
-  codecForProduct,
-  Location,
 } from "./taler-types.js";
-import {
-  Codec,
-  buildCodecForObject,
-  codecOptional,
-  codecForString,
-  codecForList,
-  codecForAny,
-  codecForBoolean,
-  codecForEither,
-  codecForConstString,
-} from "./codec.js";
+import { TalerPreciseTimestamp, TalerProtocolTimestamp } from "./time.js";
 import {
   RefreshReason,
+  ScopeInfo,
   TalerErrorDetail,
   TransactionIdStr,
   TransactionStateFilter,
   WithdrawalExchangeAccountDetails,
+  codecForScopeInfo,
 } from "./wallet-types.js";
 
 export interface TransactionsRequest {
   /**
    * return only transactions in the given currency
+   *
+   * it will be removed in next release
+   *
+   * @deprecated use scopeInfo
    */
   currency?: string;
 
+  /**
+   * return only transactions in the given scopeInfo
+   */
+  scopeInfo?: ScopeInfo;
+
   /**
    * if present, results will be limited to transactions related to the given search string
    */
@@ -121,6 +129,7 @@ export enum TransactionMinorState {
   CheckRefund = "check-refund",
   CreatePurse = "create-purse",
   DeletePurse = "delete-purse",
+  RefreshExpired = "refresh-expired",
   Ready = "ready",
   Merge = "merge",
   Repurchase = "repurchase",
@@ -136,6 +145,7 @@ export enum TransactionMinorState {
   Proposed = "proposed",
   RefundAvailable = "refund-available",
   AcceptRefund = "accept-refund",
+  PaidByOther = "paid-by-other",
 }
 
 export enum TransactionAction {
@@ -206,7 +216,8 @@ export type Transaction =
   | TransactionPeerPullDebit
   | TransactionPeerPushCredit
   | TransactionPeerPushDebit
-  | TransactionInternalWithdrawal;
+  | TransactionInternalWithdrawal
+  | TransactionRecoup;
 
 export enum TransactionType {
   Withdrawal = "withdrawal",
@@ -220,6 +231,7 @@ export enum TransactionType {
   PeerPushCredit = "peer-push-credit",
   PeerPullDebit = "peer-pull-debit",
   PeerPullCredit = "peer-pull-credit",
+  Recoup = "recoup",
 }
 
 export enum WithdrawalType {
@@ -277,6 +289,8 @@ interface WithdrawalDetailsForTalerBankIntegrationApi {
    * Is the reserve ready for withdrawal?
    */
   reserveIsReady: boolean;
+
+  exchangeCreditAccountDetails?: WithdrawalExchangeAccountDetails[];
 }
 
 /**
@@ -366,8 +380,10 @@ export interface TransactionPeerPullCredit extends TransactionCommon {
 
   /**
    * URI to send to the other party.
+   *
+   * Only available in the right state.
    */
-  talerUri: string;
+  talerUri: string | undefined;
 }
 
 /**
@@ -441,6 +457,13 @@ export interface TransactionPeerPushCredit extends TransactionCommon {
   amountEffective: AmountString;
 }
 
+/**
+ * The exchange revoked a key and the wallet recoups funds.
+ */
+export interface TransactionRecoup extends TransactionCommon {
+  type: TransactionType.Recoup;
+}
+
 export enum PaymentStatus {
   /**
    * Explicitly aborted after timeout / failure
@@ -703,9 +726,20 @@ export const codecForTransactionByIdRequest =
       .property("transactionId", codecForString())
       .build("TransactionByIdRequest");
 
+export interface WithdrawalTransactionByURIRequest {
+  talerWithdrawUri: string;
+}
+
+export const codecForWithdrawalTransactionByURIRequest =
+  (): Codec<WithdrawalTransactionByURIRequest> =>
+    buildCodecForObject<WithdrawalTransactionByURIRequest>()
+      .property("talerWithdrawUri", codecForString())
+      .build("WithdrawalTransactionByURIRequest");
+
 export const codecForTransactionsRequest = (): Codec<TransactionsRequest> =>
   buildCodecForObject<TransactionsRequest>()
     .property("currency", codecOptional(codecForString()))
+    .property("scopeInfo", codecOptional(codecForScopeInfo()))
     .property("search", codecOptional(codecForString()))
     .property(
       "sort",
@@ -739,3 +773,17 @@ export const codecForOrderShortInfo = (): Codec<OrderShortInfo> =>
     .property("summary", codecForString())
     .property("summary_i18n", codecOptional(codecForInternationalizedString()))
     .build("OrderShortInfo");
+
+export interface ListAssociatedRefreshesRequest {
+  transactionId: string;
+}
+
+export const codecForListAssociatedRefreshesRequest =
+  (): Codec<ListAssociatedRefreshesRequest> =>
+    buildCodecForObject<ListAssociatedRefreshesRequest>()
+      .property("transactionId", codecForString())
+      .build("ListAssociatedRefreshesRequest");
+
+export interface ListAssociatedRefreshesResponse {
+  transactionIds: string[];
+}
