@@ -46,7 +46,11 @@ import {
   codecOptional,
   renderContext,
 } from "./codec.js";
-import { CurrencySpecification } from "./index.js";
+import {
+  CurrencySpecification,
+  TemplateParams,
+  WithdrawalOperationStatus,
+} from "./index.js";
 import { VersionMatchResult } from "./libtool-version.js";
 import { PaytoUri } from "./payto.js";
 import { AgeCommitmentProof } from "./taler-crypto.js";
@@ -75,6 +79,7 @@ import {
   TalerProtocolDuration,
   TalerProtocolTimestamp,
   codecForAbsoluteTime,
+  codecForPreciseTimestamp,
   codecForTimestamp,
 } from "./time.js";
 import {
@@ -286,7 +291,6 @@ interface GetPlanForPaymentRequest extends GetPlanToCompleteOperation {
   wireMethod: string;
   ageRestriction: number;
   maxDepositFee: AmountString;
-  maxWireFee: AmountString;
 }
 
 // interface GetPlanForTipRequest extends GetPlanForOperationBase {
@@ -306,7 +310,6 @@ const codecForGetPlanForPaymentRequest =
   buildCodecForObject<GetPlanForPaymentRequest>()
     .property("type", codecForConstString(TransactionType.Payment))
     .property("maxDepositFee", codecForAmountString())
-    .property("maxWireFee", codecForAmountString())
     .build("GetPlanForPaymentRequest");
 
 const codecForGetPlanForPullDebitRequest =
@@ -457,10 +460,60 @@ export interface GetCurrencySpecificationResponse {
   currencySpecification: CurrencySpecification;
 }
 
-export interface InitRequest {
-  skipDefaults?: boolean;
+export interface BuiltinExchange {
+  exchangeBaseUrl: string;
+  currencyHint?: string;
 }
 
+export interface PartialWalletRunConfig {
+  builtin?: Partial<WalletRunConfig["builtin"]>;
+  testing?: Partial<WalletRunConfig["testing"]>;
+  features?: Partial<WalletRunConfig["features"]>;
+}
+
+export interface WalletRunConfig {
+  /**
+   * Initialization values useful for a complete startup.
+   *
+   * These are values may be overridden by different wallets
+   */
+  builtin: {
+    exchanges: BuiltinExchange[];
+  };
+
+  /**
+   * Unsafe options which it should only be used to create
+   * testing environment.
+   */
+  testing: {
+    /**
+     * Allow withdrawal of denominations even though they are about to expire.
+     */
+    denomselAllowLate: boolean;
+    devModeActive: boolean;
+    insecureTrustExchange: boolean;
+    preventThrottling: boolean;
+    skipDefaults: boolean;
+    emitObservabilityEvents?: boolean;
+  };
+
+  /**
+   * Configurations values that may be safe to show to the user
+   */
+  features: {
+    allowHttp: boolean;
+  };
+}
+
+export interface InitRequest {
+  config?: PartialWalletRunConfig;
+}
+
+export const codecForInitRequest = (): Codec<InitRequest> =>
+  buildCodecForObject<InitRequest>()
+    .property("config", codecForAny())
+    .build("InitRequest");
+
 export interface InitResponse {
   versionInfo: WalletCoreVersion;
 }
@@ -698,7 +751,7 @@ export interface PrepareTipResult {
   /**
    * Tip transaction ID.
    */
-  transactionId: string;
+  transactionId: TransactionIdStr;
 
   /**
    * Has the tip already been accepted?
@@ -748,7 +801,7 @@ export const codecForPrepareTipResult = (): Codec<PrepareTipResult> =>
     .property("merchantBaseUrl", codecForString())
     .property("expirationTimestamp", codecForTimestamp)
     .property("walletRewardId", codecForString())
-    .property("transactionId", codecForString())
+    .property("transactionId", codecForTransactionIdStr())
     .build("PrepareRewardResult");
 
 export interface BenchmarkResult {
@@ -778,10 +831,12 @@ export const codecForPreparePayResultPaymentPossible =
       )
       .build("PreparePayResultPaymentPossible");
 
+export interface BalanceDetails {}
+
 /**
  * Detailed reason for why the wallet's balance is insufficient.
  */
-export interface PayMerchantInsufficientBalanceDetails {
+export interface PaymentInsufficientBalanceDetails {
   /**
    * Amount requested by the merchant.
    */
@@ -805,35 +860,46 @@ export interface PayMerchantInsufficientBalanceDetails {
   /**
    * Balance of type "merchant-acceptable" (see balance.ts for definition).
    */
-  balanceMerchantAcceptable: AmountString;
+  balanceReceiverAcceptable: AmountString;
 
   /**
    * Balance of type "merchant-depositable" (see balance.ts for definition).
    */
-  balanceMerchantDepositable: AmountString;
+  balanceReceiverDepositable: AmountString;
+
+  balanceExchangeDepositable: AmountString;
 
   /**
-   * If the payment would succeed without fees
-   * (i.e. balanceMerchantDepositable >= amountRequested),
-   * this field contains an estimate of the amount that would additionally
-   * be required to cover the fees.
-   *
-   * It is not possible to give an exact value here, since it depends
-   * on the coin selection for the amount that would be additionally withdrawn.
+   * Maximum effective amount that the wallet can spend,
+   * when all fees are paid by the wallet.
    */
-  feeGapEstimate: AmountString;
+  maxEffectiveSpendAmount: AmountString;
+
+  perExchange: {
+    [url: string]: {
+      balanceAvailable: AmountString;
+      balanceMaterial: AmountString;
+      balanceExchangeDepositable: AmountString;
+      balanceAgeAcceptable: AmountString;
+      balanceReceiverAcceptable: AmountString;
+      balanceReceiverDepositable: AmountString;
+      maxEffectiveSpendAmount: AmountString;
+    };
+  };
 }
 
 export const codecForPayMerchantInsufficientBalanceDetails =
-  (): Codec<PayMerchantInsufficientBalanceDetails> =>
-    buildCodecForObject<PayMerchantInsufficientBalanceDetails>()
+  (): Codec<PaymentInsufficientBalanceDetails> =>
+    buildCodecForObject<PaymentInsufficientBalanceDetails>()
       .property("amountRequested", codecForAmountString())
       .property("balanceAgeAcceptable", codecForAmountString())
       .property("balanceAvailable", codecForAmountString())
       .property("balanceMaterial", codecForAmountString())
-      .property("balanceMerchantAcceptable", codecForAmountString())
-      .property("balanceMerchantDepositable", codecForAmountString())
-      .property("feeGapEstimate", codecForAmountString())
+      .property("balanceReceiverAcceptable", codecForAmountString())
+      .property("balanceReceiverDepositable", codecForAmountString())
+      .property("balanceExchangeDepositable", codecForAmountString())
+      .property("perExchange", codecForAny())
+      .property("maxEffectiveSpendAmount", codecForAmountString())
       .build("PayMerchantInsufficientBalanceDetails");
 
 export const codecForPreparePayResultInsufficientBalance =
@@ -923,7 +989,7 @@ export interface PreparePayResultInsufficientBalance {
   contractTerms: MerchantContractTerms;
   amountRaw: AmountString;
   talerUri: string;
-  balanceDetails: PayMerchantInsufficientBalanceDetails;
+  balanceDetails: PaymentInsufficientBalanceDetails;
 }
 
 export interface PreparePayResultAlreadyConfirmed {
@@ -942,13 +1008,14 @@ export interface PreparePayResultAlreadyConfirmed {
 }
 
 export interface BankWithdrawDetails {
-  selectionDone: boolean;
-  transferDone: boolean;
+  status: WithdrawalOperationStatus;
   amount: AmountJson;
   senderWire?: string;
   suggestedExchange?: string;
   confirmTransferUrl?: string;
   wireTypes: string[];
+  operationId: string;
+  apiBaseUrl: string;
 }
 
 export interface AcceptWithdrawalResponse {
@@ -1030,6 +1097,7 @@ export enum RefreshReason {
   AbortPay = "abort-pay",
   AbortDeposit = "abort-deposit",
   AbortPeerPushDebit = "abort-peer-push-debit",
+  AbortPeerPullDebit = "abort-peer-pull-debit",
   Recoup = "recoup",
   BackupRestored = "backup-restored",
   Scheduled = "scheduled",
@@ -1043,13 +1111,6 @@ export interface CoinRefreshRequest {
   readonly amount: AmountString;
 }
 
-/**
- * Wrapper for refresh group IDs.
- */
-export interface RefreshGroupId {
-  readonly refreshGroupId: string;
-}
-
 /**
  * Private data required to make a deposit permission.
  */
@@ -1086,14 +1147,31 @@ export interface ExchangeDetailedResponse {
 }
 
 export interface WalletCoreVersion {
+  implementationSemver: string;
+  implementationGitHash: string;
+
+  /**
+   * Wallet-core protocol version supported by this implementation
+   * of the API ("server" version).
+   */
+  version: string;
+  exchange: string;
+  merchant: string;
+
+  bankIntegrationApiRange: string;
+  bankConversionApiRange: string;
+  corebankApiRange: string;
+
+  /**
+   * @deprecated as bank was split into multiple APIs with separate versioning
+   */
+  bank: string;
+
   /**
    * @deprecated
    */
   hash: string | undefined;
-  version: string;
-  exchange: string;
-  merchant: string;
-  bank: string;
+
   /**
    * @deprecated will be removed
    */
@@ -1288,12 +1366,11 @@ export enum ExchangeEntryStatus {
 
 export enum ExchangeUpdateStatus {
   Initial = "initial",
-  InitialUpdate = "initial(update)",
+  InitialUpdate = "initial-update",
   Suspended = "suspended",
-  Failed = "failed",
-  OutdatedUpdate = "outdated(update)",
+  UnavailableUpdate = "unavailable-update",
   Ready = "ready",
-  ReadyUpdate = "ready(update)",
+  ReadyUpdate = "ready-update",
 }
 
 export interface OperationErrorInfo {
@@ -1304,9 +1381,12 @@ export interface ShortExchangeListItem {
   exchangeBaseUrl: string;
 }
 
-// FIXME: This should probably include some error status.
+/**
+ * Info about an exchange entry in the wallet.
+ */
 export interface ExchangeListItem {
   exchangeBaseUrl: string;
+  masterPub: string | undefined;
   currency: string | undefined;
   paytoUris: string[];
   tosStatus: ExchangeTosStatus;
@@ -1314,6 +1394,10 @@ export interface ExchangeListItem {
   exchangeUpdateStatus: ExchangeUpdateStatus;
   ageRestrictionOptions: number[];
 
+  scopeInfo: ScopeInfo | undefined;
+
+  lastUpdateTimestamp: TalerPreciseTimestamp | undefined;
+
   /**
    * Information about the last error that occurred when trying
    * to update the exchange info.
@@ -1378,13 +1462,17 @@ export const codecForExchangeFullDetails = (): Codec<ExchangeFullDetails> =>
 
 export const codecForExchangeListItem = (): Codec<ExchangeListItem> =>
   buildCodecForObject<ExchangeListItem>()
-    .property("currency", codecForString())
+    .property("currency", codecOptional(codecForString()))
     .property("exchangeBaseUrl", codecForString())
+    .property("masterPub", codecOptional(codecForString()))
     .property("paytoUris", codecForList(codecForString()))
     .property("tosStatus", codecForAny())
     .property("exchangeEntryStatus", codecForAny())
     .property("exchangeUpdateStatus", codecForAny())
     .property("ageRestrictionOptions", codecForList(codecForNumber()))
+    .property("scopeInfo", codecForScopeInfo())
+    .property("lastUpdateErrorInfo", codecForAny())
+    .property("lastUpdateTimestamp", codecOptional(codecForPreciseTimestamp))
     .build("ExchangeListItem");
 
 export const codecForExchangesListResponse = (): Codec<ExchangesListResponse> =>
@@ -1410,10 +1498,12 @@ export interface AcceptManualWithdrawalResult {
   transactionId: TransactionIdStr;
 }
 
-export interface ManualWithdrawalDetails {
+export interface WithdrawalDetailsForAmount {
   /**
    * Did the user accept the current version of the exchange's
    * terms of service?
+   *
+   * @deprecated the client should query the exchange entry instead
    */
   tosAccepted: boolean;
 
@@ -1437,14 +1527,14 @@ export interface ManualWithdrawalDetails {
   /**
    * Ways to pay the exchange.
    *
-   * @deprecated in favor of withdrawalAccountList
+   * @deprecated in favor of withdrawalAccountsList
    */
   paytoUris: string[];
 
   /**
    * Ways to pay the exchange, including accounts that require currency conversion.
    */
-  withdrawalAccountList: WithdrawalExchangeAccountDetails[];
+  withdrawalAccountsList: WithdrawalExchangeAccountDetails[];
 
   /**
    * If the exchange supports age-restricted coins it will return
@@ -1545,6 +1635,8 @@ export interface ExchangeWithdrawalDetails {
    *
    */
   ageRestrictionOptions?: number[];
+
+  scopeInfo: ScopeInfo;
 }
 
 export interface GetExchangeTosResult {
@@ -1569,6 +1661,18 @@ export interface GetExchangeTosResult {
    */
   contentType: string;
 
+  /**
+   * Language of the returned content.
+   *
+   * If missing, language is unknown.
+   */
+  contentLanguage: string | undefined;
+
+  /**
+   * Available languages as advertised by the exchange.
+   */
+  tosAvailableLanguages: string[];
+
   tosStatus: ExchangeTosStatus;
 }
 
@@ -1623,10 +1727,27 @@ export const codecForIntegrationTestV2Args = (): Codec<IntegrationTestV2Args> =>
     .property("corebankApiBaseUrl", codecForString())
     .build("IntegrationTestV2Args");
 
+export interface GetExchangeEntryByUrlRequest {
+  exchangeBaseUrl: string;
+}
+
+export const codecForGetExchangeEntryByUrlRequest =
+  (): Codec<GetExchangeEntryByUrlRequest> =>
+    buildCodecForObject<GetExchangeEntryByUrlRequest>()
+      .property("exchangeBaseUrl", codecForString())
+      .build("GetExchangeEntryByUrlRequest");
+
+export type GetExchangeEntryByUrlResponse = ExchangeListItem;
+
 export interface AddExchangeRequest {
   exchangeBaseUrl: string;
-  masterPub?: string;
+
+  /**
+   * @deprecated use a separate API call to start a forced exchange update instead
+   */
   forceUpdate?: boolean;
+
+  masterPub?: string;
 }
 
 export const codecForAddExchangeRequest = (): Codec<AddExchangeRequest> =>
@@ -1638,14 +1759,41 @@ export const codecForAddExchangeRequest = (): Codec<AddExchangeRequest> =>
 
 export interface UpdateExchangeEntryRequest {
   exchangeBaseUrl: string;
+  force?: boolean;
 }
 
 export const codecForUpdateExchangeEntryRequest =
   (): Codec<UpdateExchangeEntryRequest> =>
     buildCodecForObject<UpdateExchangeEntryRequest>()
       .property("exchangeBaseUrl", codecForString())
+      .property("force", codecOptional(codecForBoolean()))
       .build("UpdateExchangeEntryRequest");
 
+export interface GetExchangeResourcesRequest {
+  exchangeBaseUrl: string;
+}
+
+export const codecForGetExchangeResourcesRequest =
+  (): Codec<GetExchangeResourcesRequest> =>
+    buildCodecForObject<GetExchangeResourcesRequest>()
+      .property("exchangeBaseUrl", codecForString())
+      .build("GetExchangeResourcesRequest");
+
+export interface GetExchangeResourcesResponse {
+  hasResources: boolean;
+}
+
+export interface DeleteExchangeRequest {
+  exchangeBaseUrl: string;
+  purge?: boolean;
+}
+
+export const codecForDeleteExchangeRequest = (): Codec<DeleteExchangeRequest> =>
+  buildCodecForObject<DeleteExchangeRequest>()
+    .property("exchangeBaseUrl", codecForString())
+    .property("purge", codecOptional(codecForBoolean()))
+    .build("DeleteExchangeRequest");
+
 export interface ForceExchangeUpdateRequest {
   exchangeBaseUrl: string;
 }
@@ -1659,12 +1807,14 @@ export const codecForForceExchangeUpdateRequest =
 export interface GetExchangeTosRequest {
   exchangeBaseUrl: string;
   acceptedFormat?: string[];
+  acceptLanguage?: string;
 }
 
 export const codecForGetExchangeTosRequest = (): Codec<GetExchangeTosRequest> =>
   buildCodecForObject<GetExchangeTosRequest>()
     .property("exchangeBaseUrl", codecForString())
     .property("acceptedFormat", codecOptional(codecForList(codecForString())))
+    .property("acceptLanguage", codecOptional(codecForString()))
     .build("GetExchangeTosRequest");
 
 export interface AcceptManualWithdrawalRequest {
@@ -1673,7 +1823,7 @@ export interface AcceptManualWithdrawalRequest {
   restrictAge?: number;
 }
 
-export const codecForAcceptManualWithdrawalRequet =
+export const codecForAcceptManualWithdrawalRequest =
   (): Codec<AcceptManualWithdrawalRequest> =>
     buildCodecForObject<AcceptManualWithdrawalRequest>()
       .property("exchangeBaseUrl", codecForString())
@@ -1713,16 +1863,24 @@ export const codecForGetWithdrawalDetailsForAmountRequest =
 
 export interface AcceptExchangeTosRequest {
   exchangeBaseUrl: string;
-  etag: string | undefined;
 }
 
 export const codecForAcceptExchangeTosRequest =
   (): Codec<AcceptExchangeTosRequest> =>
     buildCodecForObject<AcceptExchangeTosRequest>()
       .property("exchangeBaseUrl", codecForString())
-      .property("etag", codecOptional(codecForString()))
       .build("AcceptExchangeTosRequest");
 
+export interface ForgetExchangeTosRequest {
+  exchangeBaseUrl: string;
+}
+
+export const codecForForgetExchangeTosRequest =
+  (): Codec<ForgetExchangeTosRequest> =>
+    buildCodecForObject<ForgetExchangeTosRequest>()
+      .property("exchangeBaseUrl", codecForString())
+      .build("ForgetExchangeTosRequest");
+
 export interface AcceptRefundRequest {
   transactionId: TransactionIdStr;
 }
@@ -1745,6 +1903,7 @@ export const codecForApplyRefundFromPurchaseIdRequest =
 export interface GetWithdrawalDetailsForUriRequest {
   talerWithdrawUri: string;
   restrictAge?: number;
+  notifyChangeFromPendingTimeoutMs?: number;
 }
 
 export const codecForGetWithdrawalDetailsForUri =
@@ -1752,6 +1911,10 @@ export const codecForGetWithdrawalDetailsForUri =
     buildCodecForObject<GetWithdrawalDetailsForUriRequest>()
       .property("talerWithdrawUri", codecForString())
       .property("restrictAge", codecOptional(codecForNumber()))
+      .property(
+        "notifyChangeFromPendingTimeoutMs",
+        codecOptional(codecForNumber()),
+      )
       .build("GetWithdrawalDetailsForUriRequest");
 
 export interface ListKnownBankAccountsRequest {
@@ -1835,7 +1998,7 @@ export const codecForSharePaymentResult = (): Codec<SharePaymentResult> =>
 
 export interface PreparePayTemplateRequest {
   talerPayTemplateUri: string;
-  templateParams: Record<string, string>;
+  templateParams?: TemplateParams;
 }
 
 export const codecForPreparePayTemplateRequest =
@@ -1850,7 +2013,7 @@ export interface ConfirmPayRequest {
    * @deprecated use transactionId instead
    */
   proposalId?: string;
-  transactionId?: string;
+  transactionId?: TransactionIdStr;
   sessionId?: string;
   forcedCoinSel?: ForcedCoinSel;
 }
@@ -1858,7 +2021,7 @@ export interface ConfirmPayRequest {
 export const codecForConfirmPayRequest = (): Codec<ConfirmPayRequest> =>
   buildCodecForObject<ConfirmPayRequest>()
     .property("proposalId", codecOptional(codecForString()))
-    .property("transactionId", codecOptional(codecForString()))
+    .property("transactionId", codecOptional(codecForTransactionIdStr()))
     .property("sessionId", codecOptional(codecForString()))
     .property("forcedCoinSel", codecForAny())
     .build("ConfirmPay");
@@ -2001,6 +2164,9 @@ export interface PrepareRefundRequest {
 }
 
 export interface StartRefundQueryForUriResponse {
+  /**
+   * Transaction id of the *payment* where the refund query was started.
+   */
   transactionId: TransactionIdStr;
 }
 
@@ -2029,12 +2195,20 @@ export const codecForPrepareRewardRequest = (): Codec<PrepareRewardRequest> =>
     .build("PrepareRewardRequest");
 
 export interface AcceptRewardRequest {
-  walletRewardId: string;
+  /**
+   * @deprecated use transactionId
+   */
+  walletRewardId?: string;
+  /**
+   * it will be required when "walletRewardId" is removed
+   */
+  transactionId?: TransactionIdStr;
 }
 
 export const codecForAcceptTipRequest = (): Codec<AcceptRewardRequest> =>
   buildCodecForObject<AcceptRewardRequest>()
-    .property("walletRewardId", codecForString())
+    .property("walletRewardId", codecOptional(codecForString()))
+    .property("transactionId", codecOptional(codecForTransactionIdStr()))
     .build("AcceptRewardRequest");
 
 export interface FailTransactionRequest {
@@ -2092,7 +2266,7 @@ export interface CreateDepositGroupRequest {
    * that occur while the operation has been created but
    * before the creation request has returned.
    */
-  transactionId?: string;
+  transactionId?: TransactionIdStr;
   depositPaytoUri: string;
   amount: AmountString;
 }
@@ -2118,7 +2292,7 @@ export const codecForCreateDepositGroupRequest =
     buildCodecForObject<CreateDepositGroupRequest>()
       .property("amount", codecForAmountString())
       .property("depositPaytoUri", codecForString())
-      .property("transactionId", codecOptional(codecForString()))
+      .property("transactionId", codecOptional(codecForTransactionIdStr()))
       .build("CreateDepositGroupRequest");
 
 export interface CreateDepositGroupResponse {
@@ -2131,6 +2305,9 @@ export interface TxIdResponse {
 }
 
 export interface WithdrawUriInfoResponse {
+  operationId: string;
+  status: WithdrawalOperationStatus;
+  confirmTransferUrl?: string;
   amount: AmountString;
   defaultExchangeBaseUrl?: string;
   possibleExchanges: ExchangeListItem[];
@@ -2139,6 +2316,17 @@ export interface WithdrawUriInfoResponse {
 export const codecForWithdrawUriInfoResponse =
   (): Codec<WithdrawUriInfoResponse> =>
     buildCodecForObject<WithdrawUriInfoResponse>()
+      .property("operationId", codecForString())
+      .property("confirmTransferUrl", codecOptional(codecForString()))
+      .property(
+        "status",
+        codecForEither(
+          codecForConstString("pending"),
+          codecForConstString("selected"),
+          codecForConstString("aborted"),
+          codecForConstString("confirmed"),
+        ),
+      )
       .property("amount", codecForAmountString())
       .property("defaultExchangeBaseUrl", codecOptional(codecForString()))
       .property("possibleExchanges", codecForList(codecForExchangeListItem()))
@@ -2157,6 +2345,20 @@ export interface WalletCurrencyInfo {
   }[];
 }
 
+export interface TestingListTasksForTransactionRequest {
+  transactionId: TransactionIdStr;
+}
+
+export interface TestingListTasksForTransactionsResponse {
+  taskIdList: string[];
+}
+
+export const codecForTestingListTasksForTransactionRequest =
+  (): Codec<TestingListTasksForTransactionRequest> =>
+    buildCodecForObject<TestingListTasksForTransactionRequest>()
+      .property("transactionId", codecForTransactionIdStr())
+      .build("TestingListTasksForTransactionRequest");
+
 export interface DeleteTransactionRequest {
   transactionId: TransactionIdStr;
 }
@@ -2368,12 +2570,40 @@ export const codecForWithdrawFakebankRequest =
       .property("exchange", codecForString())
       .build("WithdrawFakebankRequest");
 
-export interface ImportDb {
+export interface ActiveTask {
+  id: string;
+  transaction: TransactionIdStr | undefined;
+  firstTry: AbsoluteTime | undefined;
+  nextTry: AbsoluteTime | undefined;
+  counter: number | undefined;
+  lastError: TalerErrorDetail | undefined;
+}
+
+export interface GetActiveTasks {
+  tasks: ActiveTask[];
+}
+
+export const codecForActiveTask = (): Codec<ActiveTask> =>
+  buildCodecForObject<ActiveTask>()
+    .property("id", codecForString())
+    .property("transaction", codecOptional(codecForTransactionIdStr()))
+    .property("counter", codecForNumber())
+    .property("firstTry", codecForAbsoluteTime)
+    .property("nextTry", codecForAbsoluteTime)
+    .property("lastError", codecForTalerErrorDetail())
+    .build("ActiveTask");
+
+export const codecForGetActiveTasks = (): Codec<GetActiveTasks> =>
+  buildCodecForObject<GetActiveTasks>()
+    .property("tasks", codecForList(codecForActiveTask()))
+    .build("GetActiveTasks");
+
+export interface ImportDbRequest {
   dump: any;
 }
 
-export const codecForImportDbRequest = (): Codec<ImportDb> =>
-  buildCodecForObject<ImportDb>()
+export const codecForImportDbRequest = (): Codec<ImportDbRequest> =>
+  buildCodecForObject<ImportDbRequest>()
     .property("dump", codecForAny())
     .build("ImportDbRequest");
 
@@ -2395,7 +2625,15 @@ export interface ForcedCoinSel {
 }
 
 export interface TestPayResult {
-  payCoinSelection: PayCoinSelection;
+  /**
+   * Number of coins used for the payment.
+   */
+  numCoins: number;
+}
+
+export interface SelectedCoin {
+  coinPub: string;
+  contribution: AmountString;
 }
 
 /**
@@ -2403,20 +2641,7 @@ export interface TestPayResult {
  * coins with their denomination.
  */
 export interface PayCoinSelection {
-  /**
-   * Amount requested by the merchant.
-   */
-  paymentAmount: AmountString;
-
-  /**
-   * Public keys of the coins that were selected.
-   */
-  coinPubs: string[];
-
-  /**
-   * Amount that each coin contributes.
-   */
-  coinContributions: AmountString[];
+  coins: SelectedCoin[];
 
   /**
    * How much of the wire fees is the customer paying?
@@ -2453,6 +2678,17 @@ export const codecForCheckPeerPushDebitRequest =
 export interface CheckPeerPushDebitResponse {
   amountRaw: AmountString;
   amountEffective: AmountString;
+  exchangeBaseUrl: string;
+  /**
+   * Maximum expiration date, based on how close the coins
+   * used for the payment are to expiry.
+   *
+   * The value is based on when the wallet would typically
+   * automatically refresh the coins on its own, leaving enough
+   * time to get a refund for the push payment and refresh the
+   * coin.
+   */
+  maxExpirationDate: TalerProtocolTimestamp;
 }
 
 export interface InitiatePeerPushDebitRequest {
@@ -2484,15 +2720,22 @@ export interface PreparePeerPullDebitRequest {
 
 export interface PreparePeerPushCreditResponse {
   contractTerms: PeerContractTerms;
+  amountRaw: AmountString;
+  amountEffective: AmountString;
+
+  transactionId: TransactionIdStr;
+
+  exchangeBaseUrl: string;
+
+  /**
+   * @deprecated use transaction ID instead.
+   */
+  peerPushCreditId: string;
+
   /**
    * @deprecated
    */
   amount: AmountString;
-  amountRaw: AmountString;
-  amountEffective: AmountString;
-  peerPushCreditId: string;
-
-  transactionId: string;
 }
 
 export interface PreparePeerPullDebitResponse {
@@ -2507,7 +2750,7 @@ export interface PreparePeerPullDebitResponse {
 
   peerPullDebitId: string;
 
-  transactionId: string;
+  transactionId: TransactionIdStr;
 }
 
 export const codecForPreparePeerPushCreditRequest =
@@ -2555,7 +2798,7 @@ export interface ConfirmPeerPullDebitRequest {
    */
   peerPullDebitId?: string;
 
-  transactionId?: string;
+  transactionId?: TransactionIdStr;
 }
 
 export interface ApplyDevExperimentRequest {
@@ -2572,7 +2815,7 @@ export const codecForAcceptPeerPullPaymentRequest =
   (): Codec<ConfirmPeerPullDebitRequest> =>
     buildCodecForObject<ConfirmPeerPullDebitRequest>()
       .property("peerPullDebitId", codecOptional(codecForString()))
-      .property("transactionId", codecOptional(codecForString()))
+      .property("transactionId", codecOptional(codecForTransactionIdStr()))
       .build("ConfirmPeerPullDebitRequest");
 
 export interface CheckPeerPullCreditRequest {
@@ -2621,40 +2864,6 @@ export interface InitiatePeerPullCreditResponse {
   transactionId: TransactionIdStr;
 }
 
-/**
- * Detailed reason for why the wallet's balance is insufficient.
- */
-export interface PayPeerInsufficientBalanceDetails {
-  /**
-   * Amount requested by the merchant.
-   */
-  amountRequested: AmountString;
-
-  /**
-   * Balance of type "available" (see balance.ts for definition).
-   */
-  balanceAvailable: AmountString;
-
-  /**
-   * Balance of type "material" (see balance.ts for definition).
-   */
-  balanceMaterial: AmountString;
-
-  /**
-   * If non-zero, the largest fee gap estimate of an exchange
-   * where we would otherwise have enough balance available.
-   */
-  feeGapEstimate: AmountString;
-
-  perExchange: {
-    [url: string]: {
-      balanceAvailable: AmountString;
-      balanceMaterial: AmountString;
-      feeGapEstimate: AmountString;
-    };
-  };
-}
-
 export interface ValidateIbanRequest {
   iban: string;
 }
@@ -2754,7 +2963,6 @@ export interface WalletContractData {
   summary: string;
   summaryI18n: { [lang_tag: string]: string } | undefined;
   autoRefund: TalerProtocolDuration | undefined;
-  maxWireFee: AmountString;
   wireFeeAmortization: number;
   payDeadline: TalerProtocolTimestamp;
   refundDeadline: TalerProtocolTimestamp;
@@ -2767,10 +2975,26 @@ export interface WalletContractData {
 }
 
 export interface TestingWaitTransactionRequest {
-  transactionId: string;
+  transactionId: TransactionIdStr;
   txState: TransactionState;
 }
 
+export interface TestingGetDenomStatsRequest {
+  exchangeBaseUrl: string;
+}
+
+export interface TestingGetDenomStatsResponse {
+  numKnown: number;
+  numOffered: number;
+  numLost: number;
+}
+
+export const codecForTestingGetDenomStatsRequest =
+  (): Codec<TestingGetDenomStatsRequest> =>
+    buildCodecForObject<TestingGetDenomStatsRequest>()
+      .property("exchangeBaseUrl", codecForString())
+      .build("TestingGetDenomStatsRequest");
+
 export interface WithdrawalExchangeAccountDetails {
   /**
    * Payto URI to credit the exchange.
@@ -2781,6 +3005,16 @@ export interface WithdrawalExchangeAccountDetails {
    */
   paytoUri: string;
 
+  /**
+   * Status that indicates whether the account can be used
+   * by the user to send funds for a withdrawal.
+   *
+   * ok: account should be shown to the user
+   * error: account should not be shown to the user, UIs might render the error (in conversionError),
+   *   especially in dev mode.
+   */
+  status: "ok" | "error";
+
   /**
    * Transfer amount. Might be in a different currency than the requested
    * amount for withdrawal.
@@ -2803,4 +3037,231 @@ export interface WithdrawalExchangeAccountDetails {
    * exchange.
    */
   creditRestrictions?: AccountRestriction[];
+
+  /**
+   * Error that happened when attempting to request the conversion rate.
+   */
+  conversionError?: TalerErrorDetail;
 }
+
+export interface PrepareWithdrawExchangeRequest {
+  /**
+   * A taler://withdraw-exchange URI.
+   */
+  talerUri: string;
+}
+
+export const codecForPrepareWithdrawExchangeRequest =
+  (): Codec<PrepareWithdrawExchangeRequest> =>
+    buildCodecForObject<PrepareWithdrawExchangeRequest>()
+      .property("talerUri", codecForString())
+      .build("PrepareWithdrawExchangeRequest");
+
+export interface PrepareWithdrawExchangeResponse {
+  /**
+   * Base URL of the exchange that already existed
+   * or was ephemerally added as an exchange entry to
+   * the wallet.
+   */
+  exchangeBaseUrl: string;
+
+  /**
+   * Amount from the taler://withdraw-exchange URI.
+   * Only present if specified in the URI.
+   */
+  amount?: AmountString;
+}
+
+export interface ExchangeEntryState {
+  tosStatus: ExchangeTosStatus;
+  exchangeEntryStatus: ExchangeEntryStatus;
+  exchangeUpdateStatus: ExchangeUpdateStatus;
+}
+
+export interface ListGlobalCurrencyAuditorsResponse {
+  auditors: {
+    currency: string;
+    auditorBaseUrl: string;
+    auditorPub: string;
+  }[];
+}
+
+export interface ListGlobalCurrencyExchangesResponse {
+  exchanges: {
+    currency: string;
+    exchangeBaseUrl: string;
+    exchangeMasterPub: string;
+  }[];
+}
+
+export interface AddGlobalCurrencyExchangeRequest {
+  currency: string;
+  exchangeBaseUrl: string;
+  exchangeMasterPub: string;
+}
+
+export const codecForAddGlobalCurrencyExchangeRequest =
+  (): Codec<AddGlobalCurrencyExchangeRequest> =>
+    buildCodecForObject<AddGlobalCurrencyExchangeRequest>()
+      .property("currency", codecForString())
+      .property("exchangeBaseUrl", codecForString())
+      .property("exchangeMasterPub", codecForString())
+      .build("AddGlobalCurrencyExchangeRequest");
+
+export interface RemoveGlobalCurrencyExchangeRequest {
+  currency: string;
+  exchangeBaseUrl: string;
+  exchangeMasterPub: string;
+}
+
+export const codecForRemoveGlobalCurrencyExchangeRequest =
+  (): Codec<RemoveGlobalCurrencyExchangeRequest> =>
+    buildCodecForObject<RemoveGlobalCurrencyExchangeRequest>()
+      .property("currency", codecForString())
+      .property("exchangeBaseUrl", codecForString())
+      .property("exchangeMasterPub", codecForString())
+      .build("RemoveGlobalCurrencyExchangeRequest");
+
+export interface AddGlobalCurrencyAuditorRequest {
+  currency: string;
+  auditorBaseUrl: string;
+  auditorPub: string;
+}
+
+export const codecForAddGlobalCurrencyAuditorRequest =
+  (): Codec<AddGlobalCurrencyAuditorRequest> =>
+    buildCodecForObject<AddGlobalCurrencyAuditorRequest>()
+      .property("currency", codecForString())
+      .property("auditorBaseUrl", codecForString())
+      .property("auditorPub", codecForString())
+      .build("AddGlobalCurrencyAuditorRequest");
+
+export interface RemoveGlobalCurrencyAuditorRequest {
+  currency: string;
+  auditorBaseUrl: string;
+  auditorPub: string;
+}
+
+export const codecForRemoveGlobalCurrencyAuditorRequest =
+  (): Codec<RemoveGlobalCurrencyAuditorRequest> =>
+    buildCodecForObject<RemoveGlobalCurrencyAuditorRequest>()
+      .property("currency", codecForString())
+      .property("auditorBaseUrl", codecForString())
+      .property("auditorPub", codecForString())
+      .build("RemoveGlobalCurrencyAuditorRequest");
+
+export interface RetryLoopOpts {
+  /**
+   * Stop the retry loop when all lifeness-giving pending operations
+   * are done.
+   *
+   * Defaults to false.
+   */
+  stopWhenDone?: boolean;
+}
+
+/**
+ * Information about one provider.
+ *
+ * We don't store the account key here,
+ * as that's derived from the wallet root key.
+ */
+export interface ProviderInfo {
+  active: boolean;
+  syncProviderBaseUrl: string;
+  name: string;
+  terms?: BackupProviderTerms;
+  /**
+   * Last communication issue with the provider.
+   */
+  lastError?: TalerErrorDetail;
+  lastSuccessfulBackupTimestamp?: TalerPreciseTimestamp;
+  lastAttemptedBackupTimestamp?: TalerPreciseTimestamp;
+  paymentProposalIds: string[];
+  backupProblem?: BackupProblem;
+  paymentStatus: ProviderPaymentStatus;
+}
+
+export interface BackupProviderTerms {
+  supportedProtocolVersion: string;
+  annualFee: AmountString;
+  storageLimitInMegabytes: number;
+}
+
+export type BackupProblem =
+  | BackupUnreadableProblem
+  | BackupConflictingDeviceProblem;
+
+export interface BackupUnreadableProblem {
+  type: "backup-unreadable";
+}
+
+export interface BackupConflictingDeviceProblem {
+  type: "backup-conflicting-device";
+  otherDeviceId: string;
+  myDeviceId: string;
+  backupTimestamp: AbsoluteTime;
+}
+
+export type ProviderPaymentStatus =
+  | ProviderPaymentTermsChanged
+  | ProviderPaymentPaid
+  | ProviderPaymentInsufficientBalance
+  | ProviderPaymentUnpaid
+  | ProviderPaymentPending;
+
+export enum ProviderPaymentType {
+  Unpaid = "unpaid",
+  Pending = "pending",
+  InsufficientBalance = "insufficient-balance",
+  Paid = "paid",
+  TermsChanged = "terms-changed",
+}
+
+export interface ProviderPaymentUnpaid {
+  type: ProviderPaymentType.Unpaid;
+}
+
+export interface ProviderPaymentInsufficientBalance {
+  type: ProviderPaymentType.InsufficientBalance;
+  amount: AmountString;
+}
+
+export interface ProviderPaymentPending {
+  type: ProviderPaymentType.Pending;
+  talerUri?: string;
+}
+
+export interface ProviderPaymentPaid {
+  type: ProviderPaymentType.Paid;
+  paidUntil: AbsoluteTime;
+}
+
+export interface ProviderPaymentTermsChanged {
+  type: ProviderPaymentType.TermsChanged;
+  paidUntil: AbsoluteTime;
+  oldTerms: BackupProviderTerms;
+  newTerms: BackupProviderTerms;
+}
+
+// FIXME: Does not really belong here, move to sync API
+export interface SyncTermsOfServiceResponse {
+  // maximum backup size supported
+  storage_limit_in_megabytes: number;
+
+  // Fee for an account, per year.
+  annual_fee: AmountString;
+
+  // protocol version supported by the server,
+  // for now always "0.0".
+  version: string;
+}
+
+// FIXME: Does not really belong here, move to sync API
+export const codecForSyncTermsOfServiceResponse =
+  (): Codec<SyncTermsOfServiceResponse> =>
+    buildCodecForObject<SyncTermsOfServiceResponse>()
+      .property("storage_limit_in_megabytes", codecForNumber())
+      .property("annual_fee", codecForAmountString())
+      .property("version", codecForString())
+      .build("SyncTermsOfServiceResponse");
