@@ -31,10 +31,10 @@ import {
   getRandomBytes,
   j2s,
   Logger,
+  NotificationType,
   parsePaytoUri,
   parseTalerUri,
   PreparePayResultType,
-  RecoveryMergeStrategy,
   sampleWalletCoreTransactions,
   setDangerousTimetravel,
   setGlobalLogLevelFromString,
@@ -56,7 +56,6 @@ import { createPlatformHttpLib } from "@gnu-taler/taler-util/http";
 import { JsonMessage, runRpcServer } from "@gnu-taler/taler-util/twrpc";
 import {
   AccessStats,
-  createNativeWalletHost,
   createNativeWalletHost2,
   Wallet,
   WalletApiOperation,
@@ -68,6 +67,8 @@ import {
   makeNotificationWaiter,
 } from "@gnu-taler/taler-wallet-core/remote";
 
+import * as fs from "node:fs";
+
 // This module also serves as the entry point for the crypto
 // thread worker, and thus must expose these two handlers.
 export {
@@ -77,9 +78,10 @@ export {
 
 const logger = new Logger("taler-wallet-cli.ts");
 
+let observabilityEventFile: string | undefined = undefined;
+
 const EXIT_EXCEPTION = 4;
 const EXIT_API_ERROR = 5;
-const EXIT_RETRIES_EXCEEDED = 6;
 
 setUnhandledRejectionHandler((error: any) => {
   logger.error("unhandledRejection", error.message);
@@ -156,8 +158,9 @@ function applyVerbose(verbose: boolean): void {
 }
 
 declare const __VERSION__: string;
+declare const __GIT_HASH__: string;
 function printVersion(): void {
-  console.log(__VERSION__);
+  console.log(`${__VERSION__} ${__GIT_HASH__}`);
   processExit(0);
 }
 
@@ -248,6 +251,7 @@ interface CreateWalletResult {
 async function createLocalWallet(
   walletCliArgs: WalletCliArgsType,
   notificationHandler?: (n: WalletNotification) => void,
+  noInit?: boolean,
 ): Promise<CreateWalletResult> {
   const dbPath = walletCliArgs.wallet.walletDbFile ?? defaultWalletDbPath;
   const myHttpLib = createPlatformHttpLib({
@@ -264,30 +268,34 @@ async function createLocalWallet(
       }
     },
     cryptoWorkerType: walletCliArgs.wallet.cryptoWorker as any,
-    config: {
-      features: {},
-      testing: {
-        devModeActive: checkEnvFlag("TALER_WALLET_DEV_MODE"),
-        denomselAllowLate: checkEnvFlag(
-          "TALER_WALLET_DEBUG_DENOMSEL_ALLOW_LATE",
-        ),
-        skipDefaults: walletCliArgs.wallet.skipDefaults,
-      },
-    },
   });
 
   applyVerbose(walletCliArgs.wallet.verbose);
+  const res = { wallet: wh.wallet, getStats: wh.getDbStats };
+
+  if (noInit) {
+    return res;
+  }
   try {
-    await wh.wallet.handleCoreApiRequest("initWallet", "native-init", {});
-    return { wallet: wh.wallet, getStats: wh.getDbStats };
+    await wh.wallet.handleCoreApiRequest("initWallet", "native-init", {
+      config: {
+        features: {},
+        testing: {
+          devModeActive: checkEnvFlag("TALER_WALLET_DEV_MODE"),
+          denomselAllowLate: checkEnvFlag(
+            "TALER_WALLET_DEBUG_DENOMSEL_ALLOW_LATE",
+          ),
+          emitObservabilityEvents: observabilityEventFile != null,
+          skipDefaults: walletCliArgs.wallet.skipDefaults,
+        },
+      },
+    });
+    return res;
   } catch (e) {
     const ed = getErrorDetailFromException(e);
     console.error("Operation failed: " + summarizeTalerErrorDetail(ed));
     console.error("Error details:", JSON.stringify(ed, undefined, 2));
     processExit(1);
-  } finally {
-    logger.trace("operation with wallet finished, stopping");
-    logger.trace("stopped wallet");
   }
 }
 
@@ -297,10 +305,26 @@ async function withWallet<T>(
 ): Promise<T> {
   const waiter = makeNotificationWaiter();
 
+  const onNotif = (notif: WalletNotification) => {
+    waiter.notify(notif);
+    if (observabilityEventFile) {
+      switch (notif.type) {
+        case NotificationType.RequestObservabilityEvent:
+        case NotificationType.TaskObservabilityEvent:
+          fs.appendFileSync(
+            observabilityEventFile,
+            JSON.stringify(notif) + "\n",
+          );
+          break;
+      }
+    }
+  };
+
   if (walletCliArgs.wallet.walletConnection) {
     logger.info("creating remote wallet");
     const w = await createRemoteWallet({
-      notificationHandler: waiter.notify,
+      name: "wallet",
+      notificationHandler: onNotif,
       socketFilename: walletCliArgs.wallet.walletConnection,
     });
     const ctx: WalletContext = {
@@ -314,7 +338,7 @@ async function withWallet<T>(
     w.close();
     return res;
   } else {
-    const wh = await createLocalWallet(walletCliArgs, waiter.notify);
+    const wh = await createLocalWallet(walletCliArgs, onNotif);
     const ctx: WalletContext = {
       client: wh.wallet.client,
       waitForNotificationCond: waiter.waitForNotificationCond,
@@ -344,6 +368,7 @@ async function withLocalWallet<T>(
   const wh = await createLocalWallet(walletCliArgs);
   const w = wh.wallet;
   const res = await f({ client: w.client, ws: w });
+  logger.info("Work done, stopping wallet.");
   w.stop();
   return res;
 }
@@ -553,19 +578,13 @@ walletCli
   .subcommand("finishPendingOpt", "run-until-done", {
     help: "Run until no more work is left.",
   })
-  .maybeOption("maxRetries", ["--max-retries"], clk.INT)
-  .flag("failOnMaxRetries", ["--fail-on-max-retries"])
   .action(async (args) => {
     await withLocalWallet(args, async (wallet) => {
       logger.info("running until pending operations are finished");
-      const resp = await wallet.ws.runTaskLoop({
-        maxRetries: args.finishPendingOpt.maxRetries,
+      await wallet.ws.runTaskLoop({
         stopWhenDone: true,
       });
       wallet.ws.stop();
-      if (resp.retriesExceeded && args.finishPendingOpt.failOnMaxRetries) {
-        processExit(EXIT_RETRIES_EXCEEDED);
-      }
     });
   });
 
@@ -664,19 +683,6 @@ walletCli
             alwaysYes: args.handleUri.autoYes,
           });
           break;
-        case TalerUriAction.Reward: {
-          const res = await wallet.client.call(
-            WalletApiOperation.PrepareReward,
-            {
-              talerRewardUri: uri,
-            },
-          );
-          console.log("tip status", res);
-          await wallet.client.call(WalletApiOperation.AcceptReward, {
-            walletRewardId: res.walletRewardId,
-          });
-          break;
-        }
         case TalerUriAction.Refund:
           await wallet.client.call(WalletApiOperation.StartRefundQueryForUri, {
             talerRefundUri: uri,
@@ -799,9 +805,9 @@ exchangesCli
   .flag("force", ["-f", "--force"])
   .action(async (args) => {
     await withWallet(args, async (wallet) => {
-      await wallet.client.call(WalletApiOperation.AddExchange, {
+      await wallet.client.call(WalletApiOperation.UpdateExchangeEntry, {
         exchangeBaseUrl: args.exchangesUpdateCmd.url,
-        forceUpdate: args.exchangesUpdateCmd.force,
+        force: args.exchangesUpdateCmd.force,
       });
     });
   });
@@ -840,6 +846,23 @@ exchangesCli
     });
   });
 
+exchangesCli
+  .subcommand("exchangesAddCmd", "delete", {
+    help: "Delete an exchange by base URL.",
+  })
+  .requiredArgument("url", clk.STRING, {
+    help: "Base URL of the exchange.",
+  })
+  .flag("purge", ["--purge"])
+  .action(async (args) => {
+    await withWallet(args, async (wallet) => {
+      await wallet.client.call(WalletApiOperation.DeleteExchange, {
+        exchangeBaseUrl: args.exchangesAddCmd.url,
+        purge: args.exchangesAddCmd.purge,
+      });
+    });
+  });
+
 exchangesCli
   .subcommand("exchangesAcceptTosCmd", "accept-tos", {
     help: "Accept terms of service.",
@@ -847,13 +870,9 @@ exchangesCli
   .requiredArgument("url", clk.STRING, {
     help: "Base URL of the exchange.",
   })
-  .requiredArgument("etag", clk.STRING, {
-    help: "ToS version tag to accept",
-  })
   .action(async (args) => {
     await withWallet(args, async (wallet) => {
       await wallet.client.call(WalletApiOperation.SetExchangeTosAccepted, {
-        etag: args.exchangesAcceptTosCmd.etag,
         exchangeBaseUrl: args.exchangesAcceptTosCmd.url,
       });
     });
@@ -976,7 +995,6 @@ depositCli
         },
       );
       console.log(`Created deposit ${resp.depositGroupId}`);
-      await wallet.ws.runPending();
     });
   });
 
@@ -1054,13 +1072,13 @@ peerCli
 
 peerCli
   .subcommand("confirmIncomingPayPush", "confirm-push-credit")
-  .requiredArgument("peerPushCreditId", clk.STRING)
+  .requiredArgument("transactionId", clk.STRING)
   .action(async (args) => {
     await withWallet(args, async (wallet) => {
       const resp = await wallet.client.call(
         WalletApiOperation.ConfirmPeerPushCredit,
         {
-          peerPushCreditId: args.confirmIncomingPayPush.peerPushCreditId,
+          transactionId: args.confirmIncomingPayPush.transactionId,
         },
       );
       console.log(JSON.stringify(resp, undefined, 2));
@@ -1185,17 +1203,26 @@ advancedCli
   .requiredOption("unixPath", ["--unix-path"], clk.STRING, {
     default: "wallet-core.sock",
   })
+  .flag("noInit", ["--no-init"], {
+    help: "Do not initialize the wallet. The client must send the initWallet message.",
+  })
   .action(async (args) => {
     logger.info(`serving at ${args.serve.unixPath}`);
-    const wh = await createLocalWallet(args);
+    const onNotif = (notif: WalletNotification) => {
+      if (observabilityEventFile) {
+        switch (notif.type) {
+          case NotificationType.RequestObservabilityEvent:
+          case NotificationType.TaskObservabilityEvent:
+            fs.appendFileSync(
+              observabilityEventFile,
+              JSON.stringify(notif) + "\n",
+            );
+            break;
+        }
+      }
+    };
+    const wh = await createLocalWallet(args, onNotif, args.serve.noInit);
     const w = wh.wallet;
-    w.runTaskLoop()
-      .then((res) => {
-        logger.warn("task loop exited unexpectedly");
-      })
-      .catch((e) => {
-        logger.error(`error in task loop: ${e}`);
-      });
     let nextClientId = 1;
     const notifyHandlers = new Map<number, (n: WalletNotification) => void>();
     w.addNotificationListener((n) => {
@@ -1251,9 +1278,9 @@ advancedCli
     help: "Run pending operations.",
   })
   .action(async (args) => {
-    await withLocalWallet(args, async (wallet) => {
-      await wallet.ws.runPending();
-    });
+    logger.error(
+      "Subcommand run-pending not supported anymore.  Please use run-until-done or the client/server wallet.",
+    );
   });
 
 advancedCli
@@ -1308,17 +1335,117 @@ const currenciesCli = walletCli.subcommand("currencies", "currencies", {
 });
 
 currenciesCli
-  .subcommand("show", "show", { help: "Show currencies." })
+  .subcommand("listGlobalAuditors", "list-global-auditors", {
+    help: "List global-currency auditors.",
+  })
   .action(async (args) => {
     await withWallet(args, async (wallet) => {
       const currencies = await wallet.client.call(
-        WalletApiOperation.ListCurrencies,
+        WalletApiOperation.ListGlobalCurrencyAuditors,
         {},
       );
       console.log(JSON.stringify(currencies, undefined, 2));
     });
   });
 
+currenciesCli
+  .subcommand("listGlobalExchanges", "list-global-exchanges", {
+    help: "List global-currency exchanges.",
+  })
+  .action(async (args) => {
+    await withWallet(args, async (wallet) => {
+      const currencies = await wallet.client.call(
+        WalletApiOperation.ListGlobalCurrencyExchanges,
+        {},
+      );
+      console.log(JSON.stringify(currencies, undefined, 2));
+    });
+  });
+
+currenciesCli
+  .subcommand("addGlobalExchange", "add-global-exchange", {
+    help: "Add a global-currency exchange.",
+  })
+  .requiredOption("currency", ["--currency"], clk.STRING)
+  .requiredOption("exchangeBaseUrl", ["--url"], clk.STRING)
+  .requiredOption("exchangePub", ["--pub"], clk.STRING)
+  .action(async (args) => {
+    await withWallet(args, async (wallet) => {
+      const currencies = await wallet.client.call(
+        WalletApiOperation.AddGlobalCurrencyExchange,
+        {
+          currency: args.addGlobalExchange.currency,
+          exchangeBaseUrl: args.addGlobalExchange.exchangeBaseUrl,
+          exchangeMasterPub: args.addGlobalExchange.exchangePub,
+        },
+      );
+      console.log(JSON.stringify(currencies, undefined, 2));
+    });
+  });
+
+currenciesCli
+  .subcommand("removeGlobalExchange", "remove-global-exchange", {
+    help: "Remove a global-currency exchange.",
+  })
+  .requiredOption("currency", ["--currency"], clk.STRING)
+  .requiredOption("exchangeBaseUrl", ["--url"], clk.STRING)
+  .requiredOption("exchangePub", ["--pub"], clk.STRING)
+  .action(async (args) => {
+    await withWallet(args, async (wallet) => {
+      const currencies = await wallet.client.call(
+        WalletApiOperation.RemoveGlobalCurrencyExchange,
+        {
+          currency: args.removeGlobalExchange.currency,
+          exchangeBaseUrl: args.removeGlobalExchange.exchangeBaseUrl,
+          exchangeMasterPub: args.removeGlobalExchange.exchangePub,
+        },
+      );
+      console.log(JSON.stringify(currencies, undefined, 2));
+    });
+  });
+
+currenciesCli
+  .subcommand("addGlobalAuditor", "add-global-auditor", {
+    help: "Add a global-currency auditor.",
+  })
+  .requiredOption("currency", ["--currency"], clk.STRING)
+  .requiredOption("auditorBaseUrl", ["--url"], clk.STRING)
+  .requiredOption("auditorPub", ["--pub"], clk.STRING)
+  .action(async (args) => {
+    await withWallet(args, async (wallet) => {
+      const currencies = await wallet.client.call(
+        WalletApiOperation.AddGlobalCurrencyAuditor,
+        {
+          currency: args.addGlobalAuditor.currency,
+          auditorBaseUrl: args.addGlobalAuditor.auditorBaseUrl,
+          auditorPub: args.addGlobalAuditor.auditorPub,
+        },
+      );
+      console.log(JSON.stringify(currencies, undefined, 2));
+    });
+  });
+
+currenciesCli
+  .subcommand("removeGlobalAuditor", "remove-global-auditor", {
+    help: "Remove a global-currency auditor.",
+  })
+  .requiredOption("currency", ["--currency"], clk.STRING)
+  .requiredOption("auditorBaseUrl", ["--url"], clk.STRING)
+  .requiredOption("auditorPub", ["--pub"], clk.STRING)
+  .action(async (args) => {
+    await withWallet(args, async (wallet) => {
+      const currencies = await wallet.client.call(
+        WalletApiOperation.RemoveGlobalCurrencyAuditor,
+        {
+          currency: args.removeGlobalAuditor.currency,
+          auditorBaseUrl: args.removeGlobalAuditor.auditorBaseUrl,
+          auditorPub: args.removeGlobalAuditor.auditorPub,
+        },
+      );
+      console.log(JSON.stringify(currencies, undefined, 2));
+    });
+  });
+
 advancedCli
   .subcommand("clearDatabase", "clear-database", {
     help: "Clear the database, irrevocable deleting all data in the wallet.",
@@ -1645,5 +1772,9 @@ async function read(stream: NodeJS.ReadStream) {
 }
 
 export function main() {
+  const maybeFilename = getenv("TALER_WALLET_DEBUG_OBSERVE");
+  if (!!maybeFilename) {
+    observabilityEventFile = maybeFilename;
+  }
   walletCli.run();
 }
