@@ -103,6 +103,8 @@ import {
   EncryptContractForDepositResponse,
   EncryptContractRequest,
   EncryptContractResponse,
+  SignCoinHistoryRequest,
+  SignCoinHistoryResponse,
   SignDeletePurseRequest,
   SignDeletePurseResponse,
   SignPurseMergeRequest,
@@ -243,6 +245,10 @@ export interface TalerCryptoInterface {
   signDeletePurse(
     req: SignDeletePurseRequest,
   ): Promise<SignDeletePurseResponse>;
+
+  signCoinHistoryRequest(
+    req: SignCoinHistoryRequest,
+  ): Promise<SignCoinHistoryResponse>;
 }
 
 /**
@@ -427,6 +433,11 @@ export const nullCrypto: TalerCryptoInterface = {
   ): Promise<SignDeletePurseResponse> {
     throw new Error("Function not implemented.");
   },
+  signCoinHistoryRequest: function (
+    req: SignCoinHistoryRequest,
+  ): Promise<SignCoinHistoryResponse> {
+    throw new Error("Function not implemented.");
+  },
 };
 
 export type WithArg<X> = X extends (req: infer T) => infer R
@@ -1705,6 +1716,23 @@ export const nativeCryptoR: TalerCryptoInterfaceR = {
       sig: sigResp.sig,
     };
   },
+  async signCoinHistoryRequest(
+    tci: TalerCryptoInterfaceR,
+    req: SignCoinHistoryRequest,
+  ): Promise<SignCoinHistoryResponse> {
+    const coinHistorySigBlob = buildSigPS(
+      TalerSignaturePurpose.WALLET_COIN_HISTORY,
+    )
+      .put(bufferForUint64(req.startOffset))
+      .build();
+    const sigResp = await tci.eddsaSign(tci, {
+      msg: encodeCrock(coinHistorySigBlob),
+      priv: req.coinPriv,
+    });
+    return {
+      sig: sigResp.sig,
+    };
+  },
 };
 
 export interface EddsaSignRequest {
