@@ -19,13 +19,13 @@
  */
 import {
   Event,
-  GlobalIDB,
   IDBDatabase,
   IDBFactory,
   IDBObjectStore,
   IDBRequest,
   IDBTransaction,
   structuredEncapsulate,
+  structuredRevive,
 } from "@gnu-taler/idb-bridge";
 import {
   AbsoluteTime,
@@ -33,6 +33,7 @@ import {
   AmountString,
   Amounts,
   AttentionInfo,
+  BackupProviderTerms,
   Codec,
   CoinEnvelope,
   CoinPublicKeyString,
@@ -47,7 +48,6 @@ import {
   ExchangeGlobalFees,
   HashCodeString,
   Logger,
-  PayCoinSelection,
   RefreshReason,
   TalerErrorDetail,
   TalerPreciseTimestamp,
@@ -59,12 +59,12 @@ import {
   WithdrawalExchangeAccountDetails,
   codecForAny,
 } from "@gnu-taler/taler-util";
-import { DbRetryInfo, TaskIdentifiers } from "./operations/common.js";
+import { DbRetryInfo, TaskIdentifiers } from "./common.js";
 import {
   DbAccess,
+  DbAccessImpl,
   DbReadOnlyTransaction,
   DbReadWriteTransaction,
-  GetReadWriteAccess,
   IndexDescriptor,
   StoreDescriptor,
   StoreNames,
@@ -72,8 +72,9 @@ import {
   describeContents,
   describeIndex,
   describeStore,
+  describeStoreV2,
   openDatabase,
-} from "./util/query.js";
+} from "./query.js";
 
 /**
  * This file contains the database schema of the Taler wallet together
@@ -147,7 +148,7 @@ export const CURRENT_DB_CONFIG_KEY = "currentMainDbName";
  * backwards-compatible way or object stores and indices
  * are added.
  */
-export const WALLET_DB_MINOR_VERSION = 1;
+export const WALLET_DB_MINOR_VERSION = 6;
 
 declare const symDbProtocolTimestamp: unique symbol;
 
@@ -253,6 +254,16 @@ export function timestampOptionalAbsoluteFromDb(
  * nnnn=0000 should always be the most generic minor state for the major state
  */
 
+/**
+ * First possible operation status in the active range (inclusive).
+ */
+export const OPERATION_STATUS_ACTIVE_FIRST = 0x0100_0000;
+
+/**
+ * LAST possible operation status in the active range (inclusive).
+ */
+export const OPERATION_STATUS_ACTIVE_LAST = 0x0113_ffff;
+
 /**
  * Status of a withdrawal.
  */
@@ -326,14 +337,6 @@ export enum WithdrawalGroupStatus {
   AbortedBank = 0x0503_0002,
 }
 
-/**
- * Status range of nonfinal withdrawal groups.
- */
-export const withdrawalGroupNonfinalRange = GlobalIDB.KeyRange.bound(
-  WithdrawalGroupStatus.PendingRegisteringBank,
-  WithdrawalGroupStatus.PendingAml,
-);
-
 /**
  * Extra info about a withdrawal that is used
  * with a bank-integrated withdrawal.
@@ -482,6 +485,13 @@ export interface DenominationRecord {
    */
   isRevoked: boolean;
 
+  /**
+   * If set to true, the exchange announced that the private key for this
+   * denomination is lost.  Thus it can't be used to sign new coins
+   * during withdrawal/refresh/..., but the coins can still be spent.
+   */
+  isLost?: boolean;
+
   /**
    * Base URL of the exchange.
    */
@@ -568,21 +578,6 @@ export interface ExchangeDetailsRecord {
    */
   globalFees: ExchangeGlobalFees[];
 
-  /**
-   * Etag of the current ToS of the exchange.
-   */
-  tosCurrentEtag: string;
-
-  /**
-   * Information about ToS acceptance from the user.
-   */
-  tosAccepted:
-    | {
-        etag: string;
-        timestamp: DbPreciseTimestamp;
-      }
-    | undefined;
-
   wireInfo: WireInfo;
 
   /**
@@ -614,8 +609,8 @@ export enum ExchangeEntryDbUpdateStatus {
   Initial = 1,
   InitialUpdate = 2,
   Suspended = 3,
-  Failed = 4,
-  OutdatedUpdate = 5,
+  UnavailableUpdate = 4,
+  // Reserved 5 for backwards compatibility.
   Ready = 6,
   ReadyUpdate = 7,
 }
@@ -658,6 +653,21 @@ export interface ExchangeEntryRecord {
 
   updateStatus: ExchangeEntryDbUpdateStatus;
 
+  /**
+   * If set to true, the next update to the exchange
+   * status will request /keys with no-cache headers set.
+   */
+  cachebreakNextUpdate?: boolean;
+
+  /**
+   * Etag of the current ToS of the exchange.
+   */
+  tosCurrentEtag: string | undefined;
+
+  tosAcceptedEtag: string | undefined;
+
+  tosAcceptedTimestamp: DbPreciseTimestamp | undefined;
+
   /**
    * Last time when the exchange /keys info was updated.
    */
@@ -959,6 +969,7 @@ export enum RewardRecordStatus {
   DialogAccept = 0x0101_0000,
   Done = 0x0500_0000,
   Aborted = 0x0500_0000,
+  Failed = 0x0501_000,
 }
 
 export enum RefreshCoinStatus {
@@ -995,12 +1006,11 @@ export enum DepositElementStatus {
   RefundFailed = 0x0501_0000,
 }
 
-/**
- * Additional information about the reason of a refresh.
- */
-export interface RefreshReasonDetails {
-  originatingTransactionId?: string;
-  proposalId?: string;
+export interface RefreshGroupPerExchangeInfo {
+  /**
+   * (Expected) output once the refresh group succeeded.
+   */
+  outputEffective: AmountString;
 }
 
 /**
@@ -1027,10 +1037,7 @@ export interface RefreshGroupRecord {
    */
   reason: RefreshReason;
 
-  /**
-   * Extra information depending on the reason.
-   */
-  reasonDetails?: RefreshReasonDetails;
+  originatingTransactionId?: string;
 
   oldCoinPubs: string[];
 
@@ -1038,6 +1045,8 @@ export interface RefreshGroupRecord {
 
   expectedOutputPerCoin: AmountString[];
 
+  infoPerExchange?: Record<string, RefreshGroupPerExchangeInfo>;
+
   /**
    * Flag for each coin whether refreshing finished.
    * If a coin can't be refreshed (remaining value too small),
@@ -1165,6 +1174,13 @@ export enum PurchaseStatus {
    */
   FailedClaim = 0x0501_0000,
 
+  /**
+   * Tried to abort, but aborting failed or was cancelled.
+   */
+  FailedAbort = 0x0501_0001,
+
+  FailedPaidByOther = 0x0501_0002,
+
   /**
    * Payment was successful.
    */
@@ -1180,12 +1196,9 @@ export enum PurchaseStatus {
    */
   AbortedIncompletePayment = 0x0503_0000,
 
-  /**
-   * Tried to abort, but aborting failed or was cancelled.
-   */
-  FailedAbort = 0x0501_0001,
+  AbortedRefunded = 0x0503_0001,
 
-  AbortedRefunded = 0x0503_0000,
+  AbortedOrderDeleted = 0x0503_0002,
 }
 
 /**
@@ -1200,8 +1213,13 @@ export interface ProposalDownloadInfo {
   contractTermsMerchantSig: string;
 }
 
+export interface DbCoinSelection {
+  coinPubs: string[];
+  coinContributions: AmountString[];
+}
+
 export interface PurchasePayInfo {
-  payCoinSelection: PayCoinSelection;
+  payCoinSelection: DbCoinSelection;
   totalPayCost: AmountString;
   payCoinSelectionUid: string;
 }
@@ -1283,8 +1301,9 @@ export interface PurchaseRecord {
   posConfirmation: string | undefined;
 
   /**
-   * This purchase was created by sharing nonce or
-   * did the wallet made the nonce public
+   * This purchase was created by reading
+   * a payment share or the wallet
+   * the nonce public by a payment share
    */
   shared: boolean;
 
@@ -1326,6 +1345,9 @@ export enum ConfigRecordKey {
   WalletBackupState = "walletBackupState",
   CurrencyDefaultsApplied = "currencyDefaultsApplied",
   DevMode = "devMode",
+  // Only for testing, do not use!
+  TestLoopTx = "testTxLoop",
+  LastInitInfo = "lastInitInfo",
 }
 
 /**
@@ -1338,7 +1360,9 @@ export type ConfigRecord =
       value: WalletBackupConfState;
     }
   | { key: ConfigRecordKey.CurrencyDefaultsApplied; value: boolean }
-  | { key: ConfigRecordKey.DevMode; value: boolean };
+  | { key: ConfigRecordKey.DevMode; value: boolean }
+  | { key: ConfigRecordKey.TestLoopTx; value: number }
+  | { key: ConfigRecordKey.LastInitInfo; value: DbProtocolTimestamp };
 
 export interface WalletBackupConfState {
   deviceId: string;
@@ -1378,6 +1402,10 @@ export interface WgInfoBankIntegrated {
    * a Taler-integrated bank.
    */
   bankInfo: ReserveBankInfo;
+  /**
+   * Info about withdrawal accounts, possibly including currency conversion.
+   */
+  exchangeCreditAccounts?: WithdrawalExchangeAccountDetails[];
 }
 
 export interface WgInfoBankManual {
@@ -1553,6 +1581,14 @@ export interface BankWithdrawUriRecord {
   reservePub: string;
 }
 
+export enum RecoupOperationStatus {
+  Pending = 0x0100_0000,
+  Suspended = 0x0110_0000,
+
+  Finished = 0x0500_000,
+  Failed = 0x0501_000,
+}
+
 /**
  * Status of recoup operations that were grouped together.
  *
@@ -1567,6 +1603,8 @@ export interface RecoupGroupRecord {
 
   exchangeBaseUrl: string;
 
+  operationStatus: RecoupOperationStatus;
+
   timestampStarted: DbPreciseTimestamp;
 
   timestampFinished: DbPreciseTimestamp | undefined;
@@ -1609,12 +1647,6 @@ export type BackupProviderState =
       tag: BackupProviderStateTag.Retrying;
     };
 
-export interface BackupProviderTerms {
-  supportedProtocolVersion: string;
-  annualFee: AmountString;
-  storageLimitInMegabytes: number;
-}
-
 export interface BackupProviderRecord {
   /**
    * Base URL of the provider.
@@ -1695,11 +1727,6 @@ export enum DepositOperationStatus {
   Aborted = 0x0503_0000,
 }
 
-export const depositOperationNonfinalStatusRange = GlobalIDB.KeyRange.bound(
-  DepositOperationStatus.PendingDeposit,
-  DepositOperationStatus.PendingKyc,
-);
-
 export interface DepositTrackingInfo {
   // Raw wire transfer identifier of the deposit.
   wireTransferId: string;
@@ -1713,6 +1740,14 @@ export interface DepositTrackingInfo {
   exchangePub: string;
 }
 
+export interface DepositInfoPerExchange {
+  /**
+   * Expected effective amount that will be deposited
+   * from coins of this exchange.
+   */
+  amountEffective: AmountString;
+}
+
 /**
  * Group of deposits made by the wallet.
  */
@@ -1745,7 +1780,7 @@ export interface DepositGroupRecord {
 
   contractTermsHash: string;
 
-  payCoinSelection: PayCoinSelection;
+  payCoinSelection: DbCoinSelection;
 
   payCoinSelectionUid: string;
 
@@ -1764,6 +1799,8 @@ export interface DepositGroupRecord {
 
   statusPerCoin: DepositElementStatus[];
 
+  infoPerExchange?: Record<string, DepositInfoPerExchange>;
+
   /**
    * When the deposit transaction was aborted and
    * refreshes were tried, we create a refresh
@@ -1800,12 +1837,20 @@ export enum PeerPushDebitStatus {
   PendingCreatePurse = 0x0100_0000 /* ACTIVE_START */,
   PendingReady = 0x0100_0001,
   AbortingDeletePurse = 0x0103_0000,
-  AbortingRefresh = 0x0103_0001,
+  /**
+   * Refresh after the purse got deleted by the wallet.
+   */
+  AbortingRefreshDeleted = 0x0103_0001,
+  /**
+   * Refresh after the purse expired.
+   */
+  AbortingRefreshExpired = 0x0103_0002,
 
   SuspendedCreatePurse = 0x0110_0000,
   SuspendedReady = 0x0110_0001,
   SuspendedAbortingDeletePurse = 0x0113_0000,
-  SuspendedAbortingRefresh = 0x0113_0001,
+  SuspendedAbortingRefreshDeleted = 0x0113_0001,
+  SuspendedAbortingRefreshExpired = 0x0113_0002,
 
   Done = 0x0500_0000,
   Aborted = 0x0503_0000,
@@ -1813,7 +1858,7 @@ export enum PeerPushDebitStatus {
   Expired = 0x0502_0000,
 }
 
-export interface PeerPushPaymentCoinSelection {
+export interface DbPeerPushPaymentCoinSelection {
   contributions: AmountString[];
   coinPubs: CoinPublicKeyString[];
 }
@@ -1834,7 +1879,7 @@ export interface PeerPushDebitRecord {
 
   totalCost: AmountString;
 
-  coinSel: PeerPushPaymentCoinSelection;
+  coinSel: DbPeerPushPaymentCoinSelection;
 
   contractTermsHash: HashCodeString;
 
@@ -1900,6 +1945,7 @@ export enum PeerPullPaymentCreditStatus {
 
   Done = 0x0500_0000,
   Failed = 0x0501_0000,
+  Expired = 0x0502_0000,
   Aborted = 0x0503_0000,
 }
 
@@ -2185,26 +2231,12 @@ export interface DbAuditorHandle {
   auditorPub: string;
 }
 
-// Work in progress for regional currencies
-export interface CurrencySettingsRecord {
-  currency: string;
-
-  globalScopeExchanges: DbExchangeHandle[];
-
-  globalScopeAuditors: DbAuditorHandle[];
-
-  // Used to decide which auditor to show the currency under
-  // when multiple auditors apply.
-  auditorPriority: string[];
-
-  // Later, we might add stuff related to how the currency is rendered.
-}
-
 export enum RefundGroupStatus {
   Pending = 0x0100_0000,
   Done = 0x0500_0000,
   Failed = 0x0501_0000,
   Aborted = 0x0503_0000,
+  Expired = 0x0502_0000,
 }
 
 /**
@@ -2286,18 +2318,59 @@ export function passthroughCodec<T>(): Codec<T> {
   return codecForAny();
 }
 
+export interface GlobalCurrencyAuditorRecord {
+  id?: number;
+  currency: string;
+  auditorBaseUrl: string;
+  auditorPub: string;
+}
+
+export interface GlobalCurrencyExchangeRecord {
+  id?: number;
+  currency: string;
+  exchangeBaseUrl: string;
+  exchangeMasterPub: string;
+}
+
 /**
  * Schema definition for the IndexedDB
  * wallet database.
  */
 export const WalletStoresV1 = {
-  currencySettings: describeStore(
-    "currencySettings",
-    describeContents<CurrencySettingsRecord>({
-      keyPath: ["currency"],
-    }),
-    {},
-  ),
+  globalCurrencyAuditors: describeStoreV2({
+    recordCodec: passthroughCodec<GlobalCurrencyAuditorRecord>(),
+    storeName: "globalCurrencyAuditors",
+    keyPath: "id",
+    autoIncrement: true,
+    versionAdded: 3,
+    indexes: {
+      byCurrencyAndUrlAndPub: describeIndex(
+        "byCurrencyAndUrlAndPub",
+        ["currency", "auditorBaseUrl", "auditorPub"],
+        {
+          unique: true,
+          versionAdded: 4,
+        },
+      ),
+    },
+  }),
+  globalCurrencyExchanges: describeStoreV2({
+    recordCodec: passthroughCodec<GlobalCurrencyExchangeRecord>(),
+    storeName: "globalCurrencyExchanges",
+    keyPath: "id",
+    autoIncrement: true,
+    versionAdded: 3,
+    indexes: {
+      byCurrencyAndUrlAndPub: describeIndex(
+        "byCurrencyAndUrlAndPub",
+        ["currency", "exchangeBaseUrl", "exchangeMasterPub"],
+        {
+          unique: true,
+          versionAdded: 4,
+        },
+      ),
+    },
+  }),
   coinAvailability: describeStore(
     "coinAvailability",
     describeContents<CoinAvailabilityRecord>({
@@ -2371,6 +2444,9 @@ export const WalletStoresV1 = {
       autoIncrement: true,
     }),
     {
+      byExchangeBaseUrl: describeIndex("byExchangeBaseUrl", "exchangeBaseUrl", {
+        versionAdded: 2,
+      }),
       byPointer: describeIndex(
         "byDetailsPointer",
         ["exchangeBaseUrl", "currency", "masterPublicKey"],
@@ -2398,6 +2474,13 @@ export const WalletStoresV1 = {
     }),
     {
       byStatus: describeIndex("byStatus", "operationStatus"),
+      byOriginatingTransactionId: describeIndex(
+        "byOriginatingTransactionId",
+        "originatingTransactionId",
+        {
+          versionAdded: 5,
+        },
+      ),
     },
   ),
   refreshSessions: describeStore(
@@ -2412,7 +2495,11 @@ export const WalletStoresV1 = {
     describeContents<RecoupGroupRecord>({
       keyPath: "recoupGroupId",
     }),
-    {},
+    {
+      byStatus: describeIndex("byStatus", "operationStatus", {
+        versionAdded: 6,
+      }),
+    },
   ),
   purchases: describeStore(
     "purchases",
@@ -2449,6 +2536,9 @@ export const WalletStoresV1 = {
     }),
     {
       byStatus: describeIndex("byStatus", "status"),
+      byExchangeBaseUrl: describeIndex("byExchangeBaseUrl", "exchangeBaseUrl", {
+        versionAdded: 2,
+      }),
       byTalerWithdrawUri: describeIndex(
         "byTalerWithdrawUri",
         "wgInfo.bankInfo.talerWithdrawUri",
@@ -2623,6 +2713,7 @@ export const WalletStoresV1 = {
         "coinPub",
         "rtxid",
       ]),
+      // FIXME: Why is this a list of index keys? Confusing!
       byRefundGroupId: describeIndex("byRefundGroupId", ["refundGroupId"]),
     },
   ),
@@ -2635,13 +2726,23 @@ export const WalletStoresV1 = {
   ),
 };
 
-export type WalletDbReadOnlyTransaction<
-  Stores extends StoreNames<typeof WalletStoresV1> & string,
-> = DbReadOnlyTransaction<typeof WalletStoresV1, Stores>;
-
 export type WalletDbReadWriteTransaction<
-  Stores extends StoreNames<typeof WalletStoresV1> & string,
-> = DbReadWriteTransaction<typeof WalletStoresV1, Stores>;
+  StoresArr extends Array<StoreNames<typeof WalletStoresV1>>,
+> = DbReadWriteTransaction<typeof WalletStoresV1, StoresArr>;
+
+export type WalletDbReadOnlyTransaction<
+  StoresArr extends Array<StoreNames<typeof WalletStoresV1>>,
+> = DbReadOnlyTransaction<typeof WalletStoresV1, StoresArr>;
+
+export type WalletDbAllStoresReadOnlyTransaction<> = DbReadOnlyTransaction<
+  typeof WalletStoresV1,
+  Array<StoreNames<typeof WalletStoresV1>>
+>;
+
+export type WalletDbAllStoresReadWriteTransaction<> = DbReadWriteTransaction<
+  typeof WalletStoresV1,
+  Array<StoreNames<typeof WalletStoresV1>>
+>;
 
 /**
  * An applied migration.
@@ -2835,7 +2936,7 @@ function checkDbDump(x: any): x is DbDump {
 }
 
 export async function importDb(db: IDBDatabase, dumpJson: any): Promise<void> {
-  const d = dumpJson;
+  const d = structuredRevive(dumpJson);
   if (checkDbDump(d)) {
     const walletDb = d.databases[TALER_WALLET_MAIN_DB_NAME];
     if (!walletDb) {
@@ -2851,7 +2952,12 @@ export async function importDb(db: IDBDatabase, dumpJson: any): Promise<void> {
 
 export interface FixupDescription {
   name: string;
-  fn(tx: GetReadWriteAccess<typeof WalletStoresV1>): Promise<void>;
+  fn(
+    tx: DbReadWriteTransaction<
+      typeof WalletStoresV1,
+      Array<StoreNames<typeof WalletStoresV1>>
+    >,
+  ): Promise<void>;
 }
 
 /**
@@ -2865,7 +2971,7 @@ export async function applyFixups(
   db: DbAccess<typeof WalletStoresV1>,
 ): Promise<void> {
   logger.trace("applying fixups");
-  await db.mktxAll().runReadWrite(async (tx) => {
+  await db.runAllStoresReadWriteTx(async (tx) => {
     for (const fixupInstruction of walletDbFixups) {
       logger.trace(`checking fixup ${fixupInstruction.name}`);
       const fixupRecord = await tx.fixups.get(fixupInstruction.name);
@@ -2939,8 +3045,10 @@ function upgradeFromStoreMap(
           });
         } catch (e) {
           const moreInfo = e instanceof Error ? ` Reason: ${e.message}` : "";
-          throw Error(
+          throw new Error(
             `Migration failed. Could not create store ${swi.storeName}.${moreInfo}`,
+            // @ts-expect-error no support for options.cause yet
+            { cause: e },
           );
         }
       }
@@ -2967,6 +3075,8 @@ function upgradeFromStoreMap(
           const moreInfo = e instanceof Error ? ` Reason: ${e.message}` : "";
           throw Error(
             `Migration failed. Could not create index ${indexDesc.name}/${indexDesc.keyPath}. ${moreInfo}`,
+            // @ts-expect-error no support for options.cause yet
+            { cause: e },
           );
         }
       }
@@ -3068,7 +3178,7 @@ export async function openStoredBackupsDatabase(
     onStoredBackupsDbUpgradeNeeded,
   );
 
-  const handle = new DbAccess(backupsDbHandle, StoredBackupStores);
+  const handle = new DbAccessImpl(backupsDbHandle, StoredBackupStores);
   return handle;
 }
 
@@ -3091,22 +3201,20 @@ export async function openTalerDatabase(
     onMetaDbUpgradeNeeded,
   );
 
-  const metaDb = new DbAccess(metaDbHandle, walletMetadataStore);
+  const metaDb = new DbAccessImpl(metaDbHandle, walletMetadataStore);
   let currentMainVersion: string | undefined;
-  await metaDb
-    .mktx((stores) => [stores.metaConfig])
-    .runReadWrite(async (tx) => {
-      const dbVersionRecord = await tx.metaConfig.get(CURRENT_DB_CONFIG_KEY);
-      if (!dbVersionRecord) {
-        currentMainVersion = TALER_WALLET_MAIN_DB_NAME;
-        await tx.metaConfig.put({
-          key: CURRENT_DB_CONFIG_KEY,
-          value: TALER_WALLET_MAIN_DB_NAME,
-        });
-      } else {
-        currentMainVersion = dbVersionRecord.value;
-      }
-    });
+  await metaDb.runReadWriteTx(["metaConfig"], async (tx) => {
+    const dbVersionRecord = await tx.metaConfig.get(CURRENT_DB_CONFIG_KEY);
+    if (!dbVersionRecord) {
+      currentMainVersion = TALER_WALLET_MAIN_DB_NAME;
+      await tx.metaConfig.put({
+        key: CURRENT_DB_CONFIG_KEY,
+        value: TALER_WALLET_MAIN_DB_NAME,
+      });
+    } else {
+      currentMainVersion = dbVersionRecord.value;
+    }
+  });
 
   if (currentMainVersion !== TALER_WALLET_MAIN_DB_NAME) {
     switch (currentMainVersion) {
@@ -3120,14 +3228,12 @@ export async function openTalerDatabase(
       case "taler-wallet-main-v9":
         // We consider this a pre-release
         // development version, no migration is done.
-        await metaDb
-          .mktx((stores) => [stores.metaConfig])
-          .runReadWrite(async (tx) => {
-            await tx.metaConfig.put({
-              key: CURRENT_DB_CONFIG_KEY,
-              value: TALER_WALLET_MAIN_DB_NAME,
-            });
+        await metaDb.runReadWriteTx(["metaConfig"], async (tx) => {
+          await tx.metaConfig.put({
+            key: CURRENT_DB_CONFIG_KEY,
+            value: TALER_WALLET_MAIN_DB_NAME,
           });
+        });
         break;
       default:
         throw Error(
@@ -3144,7 +3250,7 @@ export async function openTalerDatabase(
     onTalerDbUpgradeNeeded,
   );
 
-  const handle = new DbAccess(mainDbHandle, WalletStoresV1);
+  const handle = new DbAccessImpl(mainDbHandle, WalletStoresV1);
 
   await applyFixups(handle);
 
