@@ -25,14 +25,25 @@
  * Imports.
  */
 
-import { Logger, parseDevExperimentUri } from "@gnu-taler/taler-util";
-import { ConfigRecordKey } from "./db.js";
-import { InternalWalletState } from "./internal-wallet-state.js";
+import {
+  Logger,
+  RefreshReason,
+  TalerPreciseTimestamp,
+  encodeCrock,
+  getRandomBytes,
+  parseDevExperimentUri,
+} from "@gnu-taler/taler-util";
 import {
   HttpRequestLibrary,
   HttpRequestOptions,
   HttpResponse,
 } from "@gnu-taler/taler-util/http";
+import {
+  RefreshGroupRecord,
+  RefreshOperationStatus,
+  timestampPreciseToDb,
+} from "./db.js";
+import { WalletExecutionContext } from "./wallet.js";
 
 const logger = new Logger("dev-experiments.ts");
 
@@ -40,7 +51,7 @@ const logger = new Logger("dev-experiments.ts");
  * Apply a dev experiment to the wallet database / state.
  */
 export async function applyDevExperiment(
-  ws: InternalWalletState,
+  wex: WalletExecutionContext,
   uri: string,
 ): Promise<void> {
   logger.info(`applying dev experiment ${uri}`);
@@ -49,11 +60,34 @@ export async function applyDevExperiment(
     logger.info("unable to parse dev experiment URI");
     return;
   }
-  if (!ws.config.testing.devModeActive) {
+  if (!wex.ws.config.testing.devModeActive) {
     throw Error(
       "can't handle devmode URI (other than enable-devmode) unless devmode is active",
     );
   }
+
+  if (parsedUri.devExperimentId == "insert-pending-refresh") {
+    await wex.db.runReadWriteTx(["refreshGroups"], async (tx) => {
+      const refreshGroupId = encodeCrock(getRandomBytes(32));
+      const newRg: RefreshGroupRecord = {
+        currency: "TESTKUDOS",
+        expectedOutputPerCoin: [],
+        inputPerCoin: [],
+        oldCoinPubs: [],
+        operationStatus: RefreshOperationStatus.Pending,
+        reason: RefreshReason.Manual,
+        refreshGroupId,
+        statusPerCoin: [],
+        timestampCreated: timestampPreciseToDb(TalerPreciseTimestamp.now()),
+        timestampFinished: undefined,
+        originatingTransactionId: undefined,
+        infoPerExchange: {},
+      };
+      await tx.refreshGroups.put(newRg);
+    });
+    return;
+  }
+
   throw Error(`dev-experiment id not understood ${parsedUri.devExperimentId}`);
 }
 
@@ -65,23 +99,6 @@ export class DevExperimentHttpLib implements HttpRequestLibrary {
     this.underlyingLib = lib;
   }
 
-  get(
-    url: string,
-    opt?: HttpRequestOptions | undefined,
-  ): Promise<HttpResponse> {
-    logger.trace(`devexperiment httplib ${url}`);
-    return this.underlyingLib.fetch(url, opt);
-  }
-
-  postJson(
-    url: string,
-    body: any,
-    opt?: HttpRequestOptions | undefined,
-  ): Promise<HttpResponse> {
-    logger.trace(`devexperiment httplib ${url}`);
-    return this.underlyingLib.postJson(url, body, opt);
-  }
-
   fetch(
     url: string,
     opt?: HttpRequestOptions | undefined,
