@@ -125,7 +125,6 @@ export const Pages = {
   ctaPayTemplate: "/cta/pay/template",
   ctaRecovery: "/cta/recovery",
   ctaRefund: "/cta/refund",
-  ctaTips: "/cta/tip",
   ctaWithdraw: "/cta/withdraw",
   ctaDeposit: "/cta/deposit",
   ctaInvoiceCreate: pageDefinition<{ amount?: string }>(
@@ -146,7 +145,6 @@ const talerUriActionToPageName: {
 } = {
   [TalerUriAction.Withdraw]: "ctaWithdraw",
   [TalerUriAction.Pay]: "ctaPay",
-  [TalerUriAction.Reward]: "ctaTips",
   [TalerUriAction.Refund]: "ctaRefund",
   [TalerUriAction.PayPull]: "ctaInvoicePay",
   [TalerUriAction.PayPush]: "ctaTransferPickup",
@@ -154,8 +152,6 @@ const talerUriActionToPageName: {
   [TalerUriAction.PayTemplate]: "ctaPayTemplate",
   [TalerUriAction.WithdrawExchange]: "ctaWithdrawManual",
   [TalerUriAction.DevExperiment]: undefined,
-  [TalerUriAction.Exchange]: undefined,
-  [TalerUriAction.Auditor]: undefined,
 };
 
 export function getPathnameForTalerURI(talerUri: string): string | undefined {
@@ -260,7 +256,7 @@ export function WalletNavBar({ path }: { path?: WalletNavBarOptions }): VNode {
           <Fragment />
         )}
 
-        <EnabledBySettings name="advanceMode">
+        <EnabledBySettings name="advancedMode">
           <a href={Pages.dev} class={path === "dev" ? "active" : ""}>
             <i18n.Translate>Dev tools</i18n.Translate>
           </a>
