@@ -18,56 +18,92 @@ import {
   AmountJson,
   Amounts,
   parsePaytoUri,
-  PaytoUri,
   segwitMinAmount,
   stringifyPaytoUri,
   TranslatedString,
+  WithdrawalExchangeAccountDetails
 } from "@gnu-taler/taler-util";
-import { Fragment, h, VNode } from "preact";
-import { useEffect, useRef, useState } from "preact/hooks";
 import {
-  useAsyncAsHook,
-  useTranslationContext,
+  useTranslationContext
 } from "@gnu-taler/web-util/browser";
+import { ComponentChildren, Fragment, h, VNode } from "preact";
+import { useEffect, useRef, useState } from "preact/hooks";
 import { CopiedIcon, CopyIcon } from "../svg/index.js";
 import { Amount } from "./Amount.js";
 import { ButtonBox, TooltipLeft, WarningBox } from "./styled/index.js";
-import { WalletApiOperation } from "@gnu-taler/taler-wallet-core";
-import { useBackendContext } from "../context/backend.js";
+import { Button } from "../mui/Button.js";
 
 export interface BankDetailsProps {
-  exchangeBaseUrl: string;
   subject: string;
   amount: AmountJson;
+  accounts: WithdrawalExchangeAccountDetails[],
 }
 
 export function BankDetailsByPaytoType({
   subject,
-  exchangeBaseUrl,
   amount,
+  accounts,
 }: BankDetailsProps): VNode {
   const { i18n } = useTranslationContext();
-  const api = useBackendContext();
+  const [index, setIndex] = useState(0)
+  // const [currency, setCurrency] = useState(amount.currency)
+  if (!accounts.length) {
+    return <div>the exchange account list is empty</div>
+  }
+  const selectedAccount = accounts[index];
+  const altCurrency = selectedAccount.currencySpecification?.name
 
-  const hook = useAsyncAsHook(async () => {
-    const details = await api.wallet.call(
-      WalletApiOperation.GetExchangeDetailedInfo,
-      {
-        exchangeBaseUrl,
-      },
-    );
-    return { details };
-  });
-
-  if (!hook || hook.hasError) return <Fragment />;
-
-  const firstPayto = hook.response.details.exchange.paytoUris[0];
-  const payto = parsePaytoUri(firstPayto);
+  const payto = parsePaytoUri(selectedAccount.paytoUri);
 
   if (!payto) return <Fragment />;
-  payto.params["amount"] = Amounts.stringify(amount);
+  payto.params["amount"] = altCurrency ? selectedAccount.transferAmount! : Amounts.stringify(amount);
   payto.params["message"] = subject;
 
+
+  function Frame({ title, children }: { title: TranslatedString, children: ComponentChildren }): VNode {
+    return <section
+      style={{
+        textAlign: "left",
+        border: "solid 1px black",
+        padding: 8,
+        borderRadius: 4,
+      }}
+    >
+      <div style={{ display: "flex", width: "100%", justifyContent: "space-between" }}>
+        <p style={{ marginTop: 0 }}>
+          {title}
+        </p>
+        <div>
+
+        </div>
+      </div>
+
+      {children}
+
+      {accounts.length > 1 ?
+        <Fragment>
+          {accounts.map((ac, acIdx) => {
+            return <Button variant={acIdx === index ? "contained" : "outlined"}
+              onClick={async () => {
+                setIndex(acIdx)
+              }}
+            >
+              <i18n.Translate>Account #{acIdx+1} ({ac.currencySpecification?.name ?? amount.currency})</i18n.Translate>
+            </Button>
+          })}
+
+          {/* <Button variant={currency === altCurrency ? "contained" : "outlined"}
+            onClick={async () => {
+              setCurrency(altCurrency)
+            }}
+          >
+            <i18n.Translate>{altCurrency}</i18n.Translate>
+          </Button> */}
+        </Fragment>
+        : undefined}
+    </section>
+  }
+
   if (payto.isKnown && payto.targetType === "bitcoin") {
     const min = segwitMinAmount(amount.currency);
     const addrs = payto.segwitAddrs.map(
@@ -76,17 +112,7 @@ export function BankDetailsByPaytoType({
     addrs.unshift(`${payto.targetPath} ${Amounts.stringifyValue(amount)}`);
     const copyContent = addrs.join("\n");
     return (
-      <section
-        style={{
-          textAlign: "left",
-          border: "solid 1px black",
-          padding: 8,
-          borderRadius: 4,
-        }}
-      >
-        <p style={{ marginTop: 0 }}>
-          <i18n.Translate>Bitcoin transfer details</i18n.Translate>
-        </p>
+      <Frame title={i18n.str`Bitcoin transfer details`}>
         <p>
           <i18n.Translate>
             The exchange need a transaction with 3 output, one output is the
@@ -126,7 +152,7 @@ export function BankDetailsByPaytoType({
             BTC, else you have to change the base unit to BTC
           </i18n.Translate>
         </p>
-      </section>
+      </Frame>
     );
   }
 
@@ -146,34 +172,37 @@ export function BankDetailsByPaytoType({
     </Fragment>
   ) : undefined;
 
-  const receiver = payto.params["receiver"] || undefined;
+  const receiver = payto.params["receiver-name"] || payto.params["receiver"] || undefined;
   return (
-    <section>
-      <div
-        style={{
-          textAlign: "left",
-          border: "solid 1px black",
-          padding: 8,
-          borderRadius: 4,
-        }}
-      >
-        <p style={{ marginTop: 0 }}>
-          <i18n.Translate>Bank transfer details</i18n.Translate>
-        </p>
-        <table>
+    <Frame title={i18n.str`Bank transfer details`}>
+      <table>
+        <tbody>
           {accountPart}
+          <Row name={i18n.str`Subject`} value={subject} literal />
+
           <Row
             name={i18n.str`Amount`}
-            value={<Amount value={amount} hideCurrency />}
+            value={<Amount value={altCurrency ? selectedAccount.transferAmount! : amount} hideCurrency />}
           />
-          <Row name={i18n.str`Subject`} value={subject} literal />
+
           {receiver ? (
             <Row name={i18n.str`Receiver name`} value={receiver} />
           ) : undefined}
-        </table>
-      </div>
-      <table>
-        <tbody>
+
+          <tr>
+            <td colSpan={3}>
+              <WarningBox style={{ margin: 0 }}>
+                <span>
+                  <i18n.Translate>
+                    Make sure ALL data is correct, including the subject; otherwise, the money will not
+                    arrive in this wallet. You can use the copy buttons (<CopyIcon />) to prevent typing errors
+                    or the "payto://" URI below to copy just one value.
+                  </i18n.Translate>
+                </span>
+              </WarningBox>
+            </td>
+          </tr>
+
           <tr>
             <td>
               <pre>
@@ -198,15 +227,7 @@ export function BankDetailsByPaytoType({
           </tr>
         </tbody>
       </table>
-      <p>
-        <WarningBox>
-          <i18n.Translate>
-            Make sure to use the correct subject, otherwise the money will not
-            arrive in this wallet.
-          </i18n.Translate>
-        </WarningBox>
-      </p>
-    </section>
+    </Frame>
   );
 }
 
