@@ -167,6 +167,7 @@ export function ErrorView({
     <Modal title="Full detail" onClose={hideHandler}>
       <ErrorAlertView
         error={alertFromError(
+          i18n,
           i18n.str`Could not load purchase proposal details`,
           error,
           { proposalId },
@@ -383,14 +384,6 @@ export function ShowView({ contractTerms, hideHandler }: States.Show): VNode {
               <Amount value={contractTerms.maxDepositFee} />
             </td>
           </tr>
-          <tr>
-            <td>
-              <i18n.Translate>Max fee</i18n.Translate>
-            </td>
-            <td>
-              <Amount value={contractTerms.maxWireFee} />
-            </td>
-          </tr>
           <tr>
             <td>
               <i18n.Translate>Minimum age</i18n.Translate>
