@@ -14,17 +14,14 @@
  GNU Taler; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
  */
 
-import { Amounts } from "@gnu-taler/taler-util";
+import { useTranslationContext } from "@gnu-taler/web-util/browser";
 import { format } from "date-fns";
 import { Fragment, h, VNode } from "preact";
-import { LogoHeader } from "../../components/LogoHeader.js";
 import { Part } from "../../components/Part.js";
 import {
-  SubTitle,
-  SvgIcon,
-  WalletAction,
+  SvgIcon
 } from "../../components/styled/index.js";
-import { useTranslationContext } from "@gnu-taler/web-util/browser";
+import { TermsOfService } from "../../components/TermsOfService/index.js";
 import { Button } from "../../mui/Button.js";
 import { TextField } from "../../mui/TextField.js";
 import editIcon from "../../svg/edit_24px.inline.svg";
@@ -80,13 +77,13 @@ export function ReadyView({
               }}
             >
               <i18n.Translate>Exchange</i18n.Translate>
-              <Button onClick={doSelectExchange.onClick} variant="text">
+              {/* <Button onClick={doSelectExchange.onClick} variant="text">
                 <SvgIcon
                   title="Edit"
                   dangerouslySetInnerHTML={{ __html: editIcon }}
                   color="black"
                 />
-              </Button>
+              </Button> */}
             </div>
           }
           text={<ExchangeDetails exchange={exchangeUrl} />}
@@ -151,9 +148,11 @@ export function ReadyView({
         />
       </section>
       <section>
-        <Button onClick={create.onClick} variant="contained" color="success">
-          <i18n.Translate>Create</i18n.Translate>
-        </Button>
+        <TermsOfService key="terms" exchangeUrl={exchangeUrl} >
+          <Button onClick={create.onClick} variant="contained" color="success">
+            <i18n.Translate>Create</i18n.Translate>
+          </Button>
+        </TermsOfService>
       </section>
     </Fragment>
   );
