@@ -64,7 +64,8 @@ export function useComponentState({
     return {
       status: "error",
       error: alertFromError(
-        i18n.str`Could not load the status of the term of service`,
+        i18n,
+        i18n.str`Could not load the transfer payment status`,
         hook,
       ),
     };
