@@ -78,7 +78,7 @@ export function BaseView(state: SupportedStates): VNode {
                 contractTermsHash: "",
                 // products: contractTerms.products!,
               }}
-              proposalId={state.payStatus.proposalId}
+              proposalId={state.payStatus.transactionId}
             />
           }
           kind="neutral"
