@@ -50,7 +50,8 @@ export function useComponentState({
     return {
       status: "error",
       error: alertFromError(
-        i18n.str`Could not load the status of the term of service`,
+        i18n,
+        i18n.str`Could not load the invoice payment status`,
         hook,
       ),
     };
@@ -61,6 +62,7 @@ export function useComponentState({
     peerPushCreditId,
     amountEffective,
     amountRaw,
+    exchangeBaseUrl
   } = hook.response;
 
   const effective = Amounts.parseOrThrow(amountEffective);
@@ -80,6 +82,7 @@ export function useComponentState({
   return {
     status: "ready",
     effective,
+    exchangeBaseUrl,
     raw,
     error: undefined,
     accept: {
