@@ -25,12 +25,14 @@ import {
   TransferPickupDetails,
 } from "../../wallet/Transaction.js";
 import { State } from "./index.js";
+import { TermsOfService } from "../../components/TermsOfService/index.js";
 
 export function ReadyView({
   accept,
   summary,
   expiration,
   effective,
+  exchangeBaseUrl,
   raw,
 }: State.Ready): VNode {
   const { i18n } = useTranslationContext();
@@ -55,11 +57,13 @@ export function ReadyView({
         />
       </section>
       <section>
-        <Button variant="contained" color="success" onClick={accept.onClick}>
-          <i18n.Translate>
-            Receive &nbsp; {<Amount value={effective} />}
-          </i18n.Translate>
-        </Button>
+        <TermsOfService key="terms" exchangeUrl={exchangeBaseUrl} >
+          <Button variant="contained" color="success" onClick={accept.onClick}>
+            <i18n.Translate>
+              Receive &nbsp; {<Amount value={effective} />}
+            </i18n.Translate>
+          </Button>
+        </TermsOfService>
       </section>
     </Fragment>
   );
