@@ -17,7 +17,9 @@
 import {
   AmountJson,
   AmountString,
+  CurrencySpecification,
   ExchangeListItem,
+  WithdrawalExchangeAccountDetails,
 } from "@gnu-taler/taler-util";
 import { Loading } from "../../components/Loading.js";
 import { State as SelectExchangeState } from "../../hooks/useSelectedExchange.js";
@@ -36,7 +38,7 @@ import { ErrorAlertView } from "../../components/CurrentAlerts.js";
 import { ErrorAlert } from "../../context/alert.js";
 import { ExchangeSelectionPage } from "../../wallet/ExchangeSelection/index.js";
 import { NoExchangesView } from "../../wallet/ExchangeSelection/views.js";
-import { SelectAmountView, SuccessView } from "./views.js";
+import { FinalStateOperation, SelectAmountView, SuccessView } from "./views.js";
 
 export interface PropsFromURI {
   talerWithdrawUri: string | undefined;
@@ -58,6 +60,7 @@ export type State =
   | SelectExchangeState.NoExchangeFound
   | SelectExchangeState.Selecting
   | State.SelectAmount
+  | State.AlreadyCompleted
   | State.Success;
 
 export namespace State {
@@ -78,6 +81,12 @@ export namespace State {
     amount: AmountFieldHandler;
     currency: string;
   }
+  export interface AlreadyCompleted {
+    status: "already-completed";
+    operationState: "confirmed" | "aborted" | "selected";
+    confirmTransferUrl?: string,
+    error: undefined;
+  }
 
   export type Success = {
     status: "success";
@@ -92,11 +101,18 @@ export namespace State {
     doWithdrawal: ButtonHandler;
     doSelectExchange: ButtonHandler;
 
+    chooseCurrencies: string[];
+    selectedCurrency: string;
+    changeCurrency: (s: string) => void;
+    conversionInfo: {
+      spec: CurrencySpecification,
+      amount: AmountJson,
+    } | undefined;
+
     ageRestriction?: SelectFieldHandler;
 
     talerWithdrawUri?: string;
     cancel: () => Promise<void>;
-    onTosUpdate: () => void;
   };
 }
 
@@ -107,6 +123,7 @@ const viewMapping: StateViewMap<State> = {
   "no-exchange-found": NoExchangesView,
   "selecting-exchange": ExchangeSelectionPage,
   success: SuccessView,
+  "already-completed": FinalStateOperation,
 };
 
 export const WithdrawPageFromURI = compose(
