@@ -20,15 +20,15 @@ import {
   Amounts,
   ExchangeFullDetails,
   ExchangeListItem,
-  ExchangeTosStatus,
+  NotificationType,
   TalerError,
   parseWithdrawExchangeUri,
 } from "@gnu-taler/taler-util";
 import { WalletApiOperation } from "@gnu-taler/taler-wallet-core";
-import { useEffect, useState } from "preact/hooks";
+import { useTranslationContext } from "@gnu-taler/web-util/browser";
+import { useCallback, useEffect, useState } from "preact/hooks";
 import { alertFromError, useAlertContext } from "../../context/alert.js";
 import { useBackendContext } from "../../context/backend.js";
-import { useTranslationContext } from "@gnu-taler/web-util/browser";
 import { useAsyncAsHook } from "../../hooks/useAsyncAsHook.js";
 import { useSelectedExchange } from "../../hooks/useSelectedExchange.js";
 import { RecursiveState } from "../../utils/index.js";
@@ -79,6 +79,7 @@ export function useComponentStateFromParams({
     return {
       status: "error",
       error: alertFromError(
+        i18n,
         i18n.str`Could not load the list of exchanges`,
         uriInfoHook,
       ),
@@ -140,8 +141,8 @@ export function useComponentStateFromParams({
         confirm: {
           onClick: isValid
             ? pushAlertOnError(async () => {
-              onAmountChanged(Amounts.stringify(amount));
-            })
+                onAmountChanged(Amounts.stringify(amount));
+              })
             : undefined,
         },
         amount: {
@@ -178,7 +179,6 @@ export function useComponentStateFromParams({
 
   return () =>
     exchangeSelectionState(
-      uriInfoHook.retry,
       doManualWithdraw,
       cancel,
       onSuccess,
@@ -209,23 +209,54 @@ export function useComponentStateFromURI({
       WalletApiOperation.GetWithdrawalDetailsForUri,
       {
         talerWithdrawUri,
+        notifyChangeFromPendingTimeoutMs: 30 * 1000,
       },
     );
-    const { amount, defaultExchangeBaseUrl, possibleExchanges } = uriInfo;
+    const {
+      amount,
+      defaultExchangeBaseUrl,
+      possibleExchanges,
+      operationId,
+      confirmTransferUrl,
+      status,
+    } = uriInfo;
+    const transaction = await api.wallet.call(
+      WalletApiOperation.GetWithdrawalTransactionByUri,
+      { talerWithdrawUri },
+    );
     return {
       talerWithdrawUri,
+      operationId,
+      status,
+      transaction,
+      confirmTransferUrl,
       amount: Amounts.parseOrThrow(amount),
       thisExchange: defaultExchangeBaseUrl,
       exchanges: possibleExchanges,
     };
   });
 
+  const readyToListen = uriInfoHook && !uriInfoHook.hasError;
+
+  useEffect(() => {
+    if (!uriInfoHook) {
+      return;
+    }
+    return api.listener.onUpdateNotification(
+      [NotificationType.WithdrawalOperationTransition],
+      (asd) => {
+        uriInfoHook.retry();
+      },
+    );
+  }, [readyToListen]);
+
   if (!uriInfoHook) return { status: "loading", error: undefined };
 
   if (uriInfoHook.hasError) {
     return {
       status: "error",
       error: alertFromError(
+        i18n,
         i18n.str`Could not load info from URI`,
         uriInfoHook,
       ),
@@ -258,9 +289,20 @@ export function useComponentStateFromURI({
     };
   }
 
-  return () =>
-    exchangeSelectionState(
-      uriInfoHook.retry,
+  if (uriInfoHook.response.status !== "pending") {
+    if (uriInfoHook.response.transaction) {
+      onSuccess(uriInfoHook.response.transaction.transactionId);
+    }
+    return {
+      status: "already-completed",
+      operationState: uriInfoHook.response.status,
+      confirmTransferUrl: uriInfoHook.response.confirmTransferUrl,
+      error: undefined,
+    };
+  }
+
+  return useCallback(() => {
+    return exchangeSelectionState(
       doManagedWithdraw,
       cancel,
       onSuccess,
@@ -269,6 +311,7 @@ export function useComponentStateFromURI({
       exchangeList,
       defaultExchange,
     );
+  }, []);
 }
 
 type ManualOrManagedWithdrawFunction = (
@@ -277,7 +320,6 @@ type ManualOrManagedWithdrawFunction = (
 ) => Promise<{ transactionId: string; confirmTransferUrl: string | undefined }>;
 
 function exchangeSelectionState(
-  onTosUpdate: () => void,
   doWithdraw: ManualOrManagedWithdrawFunction,
   cancel: () => Promise<void>,
   onSuccess: (txid: string) => Promise<void>,
@@ -297,15 +339,18 @@ function exchangeSelectionState(
     return selectedExchange;
   }
 
-  return () => {
+  return useCallback(():
+    | State.Success
+    | State.LoadingUriError
+    | State.Loading => {
     const { i18n } = useTranslationContext();
     const { pushAlertOnError } = useAlertContext();
     const [ageRestricted, setAgeRestricted] = useState(0);
     const currentExchange = selectedExchange.selected;
-    const tosNeedToBeAccepted =
-      currentExchange.tosStatus == ExchangeTosStatus.Pending ||
-      currentExchange.tosStatus == ExchangeTosStatus.Proposed;
 
+    const [selectedCurrency, setSelectedCurrency] = useState<string>(
+      chosenAmount.currency,
+    );
     /**
      * With the exchange and amount, ask the wallet the information
      * about the withdrawal
@@ -328,6 +373,7 @@ function exchangeSelectionState(
       return {
         amount: withdrawAmount,
         ageRestrictionOptions: info.ageRestrictionOptions,
+        accounts: info.withdrawalAccountsList,
       };
     }, []);
 
@@ -363,6 +409,7 @@ function exchangeSelectionState(
       return {
         status: "error",
         error: alertFromError(
+          i18n,
           i18n.str`Could not load the withdrawal details`,
           amountHook,
         ),
@@ -392,33 +439,56 @@ function exchangeSelectionState(
     //TODO: calculate based on exchange info
     const ageRestriction = ageRestrictionEnabled
       ? {
-        list: ageRestrictionOptions,
-        value: String(ageRestricted),
-        onChange: pushAlertOnError(async (v: string) =>
-          setAgeRestricted(parseInt(v, 10)),
-        ),
-      }
+          list: ageRestrictionOptions,
+          value: String(ageRestricted),
+          onChange: pushAlertOnError(async (v: string) =>
+            setAgeRestricted(parseInt(v, 10)),
+          ),
+        }
       : undefined;
 
+    const altCurrencies = amountHook.response.accounts
+      .filter((a) => !!a.currencySpecification)
+      .map((a) => a.currencySpecification!.name);
+    const chooseCurrencies =
+      altCurrencies.length === 0
+        ? []
+        : [toBeReceived.currency, ...altCurrencies];
+    const convAccount = amountHook.response.accounts.find((c) => {
+      return (
+        c.currencySpecification &&
+        c.currencySpecification.name === selectedCurrency
+      );
+    });
+    const conversionInfo = !convAccount
+      ? undefined
+      : {
+          spec: convAccount.currencySpecification!,
+          amount: Amounts.parseOrThrow(convAccount.transferAmount!),
+        };
+
     return {
       status: "success",
       error: undefined,
       doSelectExchange: selectedExchange.doSelect,
       currentExchange,
       toBeReceived,
+      chooseCurrencies,
+      selectedCurrency,
+      changeCurrency: (s) => {
+        setSelectedCurrency(s);
+      },
+      conversionInfo,
       withdrawalFee,
       chosenAmount,
       talerWithdrawUri,
       ageRestriction,
       doWithdrawal: {
-        onClick:
-          doingWithdraw || tosNeedToBeAccepted
-            ? undefined
-            : pushAlertOnError(doWithdrawAndCheckError),
-        error: withdrawError,
+        onClick: doingWithdraw
+          ? undefined
+          : pushAlertOnError(doWithdrawAndCheckError),
       },
-      onTosUpdate,
       cancel,
     };
-  };
+  }, []);
 }
