@@ -19,11 +19,11 @@
  * @author Sebastian Javier Marchano (sebasjm)
  */
 
-import { ExchangeListItem } from "@gnu-taler/taler-util";
+import { CurrencySpecification, ExchangeListItem } from "@gnu-taler/taler-util";
 import * as tests from "@gnu-taler/web-util/testing";
 import { nullFunction } from "../../mui/handlers.js";
 // import { TermsState } from "../../utils/index.js";
-import { SuccessView } from "./views.js";
+import { SuccessView, FinalStateOperation } from "./views.js";
 
 export default {
   title: "withdraw",
@@ -64,8 +64,26 @@ export const TermsOfServiceNotYetLoaded = tests.createExample(SuccessView, {
     fraction: 0,
     value: 1,
   },
+  chooseCurrencies: [],
 });
 
+export const AlreadyAborted = tests.createExample(FinalStateOperation, {
+  error: undefined,
+  status: "already-completed",
+  operationState: "aborted"
+});
+export const AlreadySelected = tests.createExample(FinalStateOperation, {
+  error: undefined,
+  status: "already-completed",
+  operationState: "selected"
+});
+export const AlreadyConfirmed = tests.createExample(FinalStateOperation, {
+  error: undefined,
+  status: "already-completed",
+  operationState: "confirmed"
+});
+
+
 export const WithSomeFee = tests.createExample(SuccessView, {
   error: undefined,
   status: "success",
@@ -90,6 +108,7 @@ export const WithSomeFee = tests.createExample(SuccessView, {
     value: 1,
   },
   doSelectExchange: {},
+  chooseCurrencies: [],
 });
 
 export const WithoutFee = tests.createExample(SuccessView, {
@@ -116,6 +135,7 @@ export const WithoutFee = tests.createExample(SuccessView, {
     fraction: 0,
     value: 2,
   },
+  chooseCurrencies: [],
 });
 
 export const EditExchangeUntouched = tests.createExample(SuccessView, {
@@ -142,6 +162,7 @@ export const EditExchangeUntouched = tests.createExample(SuccessView, {
     fraction: 0,
     value: 2,
   },
+  chooseCurrencies: [],
 });
 
 export const EditExchangeModified = tests.createExample(SuccessView, {
@@ -168,6 +189,7 @@ export const EditExchangeModified = tests.createExample(SuccessView, {
     fraction: 0,
     value: 2,
   },
+  chooseCurrencies: [],
 });
 
 export const WithAgeRestriction = tests.createExample(SuccessView, {
@@ -195,4 +217,111 @@ export const WithAgeRestriction = tests.createExample(SuccessView, {
     fraction: 0,
     value: 2,
   },
+  chooseCurrencies: [],
+});
+
+export const WithAlternateCurrenciesNETZBON = tests.createExample(SuccessView, {
+  error: undefined,
+  status: "success",
+  chosenAmount: {
+    currency: "NETZBON",
+    value: 2,
+    fraction: 10000000,
+  },
+  chooseCurrencies: ["NETZBON", "EUR"],
+  selectedCurrency: "NETZBON",
+  doWithdrawal: { onClick: nullFunction },
+  currentExchange: {
+    exchangeBaseUrl: "https://exchange.netzbon.ch",
+    tos: {},
+  } as Partial<ExchangeListItem> as any,
+  withdrawalFee: {
+    currency: "NETZBON",
+    fraction: 10000000,
+    value: 1,
+  },
+  doSelectExchange: {},
+  toBeReceived: {
+    currency: "NETZBON",
+    fraction: 0,
+    value: 1,
+  },
+});
+
+export const WithAlternateCurrenciesEURO = tests.createExample(SuccessView, {
+  error: undefined,
+  status: "success",
+  chosenAmount: {
+    currency: "NETZBON",
+    value: 2,
+    fraction: 10000000,
+  },
+  chooseCurrencies: ["NETZBON", "EUR"],
+  selectedCurrency: "EUR",
+  changeCurrency: () => { },
+  conversionInfo: {
+    spec: {
+      name: "EUR"
+    } as CurrencySpecification,
+    amount: {
+      currency: "EUR",
+      fraction: 10000000,
+      value: 1,
+    }
+  },
+  doWithdrawal: { onClick: nullFunction },
+  currentExchange: {
+    exchangeBaseUrl: "https://exchange.netzbon.ch",
+    tos: {},
+  } as Partial<ExchangeListItem> as any,
+  withdrawalFee: {
+    currency: "NETZBON",
+    fraction: 10000000,
+    value: 1,
+  },
+  doSelectExchange: {},
+  toBeReceived: {
+    currency: "NETZBON",
+    fraction: 0,
+    value: 1,
+  },
+});
+
+export const WithAlternateCurrenciesEURO11 = tests.createExample(SuccessView, {
+  error: undefined,
+  status: "success",
+  chosenAmount: {
+    currency: "NETZBON",
+    value: 2,
+    fraction: 10000000,
+  },
+  chooseCurrencies: ["NETZBON", "EUR"],
+  selectedCurrency: "EUR",
+  changeCurrency: () => { },
+  conversionInfo: {
+    spec: {
+      name: "EUR"
+    } as CurrencySpecification,
+    amount: {
+      currency: "EUR",
+      fraction: 10000000,
+      value: 2,
+    }
+  },
+  doWithdrawal: { onClick: nullFunction },
+  currentExchange: {
+    exchangeBaseUrl: "https://exchange.netzbon.ch",
+    tos: {},
+  } as Partial<ExchangeListItem> as any,
+  withdrawalFee: {
+    currency: "NETZBON",
+    fraction: 10000000,
+    value: 1,
+  },
+  doSelectExchange: {},
+  toBeReceived: {
+    currency: "NETZBON",
+    fraction: 0,
+    value: 1,
+  },
 });
