@@ -14,31 +14,58 @@
  GNU Taler; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
  */
 
-import { Amounts, ExchangeTosStatus } from "@gnu-taler/taler-util";
-import { Fragment, h, VNode } from "preact";
+import { useTranslationContext } from "@gnu-taler/web-util/browser";
+import { Fragment, VNode, h } from "preact";
 import { useState } from "preact/hooks";
 import { Amount } from "../../components/Amount.js";
+import { AmountField } from "../../components/AmountField.js";
 import { Part } from "../../components/Part.js";
 import { QR } from "../../components/QR.js";
 import { SelectList } from "../../components/SelectList.js";
-import { Input, LinkSuccess, SvgIcon } from "../../components/styled/index.js";
 import { TermsOfService } from "../../components/TermsOfService/index.js";
-import { useTranslationContext } from "@gnu-taler/web-util/browser";
+import { Input, LinkSuccess, SvgIcon, WarningBox } from "../../components/styled/index.js";
 import { Button } from "../../mui/Button.js";
+import { Grid } from "../../mui/Grid.js";
 import editIcon from "../../svg/edit_24px.inline.svg";
 import {
   ExchangeDetails,
-  getAmountWithFee,
   WithdrawDetails,
+  getAmountWithFee,
 } from "../../wallet/Transaction.js";
 import { State } from "./index.js";
-import { Grid } from "../../mui/Grid.js";
-import { AmountField } from "../../components/AmountField.js";
+import { EnabledBySettings } from "../../components/EnabledBySettings.js";
+
+export function FinalStateOperation(state: State.AlreadyCompleted): VNode {
+  const { i18n } = useTranslationContext();
+
+  switch (state.operationState) {
+    case "confirmed": return <WarningBox>
+      <div style={{ justifyContent: "center", lineHeight: "25px" }}>
+        <i18n.Translate>This operation has already been completed by another wallet.</i18n.Translate>
+      </div>
+    </WarningBox>
+    case "aborted": return <WarningBox>
+      <div style={{ justifyContent: "center", lineHeight: "25px" }}>
+        <i18n.Translate>This operation has already been aborted</i18n.Translate>
+      </div>
+    </WarningBox>
+    case "selected": return <WarningBox>
+      <div style={{ justifyContent: "center", lineHeight: "25px" }}>
+        <i18n.Translate>This operation has already been used by another wallet.</i18n.Translate>
+      </div>
+      <div style={{ justifyContent: "center", lineHeight: "25px" }}>
+        <i18n.Translate>It can be confirmed in</i18n.Translate>&nbsp;<a target="_bank" rel="noreferrer" href={state.confirmTransferUrl}>
+          <i18n.Translate>this page</i18n.Translate>
+        </a>
+      </div>
+    </WarningBox>
+  }
+}
 
 export function SuccessView(state: State.Success): VNode {
   const { i18n } = useTranslationContext();
-  const currentTosVersionIsAccepted =
-    state.currentExchange.tosStatus === ExchangeTosStatus.Accepted;
+  // const currentTosVersionIsAccepted =
+  //   state.currentExchange.tosStatus === ExchangeTosStatus.Accepted;
   return (
     <Fragment>
       <section style={{ textAlign: "left" }}>
@@ -51,13 +78,15 @@ export function SuccessView(state: State.Success): VNode {
               }}
             >
               <i18n.Translate>Exchange</i18n.Translate>
-              <Button onClick={state.doSelectExchange.onClick} variant="text">
-                <SvgIcon
-                  title="Edit"
-                  dangerouslySetInnerHTML={{ __html: editIcon }}
-                  color="black"
-                />
-              </Button>
+              <EnabledBySettings name="showExchangeManagement">
+                <Button onClick={state.doSelectExchange.onClick} variant="text">
+                  <SvgIcon
+                    title="Edit"
+                    dangerouslySetInnerHTML={{ __html: editIcon }}
+                    color="black"
+                  />
+                </Button>
+              </EnabledBySettings>
             </div>
           }
           text={
@@ -66,10 +95,27 @@ export function SuccessView(state: State.Success): VNode {
           kind="neutral"
           big
         />
+        {state.chooseCurrencies.length > 0 ?
+          <Fragment>
+            <p>
+              {state.chooseCurrencies.map(currency => {
+                return <Button variant={currency === state.selectedCurrency ? "contained" : "outlined"}
+                  onClick={async () => {
+                    state.changeCurrency(currency)
+                  }}
+                >
+                  {currency}
+                </Button>
+              })}
+            </p>
+          </Fragment>
+          : <Fragment />}
+
         <Part
           title={i18n.str`Details`}
           text={
             <WithdrawDetails
+              conversion={state.conversionInfo?.amount}
               amount={getAmountWithFee(
                 state.toBeReceived,
                 state.chosenAmount,
@@ -92,7 +138,8 @@ export function SuccessView(state: State.Success): VNode {
       </section>
 
       <section>
-        {currentTosVersionIsAccepted ? (
+        {/* <div> */}
+        <TermsOfService exchangeUrl={state.currentExchange.exchangeBaseUrl}>
           <Button
             variant="contained"
             color="success"
@@ -103,12 +150,21 @@ export function SuccessView(state: State.Success): VNode {
               Withdraw &nbsp; <Amount value={state.toBeReceived} />
             </i18n.Translate>
           </Button>
-        ) : (
-          <TermsOfService
-            exchangeUrl={state.currentExchange.exchangeBaseUrl}
-            onChange={state.onTosUpdate}
-          />
-        )}
+        </TermsOfService>
+        {/* </div>
+        <div style={{ marginTop: 20 }}>
+          <Button
+            variant="text"
+            color="success"
+
+            disabled={!state.doAbort.onClick}
+            onClick={state.doAbort.onClick}
+          >
+            <i18n.Translate>
+              Cancel
+            </i18n.Translate>
+          </Button>
+        </div> */}
       </section>
       {state.talerWithdrawUri ? (
         <WithdrawWithMobile talerWithdrawUri={state.talerWithdrawUri} />
