@@ -16,11 +16,10 @@
 
 import {
   Logger,
-  TalerErrorCode,
-  TalerUriAction,
   TalerError,
-  parseTalerUri,
+  TalerErrorCode,
   TalerUri,
+  TalerUriAction,
   stringifyTalerUri,
 } from "@gnu-taler/taler-util";
 import { WalletOperations } from "@gnu-taler/taler-wallet-core";
@@ -28,11 +27,11 @@ import { BackgroundOperations } from "../wxApi.js";
 import {
   BackgroundPlatformAPI,
   CrossBrowserPermissionsApi,
+  ExtensionNotificationType,
   ForegroundPlatformAPI,
   MessageFromBackend,
   MessageFromFrontend,
   MessageResponse,
-  Permissions,
   Settings,
   defaultSettings,
 } from "./api.js";
@@ -43,7 +42,9 @@ const api: BackgroundPlatformAPI & ForegroundPlatformAPI = {
   findTalerUriInActiveTab,
   findTalerUriInClipboard,
   getPermissionsApi,
+  runningOnPrivateMode,
   getWalletWebExVersion,
+  triggerWalletEvent,
   listenToWalletBackground,
   notifyWhenAppIsReady,
   openWalletPage,
@@ -66,19 +67,24 @@ export default api;
 
 const logger = new Logger("chrome.ts");
 
-async function getSettingsFromStorage(): Promise<Settings> {
-  const data = await chrome.storage.local.get("wallet-settings");
-  if (!data) return defaultSettings;
-  const settings = data["wallet-settings"];
-  if (!settings) return defaultSettings;
+const WALLET_STORAGE_KEY = "wallet-settings";
+
+function jsonParseOrDefault(unparsed: any, def: any) {
+  if (!unparsed) return def;
   try {
-    const parsed = JSON.parse(settings);
-    return parsed;
+    return JSON.parse(unparsed);
   } catch (e) {
-    return defaultSettings;
+    return def;
   }
 }
 
+async function getSettingsFromStorage(): Promise<Settings> {
+  const data = await chrome.storage.local.get(WALLET_STORAGE_KEY);
+  if (!data) return defaultSettings;
+  const settings = data[WALLET_STORAGE_KEY];
+  return jsonParseOrDefault(settings, defaultSettings);
+}
+
 function keepAlive(callback: any): void {
   if (extensionIsManifestV3()) {
     chrome.alarms.create("wallet-worker", { periodInMinutes: 1 });
@@ -96,7 +102,6 @@ function isFirefox(): boolean {
   return false;
 }
 
-
 export function containsClipboardPermissions(): Promise<boolean> {
   return new Promise((res, rej) => {
     res(false);
@@ -123,8 +128,6 @@ export async function requestClipboardPermissions(): Promise<boolean> {
   });
 }
 
-
-
 export function removeClipboardPermissions(): Promise<boolean> {
   return new Promise((res, rej) => {
     res(true);
@@ -138,18 +141,8 @@ export function removeClipboardPermissions(): Promise<boolean> {
   });
 }
 
-function addPermissionsListener(
-  callback: (p: Permissions, lastError?: string) => void,
-): void {
-  chrome.permissions.onAdded.addListener((perm: Permissions) => {
-    const lastError = chrome.runtime.lastError?.message;
-    callback(perm, lastError);
-  });
-}
-
 function getPermissionsApi(): CrossBrowserPermissionsApi {
   return {
-    addPermissionsListener,
     requestClipboardPermissions,
     removeClipboardPermissions,
     containsClipboardPermissions,
@@ -178,7 +171,6 @@ function openWalletURIFromPopup(uri: TalerUri): void {
   // the target pathname should handle what happens if the endpoint is not there
   // like "trying to open from popup but this uri is not handled"
 
-  encodeURIComponent;
   let url: string | undefined = undefined;
   switch (uri.type) {
     case TalerUriAction.WithdrawExchange:
@@ -201,11 +193,6 @@ function openWalletURIFromPopup(uri: TalerUri): void {
         `static/wallet.html#/cta/pay?talerUri=${encodeURIComponent(talerUri)}`,
       );
       break;
-    case TalerUriAction.Reward:
-      url = chrome.runtime.getURL(
-        `static/wallet.html#/cta/tip?talerUri=${encodeURIComponent(talerUri)}`,
-      );
-      break;
     case TalerUriAction.Refund:
       url = chrome.runtime.getURL(
         `static/wallet.html#/cta/refund?talerUri=${encodeURIComponent(
@@ -237,12 +224,6 @@ function openWalletURIFromPopup(uri: TalerUri): void {
     case TalerUriAction.DevExperiment:
       logger.warn(`taler://dev-experiment URIs are not allowed in headers`);
       return;
-    case TalerUriAction.Exchange:
-      logger.warn(`taler://exchange not yet supported`);
-      return;
-    case TalerUriAction.Auditor:
-      logger.warn(`taler://auditor not yet supported`);
-      return;
     default: {
       const error: never = uri;
       logger.warn(
@@ -267,7 +248,6 @@ function openWalletPageFromPopup(page: string): void {
   chrome.tabs.create({ active: true, url }, () => {
     window.close();
   });
-
 }
 function openNewURLFromPopup(url: URL): void {
   // const url = chrome.runtime.getURL(`/static/wallet.html#${page}`);
@@ -286,7 +266,8 @@ let nextMessageIndex = 0;
 async function sendMessageToBackground<
   Op extends WalletOperations | BackgroundOperations,
 >(message: MessageFromFrontend<Op>): Promise<MessageResponse> {
-  const messageWithId = { ...message, id: `id_${nextMessageIndex++ % 1000}` };
+  nextMessageIndex = (nextMessageIndex + 1) % (Number.MAX_SAFE_INTEGER - 100);
+  const messageWithId = { ...message, id: `id_${nextMessageIndex}` };
 
   return new Promise<any>((resolve, reject) => {
     logger.trace("send operation to the wallet background", message);
@@ -330,6 +311,17 @@ function listenToWalletBackground(listener: (m: any) => void): () => void {
 
 const allPorts: chrome.runtime.Port[] = [];
 
+function triggerWalletEvent(message: MessageFromBackend): void {
+  for (const notif of allPorts) {
+    // const message: MessageFromBackend = { type: msg.type };
+    try {
+      notif.postMessage(message);
+    } catch (e) {
+      logger.error("error posting a message", e);
+    }
+  }
+}
+
 function sendMessageToAllChannels(message: MessageFromBackend): void {
   for (const notif of allPorts) {
     // const message: MessageFromBackend = { type: msg.type };
@@ -359,6 +351,20 @@ function registerAllIncomingConnections(): void {
       logger.error("error trying to save incoming connection", e);
     }
   });
+  chrome.storage.onChanged.addListener((event) => {
+    if (event[WALLET_STORAGE_KEY]) {
+      sendMessageToAllChannels({
+        type: "web-extension",
+        notification: {
+          type: ExtensionNotificationType.SettingsChange,
+          currentValue: jsonParseOrDefault(
+            event[WALLET_STORAGE_KEY],
+            defaultSettings,
+          ),
+        },
+      });
+    }
+  });
 }
 
 function listenToAllChannels(
@@ -405,7 +411,10 @@ async function redirectCurrentTabToWalletPage(page: string): Promise<void> {
   return redirectTabToWalletPage(tab.id!, page);
 }
 
-async function redirectTabToWalletPage(tabId: number, page: string): Promise<void> {
+async function redirectTabToWalletPage(
+  tabId: number,
+  page: string,
+): Promise<void> {
   const url = chrome.runtime.getURL(`/static/wallet.html#${page}`);
   logger.trace("redirecting tabId: ", tabId, " to: ", url);
   await chrome.tabs.update(tabId, { url });
@@ -461,26 +470,26 @@ function setAlertedIcon(): void {
 
 interface OffscreenCanvasRenderingContext2D
   extends CanvasState,
-  CanvasTransform,
-  CanvasCompositing,
-  CanvasImageSmoothing,
-  CanvasFillStrokeStyles,
-  CanvasShadowStyles,
-  CanvasFilters,
-  CanvasRect,
-  CanvasDrawPath,
-  CanvasUserInterface,
-  CanvasText,
-  CanvasDrawImage,
-  CanvasImageData,
-  CanvasPathDrawingStyles,
-  CanvasTextDrawingStyles,
-  CanvasPath {
+    CanvasTransform,
+    CanvasCompositing,
+    CanvasImageSmoothing,
+    CanvasFillStrokeStyles,
+    CanvasShadowStyles,
+    CanvasFilters,
+    CanvasRect,
+    CanvasDrawPath,
+    CanvasUserInterface,
+    CanvasText,
+    CanvasDrawImage,
+    CanvasImageData,
+    CanvasPathDrawingStyles,
+    CanvasTextDrawingStyles,
+    CanvasPath {
   readonly canvas: OffscreenCanvas;
 }
 declare const OffscreenCanvasRenderingContext2D: {
   prototype: OffscreenCanvasRenderingContext2D;
-  new(): OffscreenCanvasRenderingContext2D;
+  new (): OffscreenCanvasRenderingContext2D;
 };
 
 interface OffscreenCanvas extends EventTarget {
@@ -493,7 +502,7 @@ interface OffscreenCanvas extends EventTarget {
 }
 declare const OffscreenCanvas: {
   prototype: OffscreenCanvas;
-  new(width: number, height: number): OffscreenCanvas;
+  new (width: number, height: number): OffscreenCanvas;
 };
 
 function createCanvas(size: number): OffscreenCanvas {
@@ -719,247 +728,6 @@ function listenNetworkConnectionState(
   };
 }
 
-// type HeaderListenerFunc = (
-//   details: chrome.webRequest.WebResponseHeadersDetails,
-// ) => void;
-// let currentHeaderListener: HeaderListenerFunc | undefined = undefined;
-
-// type TabListenerFunc = (tabId: number, info: chrome.tabs.TabChangeInfo) => void;
-// let currentTabListener: TabListenerFunc | undefined = undefined;
-
-
-// function containsTalerHeaderListener(): boolean {
-//   return (
-//     currentHeaderListener !== undefined || currentTabListener !== undefined
-//   );
-// }
-
-// function headerListener(
-//   details: chrome.webRequest.WebResponseHeadersDetails,
-// ): chrome.webRequest.BlockingResponse | undefined {
-//   if (chrome.runtime.lastError) {
-//     logger.error(JSON.stringify(chrome.runtime.lastError));
-//     return;
-//   }
-//   console.log("HEADER", JSON.stringify(details, undefined, 2))
-//   if (
-//     details.statusCode === 402 ||
-//     details.statusCode === 202 ||
-//     details.statusCode === 200
-//   ) {
-//     const values = (details.responseHeaders || [])
-//       .filter((h) => h.name.toLowerCase() === "taler")
-//       .map((h) => h.value)
-//       .filter((value): value is string => !!value);
-//     if (values.length > 0) {
-//       logger.info(
-//         `Found a Taler URI in a response header for the request ${details.url} from tab ${details.tabId}`,
-//       );
-//       const redirectUrl = redirectTabToWalletPage(details.tabId, values[0]);
-//       return { redirectUrl }
-//     }
-//   }
-//   return details;
-// }
-// function parseTalerUriAndRedirect(tabId: number, maybeTalerUri: string): void {
-//   const talerUri = maybeTalerUri.startsWith("ext+")
-//     ? maybeTalerUri.substring(4)
-//     : maybeTalerUri;
-//   const uri = parseTalerUri(talerUri);
-//   if (!uri) {
-//     logger.warn(
-//       `Response with HTTP 402 the Taler header but could not classify ${talerUri}`,
-//     );
-//     return;
-//   }
-//   redirectTabToWalletPage(
-//     tabId,
-//     `/taler-uri/${encodeURIComponent(talerUri)}`,
-//   );
-// }
-
-// async function tabListener(
-//   tabId: number,
-//   info: chrome.tabs.TabChangeInfo,
-// ): Promise<void> {
-//   if (tabId < 0) return;
-//   const tabLocationHasBeenUpdated = info.status === "complete";
-//   const tabTitleHasBeenUpdated = info.title !== undefined;
-//   if (tabLocationHasBeenUpdated || tabTitleHasBeenUpdated) {
-//     const uri = await findTalerUriInTab(tabId);
-//     if (!uri) return;
-//     logger.info(`Found a Taler URI in the tab ${tabId}`);
-//     parseTalerUriAndRedirect(tabId, uri);
-//   }
-// }
-
-/**
- * unused, declarative redirect is not good enough
- *
- */
-// async function registerDeclarativeRedirect() {
-//   await chrome.declarativeNetRequest.updateDynamicRules({
-//     removeRuleIds: [1],
-//     addRules: [
-//       {
-//         id: 1,
-//         priority: 1,
-//         condition: {
-//           urlFilter: "https://developer.chrome.com/docs/extensions/mv2/",
-//           regexFilter: ".*taler_uri=([^&]*).*",
-//           // isUrlFilterCaseSensitive: false,
-//           // requestMethods: [chrome.declarativeNetRequest.RequestMethod.GET]
-//           // resourceTypes: [chrome.declarativeNetRequest.ResourceType.MAIN_FRAME],
-//         },
-//         action: {
-//           type: chrome.declarativeNetRequest.RuleActionType.REDIRECT,
-//           redirect: {
-//             regexSubstitution: `chrome-extension://${chrome.runtime.id}/static/wallet.html?action=\\1`,
-//           },
-//         },
-//       },
-//     ],
-//   });
-// }
-
-// function registerTalerHeaderListener(
-// ): void {
-//   logger.info("setting up header listener");
-
-//   const prevHeaderListener = currentHeaderListener;
-//   const prevTabListener = currentTabListener;
-
-//   getPermissionsApi()
-//     .containsHostPermissions()
-//     .then((result) => {
-//       //if there is a handler already, remove it
-//       if (
-//         prevHeaderListener &&
-//         chrome?.webRequest?.onHeadersReceived?.hasListener(prevHeaderListener)
-//       ) {
-//         console.log("removming on header listener")
-//         chrome.webRequest.onHeadersReceived.removeListener(prevHeaderListener);
-//         // chrome.webRequest.onCompleted.removeListener(prevHeaderListener);
-//         // chrome.webRequest.onResponseStarted.removeListener(prevHeaderListener);
-//         // chrome.webRequest.onErrorOccurred.removeListener(prevHeaderListener);
-//       }
-//       if (
-//         prevTabListener &&
-//         chrome?.tabs?.onUpdated?.hasListener(prevTabListener)
-//       ) {
-//         console.log("removming on tab listener")
-//         chrome.tabs.onUpdated.removeListener(prevTabListener);
-//       }
-
-//       //if the result was positive, add the headerListener
-//       if (result) {
-//         console.log("headers on, disabled:", chrome?.webRequest?.onHeadersReceived === undefined)
-//         if (chrome?.webRequest) {
-//           chrome.webRequest.onHeadersReceived.addListener(headerListener,
-//             { urls: ["<all_urls>"] },
-//             ["responseHeaders", "extraHeaders"]
-//           );
-//           // chrome.webRequest.onCompleted.addListener(headerListener,
-//           //   { urls: ["<all_urls>"] },
-//           //   ["responseHeaders", "extraHeaders"]
-//           // );
-//           // chrome.webRequest.onResponseStarted.addListener(headerListener,
-//           //   { urls: ["<all_urls>"] },
-//           //   ["responseHeaders", "extraHeaders"]
-//           // );
-//           // chrome.webRequest.onErrorOccurred.addListener(headerListener,
-//           //   { urls: ["<all_urls>"] },
-//           //   ["extraHeaders"]
-//           // );
-//           currentHeaderListener = headerListener;
-//         }
-
-//         const tabsEvent: chrome.tabs.TabUpdatedEvent | undefined =
-//           chrome?.tabs?.onUpdated;
-//         if (tabsEvent) {
-//           tabsEvent.addListener(tabListener);
-//           currentTabListener = tabListener;
-//         }
-//       } else {
-//         console.log("headers off")
-//       }
-
-//       //notify the browser about this change, this operation is expensive
-//       chrome?.webRequest?.handlerBehaviorChanged(() => {
-//         if (chrome.runtime.lastError) {
-//           logger.error(JSON.stringify(chrome.runtime.lastError));
-//         }
-//       });
-//     });
-// }
-
-// const hostPermissions = {
-//   permissions: ["webRequest"],
-//   origins: ["http://*/*", "https://*/*"],
-// };
-
-// export function containsHostPermissions(): Promise<boolean> {
-//   return new Promise((res, rej) => {
-//     chrome.permissions.contains(hostPermissions, (resp) => {
-//       const le = chrome.runtime.lastError?.message;
-//       if (le) {
-//         rej(le);
-//       }
-//       res(resp);
-//     });
-//   });
-// }
-
-// export async function requestHostPermissions(): Promise<boolean> {
-//   return new Promise((res, rej) => {
-//     chrome.permissions.request(hostPermissions, (resp) => {
-//       const le = chrome.runtime.lastError?.message;
-//       if (le) {
-//         rej(le);
-//       }
-//       res(resp);
-//     });
-//   });
-// }
-
-// export async function removeHostPermissions(): Promise<boolean> {
-//   //if there is a handler already, remove it
-//   if (
-//     currentHeaderListener &&
-//     chrome?.webRequest?.onHeadersReceived?.hasListener(currentHeaderListener)
-//   ) {
-//     chrome.webRequest.onHeadersReceived.removeListener(currentHeaderListener);
-//   }
-//   if (
-//     currentTabListener &&
-//     chrome?.tabs?.onUpdated?.hasListener(currentTabListener)
-//   ) {
-//     chrome.tabs.onUpdated.removeListener(currentTabListener);
-//   }
-
-//   currentHeaderListener = undefined;
-//   currentTabListener = undefined;
-
-//   //notify the browser about this change, this operation is expensive
-//   if ("webRequest" in chrome) {
-//     chrome.webRequest.handlerBehaviorChanged(() => {
-//       if (chrome.runtime.lastError) {
-//         logger.error(JSON.stringify(chrome.runtime.lastError));
-//       }
-//     });
-//   }
-
-//   if (extensionIsManifestV3()) {
-//     // Trying to remove host permissions with manifest >= v3 throws an error
-//     return true;
-//   }
-//   return new Promise((res, rej) => {
-//     chrome.permissions.remove(hostPermissions, (resp) => {
-//       const le = chrome.runtime.lastError?.message;
-//       if (le) {
-//         rej(le);
-//       }
-//       res(resp);
-//     });
-//   });
-// }
\ No newline at end of file
+function runningOnPrivateMode(): boolean {
+  return chrome.extension.inIncognitoContext;
+}
