@@ -31,7 +31,8 @@ export function NoBalanceHelp({
   goToWalletManualWithdraw: ButtonHandler;
 }): VNode {
   const { i18n } = useTranslationContext();
-  return (
+  return (<Fragment>
+
     <Paper class={margin}>
       <Alert title={i18n.str`Your wallet is empty.`} severity="info">
         <Button
@@ -44,5 +45,9 @@ export function NoBalanceHelp({
         </Button>
       </Alert>
     </Paper>
+    <a href="https://demo.taler.net/" style={{ display: "block" }}>
+      <i18n.Translate>Try the demo bank and withdraw test money.</i18n.Translate> »
+    </a>
+  </Fragment>
   );
 }
