@@ -65,17 +65,6 @@ function ContentByUriType({
           </Button>
         </div>
       );
-    case TalerUriAction.Reward:
-      return (
-        <div>
-          <p>
-            <i18n.Translate>This page has a tip action.</i18n.Translate>
-          </p>
-          <Button variant="contained" color="success" onClick={onConfirm}>
-            <i18n.Translate>Open tip page</i18n.Translate>
-          </Button>
-        </div>
-      );
 
     case TalerUriAction.Refund:
       return (
@@ -93,8 +82,6 @@ function ContentByUriType({
     case TalerUriAction.PayPull:
     case TalerUriAction.PayPush:
     case TalerUriAction.Restore:
-    case TalerUriAction.Auditor:
-    case TalerUriAction.Exchange:
       return null;
     default: {
       const error: never = uri;
