@@ -14,7 +14,7 @@
  GNU Taler; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
  */
 
-import { NotificationType, TalerBankIntegrationHttpClient, TalerCoreBankHttpClient, TalerRevenueHttpClient, TalerWireGatewayHttpClient } from "@gnu-taler/taler-util";
+import { NotificationType, TalerBankIntegrationHttpClient, TalerCoreBankHttpClient, TalerRevenueHttpClient, TalerWireGatewayHttpClient, WalletNotification } from "@gnu-taler/taler-util";
 import {
   WalletCoreApiClient,
   WalletCoreOpKeys,
@@ -46,7 +46,7 @@ interface MockHandler {
 
   getCallingQueueState(): "empty" | string;
 
-  notifyEventFromWallet(event: NotificationType): void;
+  notifyEventFromWallet(notif: WalletNotification): void;
 }
 
 type CallRecord = WalletCallRecord | BackgroundCallRecord;
@@ -65,7 +65,7 @@ interface BackgroundCallRecord {
 }
 
 type Subscriptions = {
-  [key in NotificationType]?: VoidFunction;
+  [key in NotificationType]?: (d: WalletNotification) => void;
 };
 
 export function createWalletApiMock(): {
@@ -115,9 +115,12 @@ export function createWalletApiMock(): {
       },
     }),
     listener: {
+      trigger: () => {
+
+      },
       onUpdateNotification(
         mTypes: NotificationType[],
-        callback: (() => void) | undefined,
+        callback: ((d: WalletNotification) => void) | undefined,
       ): () => void {
         mTypes.forEach((m) => {
           subscriptions[m] = callback;
@@ -164,11 +167,11 @@ export function createWalletApiMock(): {
       });
       return handler;
     },
-    notifyEventFromWallet(event: NotificationType): void {
-      const callback = subscriptions[event];
+    notifyEventFromWallet(event: WalletNotification): void {
+      const callback = subscriptions[event.type];
       if (!callback)
         throw Error(`Expected to have a subscription for ${event}`);
-      return callback();
+      return callback(event);
     },
     getCallingQueueState() {
       return calls.length === 0 ? "empty" : `${calls.length} left`;
