@@ -59,7 +59,6 @@ import { PaymentPage } from "../cta/Payment/index.js";
 import { PaymentTemplatePage } from "../cta/PaymentTemplate/index.js";
 import { RecoveryPage } from "../cta/Recovery/index.js";
 import { RefundPage } from "../cta/Refund/index.js";
-import { TipPage } from "../cta/Reward/index.js";
 import { TransferCreatePage } from "../cta/TransferCreate/index.js";
 import { TransferPickupPage } from "../cta/TransferPickup/index.js";
 import {
@@ -82,6 +81,8 @@ import { QrReaderPage } from "./QrReader.js";
 import { SettingsPage } from "./Settings.js";
 import { TransactionPage } from "./Transaction.js";
 import { WelcomePage } from "./Welcome.js";
+import { WalletActivity } from "../components/WalletActivity.js";
+import { EnabledBySettings } from "../components/EnabledBySettings.js";
 
 export function Application(): VNode {
   const { i18n } = useTranslationContext();
@@ -365,20 +366,6 @@ export function Application(): VNode {
               </CallToActionTemplate>
             )}
           />
-          <Route
-            path={Pages.ctaTips}
-            component={({ talerUri }: { talerUri: string }) => (
-              <CallToActionTemplate title={i18n.str`Digital cash tip`}>
-                <TipPage
-                  talerTipUri={decodeURIComponent(talerUri)}
-                  onCancel={() => redirectTo(Pages.balance)}
-                  onSuccess={(tid: string) =>
-                    redirectTo(Pages.balanceTransaction({ tid }))
-                  }
-                />
-              </CallToActionTemplate>
-            )}
-          />
           <Route
             path={Pages.ctaWithdraw}
             component={({ talerUri }: { talerUri: string }) => (
@@ -404,10 +391,8 @@ export function Application(): VNode {
             }) => (
               <CallToActionTemplate title={i18n.str`Digital cash withdrawal`}>
                 <WithdrawPageFromParams
-                  onAmountChanged={async (e) => {
-                    const page = `${Pages.ctaWithdrawManual({
-                      amount,
-                    })}?talerUri=${encodeURIComponent(talerUri)}`;
+                  onAmountChanged={async (newamount) => {
+                    const page = `${Pages.ctaWithdrawManual({ amount: newamount })}?talerUri=${encodeURIComponent(talerUri)}`;
                     redirectTo(page);
                   }}
                   talerExchangeWithdrawUri={talerUri}
@@ -527,6 +512,9 @@ export function Application(): VNode {
             component={() => <Redirect to={Pages.balanceHistory({})} />}
           />
         </Router>
+        <EnabledBySettings name="showWalletActivity">
+          <WalletActivity />
+        </EnabledBySettings>
       </IoCProviderForRuntime>
     </TranslationProvider>
   );
