@@ -60,8 +60,8 @@ export function useComponentState({
     parsed !== undefined
       ? parsed
       : currency !== undefined
-      ? Amounts.zeroOfCurrency(currency)
-      : undefined;
+        ? Amounts.zeroOfCurrency(currency)
+        : undefined;
   // const [accountIdx, setAccountIdx] = useState<number>(0);
   const [selectedAccount, setSelectedAccount] = useState<PaytoUri>();
 
@@ -83,7 +83,8 @@ export function useComponentState({
   if (hook.hasError) {
     return {
       status: "error",
-      error: alertFromError(i18n.str`Could not load balance information`, hook),
+      error: alertFromError(i18n,
+        i18n.str`Could not load balance information`, hook),
     };
   }
   const { accounts, balances } = hook.response;
@@ -169,6 +170,7 @@ export function useComponentState({
       return {
         status: "error",
         error: alertFromError(
+          i18n,
           i18n.str`Could not load fee for amount ${amountStr}`,
           hook,
         ),
@@ -193,8 +195,8 @@ export function useComponentState({
     const amountError = !isDirty
       ? undefined
       : Amounts.cmp(balance, amount) === -1
-      ? `Too much, your current balance is ${Amounts.stringifyValue(balance)}`
-      : undefined;
+        ? `Too much, your current balance is ${Amounts.stringifyValue(balance)}`
+        : undefined;
 
     const unableToDeposit =
       Amounts.isZero(totalToDeposit) || //deposit may be zero because of fee
