@@ -20,71 +20,33 @@ import {
   CoinDumpJson,
   CoinStatus,
   ExchangeListItem,
+  ExchangeTosStatus,
   LogLevel,
   NotificationType,
+  ScopeType,
+  parseWithdrawUri,
+  stringifyWithdrawExchange,
 } from "@gnu-taler/taler-util";
-import {
-  PendingTaskInfo,
-  WalletApiOperation,
-} from "@gnu-taler/taler-wallet-core";
+import { WalletApiOperation } from "@gnu-taler/taler-wallet-core";
 import { useTranslationContext } from "@gnu-taler/web-util/browser";
 import { format } from "date-fns";
 import { Fragment, VNode, h } from "preact";
 import { useEffect, useRef, useState } from "preact/hooks";
+import { Checkbox } from "../components/Checkbox.js";
 import { SelectList } from "../components/SelectList.js";
 import { Time } from "../components/Time.js";
-import { NotifyUpdateFadeOut } from "../components/styled/index.js";
+import { DestructiveText, LinkPrimary, NotifyUpdateFadeOut, SubTitle, SuccessText, WarningText } from "../components/styled/index.js";
+import { useAlertContext } from "../context/alert.js";
 import { useBackendContext } from "../context/backend.js";
 import { useAsyncAsHook } from "../hooks/useAsyncAsHook.js";
+import { useSettings } from "../hooks/useSettings.js";
 import { Button } from "../mui/Button.js";
 import { Grid } from "../mui/Grid.js";
 import { Paper } from "../mui/Paper.js";
 import { TextField } from "../mui/TextField.js";
-
-export function DeveloperPage(): VNode {
-
-  const listenAllEvents = Array.from<NotificationType>({ length: 1 });
-
-  const api = useBackendContext();
-
-  const response = useAsyncAsHook(async () => {
-    const op = await api.wallet.call(
-      WalletApiOperation.GetPendingOperations,
-      {},
-    );
-    const c = await api.wallet.call(WalletApiOperation.DumpCoins, {});
-    const ex = await api.wallet.call(WalletApiOperation.ListExchanges, {});
-    return {
-      operations: op.pendingOperations,
-      coins: c.coins,
-      exchanges: ex.exchanges,
-    };
-  });
-
-  useEffect(() => {
-    return api.listener.onUpdateNotification(listenAllEvents, response?.retry);
-  });
-
-  const nonResponse = { operations: [], coins: [], exchanges: [] };
-  const { operations, coins, exchanges } =
-    response === undefined
-      ? nonResponse
-      : response.hasError
-        ? nonResponse
-        : response.response;
-
-  return (
-    <View
-      operations={operations}
-      coins={coins}
-      exchanges={exchanges}
-      onDownloadDatabase={async () => {
-        const db = await api.wallet.call(WalletApiOperation.ExportDb, {});
-        return JSON.stringify(db);
-      }}
-    />
-  );
-}
+import { Pages } from "../NavigationBar.js";
+import { CoinInfo } from "@gnu-taler/taler-wallet-core/dbless";
+import { ActiveTasksTable } from "../components/WalletActivity.js";
 
 type CoinsInfo = CoinDumpJson["coins"];
 type CalculatedCoinfInfo = {
@@ -103,27 +65,21 @@ type SplitedCoinInfo = {
 };
 
 export interface Props {
-  operations: PendingTaskInfo[];
-  coins: CoinsInfo;
-  exchanges: ExchangeListItem[];
-  onDownloadDatabase: () => Promise<string>;
+  // FIXME: Pending operations don't exist anymore.
 }
 
 function hashObjectId(o: any): string {
   return JSON.stringify(o);
 }
 
-export function View({
-  operations,
-  coins,
-  onDownloadDatabase,
-}: Props): VNode {
+export function DeveloperPage({ }: Props): VNode {
   const { i18n } = useTranslationContext();
   const [downloadedDatabase, setDownloadedDatabase] = useState<
     { time: Date; content: string } | undefined
   >(undefined);
   async function onExportDatabase(): Promise<void> {
-    const content = await onDownloadDatabase();
+    const db = await api.wallet.call(WalletApiOperation.ExportDb, {});
+    const content = JSON.stringify(db);
     setDownloadedDatabase({
       time: new Date(),
       content,
@@ -133,15 +89,31 @@ export function View({
 
   const fileRef = useRef<HTMLInputElement>(null);
   async function onImportDatabase(str: string): Promise<void> {
-    return api.wallet.call(WalletApiOperation.ImportDb, {
+    await api.wallet.call(WalletApiOperation.ImportDb, {
       dump: JSON.parse(str),
     });
   }
+  const [settings, updateSettings] = useSettings();
+  const { safely } = useAlertContext();
 
-  const hook = useAsyncAsHook(() =>
-    api.wallet.call(WalletApiOperation.ListExchanges, {}),
-  );
+  const listenAllEvents = Array.from<NotificationType>({ length: 1 });
+  // listenAllEvents.includes = () => true
+
+  const hook = useAsyncAsHook(async () => {
+    const list = await api.wallet.call(WalletApiOperation.ListExchanges, {});
+    const version = await api.wallet.call(WalletApiOperation.GetVersion, {});
+    const coins = await api.wallet.call(WalletApiOperation.DumpCoins, {});
+    return { exchanges: list.exchanges, version, coins };
+  });
   const exchangeList = hook && !hook.hasError ? hook.response.exchanges : [];
+  const coins = hook && !hook.hasError ? hook.response.coins.coins : [];
+
+  useEffect(() => {
+    return api.listener.onUpdateNotification(listenAllEvents, (ev) => {
+      console.log("event", ev)
+      return hook?.retry()
+    });
+  });
 
   const currencies: { [ex: string]: string } = {};
   const money_by_exchange = coins.reduce(
@@ -203,30 +175,6 @@ export function View({
             <i18n.Translate>run gc</i18n.Translate>
           </Button>
         </Grid>
-        <Grid item>
-          <Button
-            variant="contained"
-            onClick={() => {
-              return api.background.call("sum", [1, 2, 3]).then((r) => {
-                console.log("SUM", r);
-              });
-            }}
-          >
-            <i18n.Translate>sum 123</i18n.Translate>
-          </Button>
-        </Grid>
-        <Grid item>
-          <Button
-            variant="contained"
-            onClick={() => {
-              return api.background.call("freeze", 4000).then(() => {
-                console.log("WAIT");
-              });
-            }}
-          >
-            <i18n.Translate>freeze 4000</i18n.Translate>
-          </Button>
-        </Grid>
         <Grid item>
           <Button
             variant="contained"
@@ -257,64 +205,6 @@ export function View({
             <i18n.Translate>export database</i18n.Translate>
           </Button>
         </Grid>
-        <Grid item>
-          <Button
-            variant="contained"
-            onClick={async () => {
-              navigator.registerProtocolHandler(
-                "taler",
-                `${window.location.origin}/static/wallet.html#/cta/withdraw?talerWithdrawUri=%s`,
-              );
-            }}
-          >
-            <i18n.Translate>Register taler:// handler</i18n.Translate>
-          </Button>
-        </Grid>
-        <Grid item>
-          <Button
-            variant="contained"
-            onClick={async () => {
-              const n = navigator as any;
-              if ("unregisterProtocolHandler" in n) {
-                n.unregisterProtocolHandler(
-                  "taler",
-                  `${window.location.origin}/static/wallet.html#/cta/withdraw?talerWithdrawUri=%s`,
-                );
-              }
-            }}
-          >
-            <i18n.Translate>Remove taler:// handler</i18n.Translate>
-          </Button>
-        </Grid>{" "}
-        <Grid item>
-          <Button
-            variant="contained"
-            onClick={async () => {
-              navigator.registerProtocolHandler(
-                "ext+taler",
-                `${window.location.origin}/static/wallet.html#/cta/withdraw?talerWithdrawUri=%s`,
-              );
-            }}
-          >
-            <i18n.Translate>Register ext+taler:// handler</i18n.Translate>
-          </Button>
-        </Grid>
-        <Grid item>
-          <Button
-            variant="contained"
-            onClick={async () => {
-              const n = navigator as any;
-              if ("unregisterProtocolHandler" in n) {
-                n.unregisterProtocolHandler(
-                  "ext+taler",
-                  `${window.location.origin}/static/wallet.html#/cta/withdraw?talerWithdrawUri=%s`,
-                );
-              }
-            }}
-          >
-            <i18n.Translate>Remove ext+taler:// handler</i18n.Translate>
-          </Button>
-        </Grid>
         <Grid item>
           <Button
             variant="contained"
@@ -345,6 +235,241 @@ export function View({
           </Button>
         </Grid>{" "}
       </Grid>
+      {downloadedDatabase && (
+        <div>
+          <i18n.Translate>
+            Database exported at{" "}
+            <Time
+              timestamp={AbsoluteTime.fromMilliseconds(
+                downloadedDatabase.time.getTime(),
+              )}
+              format="yyyy/MM/dd HH:mm:ss"
+            />{" "}
+            <a
+              href={`data:text/plain;charset=utf-8;base64,${toBase64(
+                downloadedDatabase.content,
+              )}`}
+              download={`taler-wallet-database-${format(
+                downloadedDatabase.time,
+                "yyyy/MM/dd_HH:mm",
+              )}.json`}
+            >
+              <i18n.Translate>click here</i18n.Translate>
+            </a>{" "}
+            to download
+          </i18n.Translate>
+        </div>
+      )}
+      <Checkbox
+        label={i18n.str`Inject Taler support in all pages`}
+        name="inject"
+        description={
+          <i18n.Translate>
+            Enabling this option will make `window.taler` be available in all
+            sites
+          </i18n.Translate>
+        }
+        enabled={settings.injectTalerSupport!}
+        onToggle={safely("update support injection", async () => {
+          updateSettings("injectTalerSupport", !settings.injectTalerSupport);
+        })}
+      />
+
+
+      <SubTitle>
+        <i18n.Translate>Exchange Entries</i18n.Translate>
+      </SubTitle>
+      {!exchangeList || !exchangeList.length ? (
+        <div>
+          <i18n.Translate>No exchange yet</i18n.Translate>
+        </div>
+      ) : (
+        <Fragment>
+          <table>
+            <thead>
+              <tr>
+                <th>
+                  <i18n.Translate>Currency</i18n.Translate>
+                </th>
+                <th>
+                  <i18n.Translate>URL</i18n.Translate>
+                </th>
+                <th>
+                  <i18n.Translate>Status</i18n.Translate>
+                </th>
+                <th>
+                  <i18n.Translate>Terms of Service</i18n.Translate>
+                </th>
+                <th>
+                  <i18n.Translate>Last Update</i18n.Translate>
+                </th>
+                <th>
+                  <i18n.Translate>Actions</i18n.Translate>
+                </th>
+              </tr>
+            </thead>
+            <tbody>
+              {exchangeList.map((e, idx) => {
+                function TosStatus(): VNode {
+                  switch (e.tosStatus) {
+                    case ExchangeTosStatus.Accepted:
+                      return (
+                        <SuccessText>
+                          <i18n.Translate>ok</i18n.Translate>
+                        </SuccessText>
+                      );
+                    case ExchangeTosStatus.Pending:
+                      return (
+                        <WarningText>
+                          <i18n.Translate>pending</i18n.Translate>
+                        </WarningText>
+                      );
+                    case ExchangeTosStatus.Proposed:
+                      return <i18n.Translate>proposed</i18n.Translate>;
+                    default:
+                      return (
+                        <DestructiveText>
+                          <i18n.Translate>
+                            unknown (exchange status should be updated)
+                          </i18n.Translate>
+                        </DestructiveText>
+                      );
+                  }
+                }
+                const uri = !e.masterPub ? undefined : stringifyWithdrawExchange({
+                  exchangeBaseUrl: e.exchangeBaseUrl,
+                  exchangePub: e.masterPub,
+                });
+                return (
+                  <tr key={idx}>
+                    <td>
+                      <a href={!uri ? undefined : Pages.defaultCta({ uri })}>
+                        {e.scopeInfo ? `${e.scopeInfo.currency} (${e.scopeInfo.type === ScopeType.Global ? "global" : "regional"})` : e.currency}
+                      </a>
+                    </td>
+                    <td>
+                      <a href={new URL(`/keys`, e.exchangeBaseUrl).href} target="_blank">{e.exchangeBaseUrl}</a>
+                    </td>
+                    <td>
+                      {e.exchangeEntryStatus} / {e.exchangeUpdateStatus}
+                    </td>
+                    <td>
+                      <TosStatus />
+                    </td>
+                    <td>
+                      {e.lastUpdateTimestamp
+                        ? AbsoluteTime.toIsoString(
+                          AbsoluteTime.fromPreciseTimestamp(
+                            e.lastUpdateTimestamp,
+                          ),
+                        )
+                        : "never"}
+                    </td>
+                    <td>
+                      <button
+                        onClick={() => {
+                          api.wallet.call(
+                            WalletApiOperation.UpdateExchangeEntry,
+                            {
+                              exchangeBaseUrl: e.exchangeBaseUrl,
+                              force: true,
+                            },
+                          );
+                        }}
+                      >
+                        Reload
+                      </button>
+                      <button
+                        onClick={() => {
+                          api.wallet.call(
+                            WalletApiOperation.DeleteExchange,
+                            {
+                              exchangeBaseUrl: e.exchangeBaseUrl,
+                            },
+                          );
+                        }}
+                      >
+                        Delete
+                      </button>
+                      <button
+                        onClick={() => {
+                          api.wallet.call(
+                            WalletApiOperation.DeleteExchange,
+                            {
+                              exchangeBaseUrl: e.exchangeBaseUrl,
+                              purge: true,
+                            },
+                          );
+                        }}
+                      >
+                        Purge
+                      </button>
+                      {e.scopeInfo && e.masterPub && e.currency ?
+                        (e.scopeInfo.type === ScopeType.Global ?
+                          <button
+                            onClick={() => {
+                              api.wallet.call(
+                                WalletApiOperation.RemoveGlobalCurrencyExchange,
+                                {
+                                  exchangeBaseUrl: e.exchangeBaseUrl,
+                                  currency: e.currency!,
+                                  exchangeMasterPub: e.masterPub!,
+                                },
+                              );
+                            }}
+                          >
+
+                            Make regional
+                          </button>
+                          : e.scopeInfo.type === ScopeType.Auditor ?
+                            undefined
+
+                            : e.scopeInfo.type === ScopeType.Exchange ?
+                              <button
+                                onClick={() => {
+                                  api.wallet.call(
+                                    WalletApiOperation.AddGlobalCurrencyExchange,
+                                    {
+                                      exchangeBaseUrl: e.exchangeBaseUrl,
+                                      currency: e.currency!,
+                                      exchangeMasterPub: e.masterPub!,
+                                    },
+                                  );
+                                }}
+                              >
+
+                                Make global
+                              </button>
+                              : undefined) : undefined
+                      }
+                      <button
+                        onClick={() => {
+                          api.wallet.call(
+                            WalletApiOperation.SetExchangeTosForgotten,
+                            {
+                              exchangeBaseUrl: e.exchangeBaseUrl,
+                            },
+                          );
+                        }}
+                      >
+                        Forget ToS
+                      </button>
+                    </td>
+                  </tr>
+                );
+              })}
+            </tbody>
+          </table>
+        </Fragment>
+      )}
+      <div style={{ display: "flex", justifyContent: "space-between" }}>
+        <div />
+        <LinkPrimary href={Pages.settingsExchangeAdd({})}>
+          <i18n.Translate>Add an exchange</i18n.Translate>
+        </LinkPrimary>
+      </div>
+
+
       <Paper style={{ padding: 10, margin: 10 }}>
         <h3>Logging</h3>
         <div>
@@ -381,28 +506,7 @@ export function View({
           Set log level
         </Button>
       </Paper>
-      {downloadedDatabase && (
-        <div>
-          <i18n.Translate>
-            Database exported at <Time
-              timestamp={AbsoluteTime.fromMilliseconds(
-                downloadedDatabase.time.getTime(),
-              )}
-              format="yyyy/MM/dd HH:mm:ss"
-            /> <a
-              href={`data:text/plain;charset=utf-8;base64,${toBase64(
-                downloadedDatabase.content,
-              )}`}
-              download={`taler-wallet-database-${format(
-                downloadedDatabase.time,
-                "yyyy/MM/dd_HH:mm",
-              )}.json`}
-            >
-              <i18n.Translate>click here</i18n.Translate>
-            </a> to download
-          </i18n.Translate>
-        </div>
-      )}
+
       <br />
       <p>
         <i18n.Translate>Coins</i18n.Translate>:
@@ -438,31 +542,9 @@ export function View({
         );
       })}
       <br />
-      {operations && operations.length > 0 && (
-        <Fragment>
-          <p>
-            <i18n.Translate>Pending operations</i18n.Translate>
-          </p>
-          <dl>
-            {operations.reverse().map((o) => {
-              return (
-                <NotifyUpdateFadeOut key={hashObjectId(o)}>
-                  <dt>
-                    {o.type}{" "}
-                    <Time
-                      timestamp={o.timestampDue}
-                      format="yy/MM/dd HH:mm:ss"
-                    />
-                  </dt>
-                  <dd>
-                    <pre>{JSON.stringify(o, undefined, 2)}</pre>
-                  </dd>
-                </NotifyUpdateFadeOut>
-              );
-            })}
-          </dl>
-        </Fragment>
-      )}
+      <NotifyUpdateFadeOut>
+        <ActiveTasksTable />
+      </NotifyUpdateFadeOut>
     </div>
   );
 }
