@@ -35,7 +35,6 @@ import {
   TransactionPeerPushDebit,
   TransactionRefresh,
   TransactionRefund,
-  TransactionReward,
   TransactionType,
   TransactionWithdrawal,
   WithdrawalType,
@@ -112,11 +111,6 @@ const exampleData = {
     exchangeBaseUrl: "http://exchange.taler",
     refreshReason: RefreshReason.PayMerchant,
   } as TransactionRefresh,
-  tip: {
-    ...commonTransaction(),
-    type: TransactionType.Reward,
-    merchantBaseUrl: "http://ads.merchant.taler.net/",
-  } as TransactionReward,
   refund: {
     ...commonTransaction(),
     type: TransactionType.Refund,
@@ -168,11 +162,6 @@ const exampleData = {
   } as TransactionPeerPullDebit,
 };
 
-export const NoBalance = tests.createExample(TestedComponent, {
-  transactions: [],
-  balances: [],
-});
-
 export const SomeBalanceWithNoTransactions = tests.createExample(
   TestedComponent,
   {
@@ -192,6 +181,7 @@ export const SomeBalanceWithNoTransactions = tests.createExample(
         },
       },
     ],
+    balanceIndex: 0,
   },
 );
 
@@ -212,6 +202,8 @@ export const OneSimpleTransaction = tests.createExample(TestedComponent, {
       },
     },
   ],
+  balanceIndex: 0,
+
 });
 
 export const TwoTransactionsAndZeroBalance = tests.createExample(
@@ -233,6 +225,7 @@ export const TwoTransactionsAndZeroBalance = tests.createExample(
         },
       },
     ],
+    balanceIndex: 0,
   },
 );
 
@@ -260,6 +253,7 @@ export const OneTransactionPending = tests.createExample(TestedComponent, {
       },
     },
   ],
+  balanceIndex: 0,
 });
 
 export const SomeTransactions = tests.createExample(TestedComponent, {
@@ -277,7 +271,6 @@ export const SomeTransactions = tests.createExample(TestedComponent, {
       },
     },
     exampleData.refund,
-    exampleData.tip,
     exampleData.deposit,
   ],
   balances: [
@@ -295,6 +288,7 @@ export const SomeTransactions = tests.createExample(TestedComponent, {
       },
     },
   ],
+  balanceIndex: 0,
 });
 
 export const SomeTransactionsInDifferentStates = tests.createExample(
@@ -371,7 +365,6 @@ export const SomeTransactionsInDifferentStates = tests.createExample(
         },
       },
       exampleData.refund,
-      exampleData.tip,
       exampleData.deposit,
     ],
     balances: [
@@ -389,6 +382,7 @@ export const SomeTransactionsInDifferentStates = tests.createExample(
         },
       },
     ],
+    balanceIndex: 0,
   },
 );
 
@@ -402,7 +396,6 @@ export const SomeTransactionsWithTwoCurrencies = tests.createExample(
       exampleData.payment,
       exampleData.refresh,
       exampleData.refund,
-      exampleData.tip,
       exampleData.deposit,
     ],
     balances: [
@@ -433,6 +426,7 @@ export const SomeTransactionsWithTwoCurrencies = tests.createExample(
         },
       },
     ],
+    balanceIndex: 0,
   },
 );
 
@@ -505,6 +499,7 @@ export const FiveOfficialCurrencies = tests.createExample(TestedComponent, {
       },
     },
   ],
+  balanceIndex: 0,
 });
 
 export const FiveOfficialCurrenciesWithHighValue = tests.createExample(
@@ -578,6 +573,7 @@ export const FiveOfficialCurrenciesWithHighValue = tests.createExample(
         },
       },
     ],
+    balanceIndex: 0,
   },
 );
 
@@ -603,4 +599,5 @@ export const PeerToPeer = tests.createExample(TestedComponent, {
       },
     },
   ],
+  balanceIndex: 0,
 });
