@@ -18,11 +18,13 @@ import {
   AbsoluteTime,
   Amounts,
   NotificationType,
+  ScopeType,
   Transaction,
-  WalletBalance,
+  WalletBalance
 } from "@gnu-taler/taler-util";
 import { WalletApiOperation } from "@gnu-taler/taler-wallet-core";
 import { useTranslationContext } from "@gnu-taler/web-util/browser";
+import { startOfDay } from "date-fns";
 import { Fragment, VNode, h } from "preact";
 import { useEffect, useState } from "preact/hooks";
 import { ErrorAlertView } from "../components/CurrentAlerts.js";
@@ -38,11 +40,11 @@ import {
 import { alertFromError, useAlertContext } from "../context/alert.js";
 import { useBackendContext } from "../context/backend.js";
 import { useAsyncAsHook } from "../hooks/useAsyncAsHook.js";
+import { useSettings } from "../hooks/useSettings.js";
 import { Button } from "../mui/Button.js";
 import { NoBalanceHelp } from "../popup/NoBalanceHelp.js";
 import DownloadIcon from "../svg/download_24px.inline.svg";
 import UploadIcon from "../svg/upload_24px.inline.svg";
-import { getDate, startOfDay } from "date-fns";
 
 interface Props {
   currency?: string;
@@ -56,10 +58,18 @@ export function HistoryPage({
 }: Props): VNode {
   const { i18n } = useTranslationContext();
   const api = useBackendContext();
-  const state = useAsyncAsHook(async () => ({
-    b: await api.wallet.call(WalletApiOperation.GetBalances, {}),
-    tx: await api.wallet.call(WalletApiOperation.GetTransactions, {}),
-  }));
+  const [balanceIndex, setBalanceIndex] = useState<number>(0);
+  const [settings] = useSettings();
+  const state = useAsyncAsHook(async () => {
+    const b = await api.wallet.call(WalletApiOperation.GetBalances, {})
+    const balance = b.balances.length > 0 ? b.balances[balanceIndex] : undefined
+    const tx = await api.wallet.call(WalletApiOperation.GetTransactions, {
+      scopeInfo: balance?.scopeInfo,
+      sort: "descending",
+      includeRefreshes: settings.showRefeshTransactions,
+    })
+    return { b, tx }
+  }, [balanceIndex]);
 
   useEffect(() => {
     return api.listener.onUpdateNotification(
@@ -67,6 +77,7 @@ export function HistoryPage({
       state?.retry,
     );
   });
+  const { pushAlertOnError } = useAlertContext();
 
   if (!state) {
     return <Loading />;
@@ -76,6 +87,7 @@ export function HistoryPage({
     return (
       <ErrorAlertView
         error={alertFromError(
+          i18n,
           i18n.str`Could not load the list of transactions`,
           state,
         )}
@@ -83,78 +95,52 @@ export function HistoryPage({
     );
   }
 
+  if (!state.response.b.balances.length) {
+    return (
+      <NoBalanceHelp
+        goToWalletManualWithdraw={{
+          onClick: pushAlertOnError(goToWalletManualWithdraw),
+        }}
+      />
+    );
+  }
   return (
     <HistoryView
+      balanceIndex={balanceIndex}
+      changeBalanceIndex={b => setBalanceIndex(b)}
       balances={state.response.b.balances}
-      defaultCurrency={currency}
       goToWalletManualWithdraw={goToWalletManualWithdraw}
       goToWalletDeposit={goToWalletDeposit}
-      transactions={[...state.response.tx.transactions].reverse()}
+      transactions={state.response.tx.transactions}
     />
   );
 }
 
-const term = 1000 * 60 * 60 * 24;
-function normalizeToDay(x: number): number {
-  return Math.round(x / term) * term;
-}
-
 export function HistoryView({
-  defaultCurrency,
-  transactions,
   balances,
+  balanceIndex,
+  changeBalanceIndex,
+  transactions,
   goToWalletManualWithdraw,
   goToWalletDeposit,
 }: {
+  balanceIndex: number,
+  changeBalanceIndex: (s: number) => void;
   goToWalletDeposit: (currency: string) => Promise<void>;
   goToWalletManualWithdraw: (currency?: string) => Promise<void>;
-  defaultCurrency?: string;
   transactions: Transaction[];
   balances: WalletBalance[];
 }): VNode {
   const { i18n } = useTranslationContext();
-  const { pushAlertOnError } = useAlertContext();
 
-  const transactionByCurrency = transactions.reduce((prev, cur) => {
-    const c = Amounts.parseOrThrow(cur.amountEffective).currency;
-    if (!prev[c]) {
-      prev[c] = [];
-    }
-    prev[c].push(cur);
-    return prev;
-  }, {} as Record<string, Transaction[]>);
+  const balance = balances[balanceIndex];
 
-  const currencies = balances
-    .filter((b) => {
-      const av = Amounts.parseOrThrow(b.available);
-      return (
-        Amounts.isNonZero(av) ||
-        (transactionByCurrency[av.currency] &&
-          transactionByCurrency[av.currency].length > 0)
-      );
-    })
-    .map((b) => b.available.split(":")[0]);
-
-  const defaultCurrencyIndex = currencies.findIndex(
-    (c) => c === defaultCurrency,
-  );
-  const [currencyIndex, setCurrencyIndex] = useState(
-    defaultCurrencyIndex === -1 ? 0 : defaultCurrencyIndex,
-  );
-  const selectedCurrency =
-    currencies.length > 0 ? currencies[currencyIndex] : undefined;
-
-  const currencyAmount = balances[currencyIndex]
-    ? Amounts.jsonifyAmount(balances[currencyIndex].available)
+  const available = balance
+    ? Amounts.jsonifyAmount(balance.available)
     : undefined;
 
-  const ts =
-    selectedCurrency === undefined
-      ? []
-      : transactionByCurrency[selectedCurrency] ?? [];
-
   const datesWithTransaction: string[] = [];
-  const byDate = ts.reduce((rv, x) => {
+  const byDate = transactions.reduce((rv, x) => {
     const startDay =
       x.timestamp.t_s === "never" ? 0 : startOfDay(x.timestamp.t_s * 1000).getTime();
     if (startDay) {
@@ -168,15 +154,6 @@ export function HistoryView({
     return rv;
   }, {} as { [x: string]: Transaction[] });
 
-  if (balances.length === 0 || !selectedCurrency) {
-    return (
-      <NoBalanceHelp
-        goToWalletManualWithdraw={{
-          onClick: pushAlertOnError(goToWalletManualWithdraw),
-        }}
-      />
-    );
-  }
   return (
     <Fragment>
       <section>
@@ -194,32 +171,35 @@ export function HistoryView({
               display: "flex",
             }}
           >
-            {currencies.length === 1 ? (
+            {balances.length === 1 ? (
               <CenteredText style={{ fontSize: "x-large", margin: 8 }}>
-                {selectedCurrency}
+                {balance.scopeInfo.currency}
               </CenteredText>
             ) : (
-              <NiceSelect>
+              <NiceSelect style={{ flexDirection: "column" }}>
                 <select
                   style={{
                     fontSize: "x-large",
                   }}
-                  value={currencyIndex}
+                  value={balanceIndex}
                   onChange={(e) => {
-                    setCurrencyIndex(Number(e.currentTarget.value));
+                    changeBalanceIndex(Number.parseInt(e.currentTarget.value, 10));
                   }}
                 >
-                  {currencies.map((currency, index) => {
+                  {balances.map((entry, index) => {
                     return (
-                      <option value={index} key={currency}>
-                        {currency}
+                      <option value={index} key={entry.scopeInfo.currency}>
+                        {entry.scopeInfo.currency}
                       </option>
                     );
                   })}
                 </select>
+                <div style={{ fontSize: "small", color: "grey" }}>
+                  {balance.scopeInfo.type === ScopeType.Exchange || balance.scopeInfo.type === ScopeType.Auditor ? balance.scopeInfo.url : undefined}
+                </div>
               </NiceSelect>
             )}
-            {currencyAmount && (
+            {available && (
               <CenteredBoldText
                 style={{
                   display: "inline-block",
@@ -227,7 +207,7 @@ export function HistoryView({
                   margin: 8,
                 }}
               >
-                {Amounts.stringifyValue(currencyAmount, 2)}
+                {Amounts.stringifyValue(available, 2)}
               </CenteredBoldText>
             )}
           </div>
@@ -236,17 +216,17 @@ export function HistoryView({
               tooltip="Transfer money to the wallet"
               startIcon={DownloadIcon}
               variant="contained"
-              onClick={() => goToWalletManualWithdraw(selectedCurrency)}
+              onClick={() => goToWalletManualWithdraw(balance.scopeInfo.currency)}
             >
               <i18n.Translate>Add</i18n.Translate>
             </Button>
-            {currencyAmount && Amounts.isNonZero(currencyAmount) && (
+            {available && Amounts.isNonZero(available) && (
               <Button
                 tooltip="Transfer money from the wallet"
                 startIcon={UploadIcon}
                 variant="outlined"
                 color="primary"
-                onClick={() => goToWalletDeposit(selectedCurrency)}
+                onClick={() => goToWalletDeposit(balance.scopeInfo.currency)}
               >
                 <i18n.Translate>Send</i18n.Translate>
               </Button>
