@@ -19,7 +19,6 @@ import {
   AmountJson,
   Amounts,
   AmountString,
-  Location,
   MerchantInfo,
   NotificationType,
   OrderShortInfo,
@@ -28,7 +27,6 @@ import {
   stringifyPaytoUri,
   TalerErrorCode,
   TalerPreciseTimestamp,
-  TalerProtocolTimestamp,
   Transaction,
   TransactionAction,
   TransactionDeposit,
@@ -47,13 +45,12 @@ import { styled } from "@linaria/react";
 import { isPast } from "date-fns";
 import { ComponentChildren, Fragment, h, VNode } from "preact";
 import { useEffect, useState } from "preact/hooks";
-import emptyImg from "../../static/img/empty.png";
 import { Amount } from "../components/Amount.js";
 import { BankDetailsByPaytoType } from "../components/BankDetailsByPaytoType.js";
 import { AlertView, ErrorAlertView } from "../components/CurrentAlerts.js";
 import { EnabledBySettings } from "../components/EnabledBySettings.js";
 import { Loading } from "../components/Loading.js";
-import { Kind, Part, PartCollapsible, PartPayto } from "../components/Part.js";
+import { Kind, Part, PartPayto } from "../components/Part.js";
 import { QR } from "../components/QR.js";
 import { ShowFullContractTermPopup } from "../components/ShowFullContractTermPopup.js";
 import {
@@ -61,9 +58,7 @@ import {
   ErrorBox,
   InfoBox,
   Link,
-  ListOfProducts,
   Overlay,
-  Row,
   SmallLightText,
   SubTitle,
   SvgIcon,
@@ -77,12 +72,8 @@ import { useSettings } from "../hooks/useSettings.js";
 import { Button } from "../mui/Button.js";
 import { SafeHandler } from "../mui/handlers.js";
 import { Pages } from "../NavigationBar.js";
-import { assertUnreachable } from "../utils/index.js";
 import refreshIcon from "../svg/refresh_24px.inline.svg";
-import refreshIcon1 from "../svg/refresh_outlined_24px.inline.svg";
-import refreshIcon2 from "../svg/refresh_rounded_24px.inline.svg";
-import refreshIcon3 from "../svg/refresh_sharp_24px.inline.svg";
-import refreshIcon4 from "../svg/refresh_two_tone_24px.inline.svg";
+import { assertUnreachable } from "../utils/index.js";
 
 interface Props {
   tid: string;
@@ -116,6 +107,7 @@ export function TransactionPage({ tid, goToWalletHistory }: Props): VNode {
     return (
       <ErrorAlertView
         error={alertFromError(
+          i18n,
           i18n.str`Could not load transaction information`,
           state,
         )}
@@ -243,6 +235,7 @@ function TransactionTemplate({
           TalerErrorCode.WALLET_WITHDRAWAL_KYC_REQUIRED ? (
           <ErrorAlertView
             error={alertFromError(
+              i18n,
               i18n.str`There was an error trying to complete the transaction`,
               transaction.error,
             )}
@@ -258,7 +251,13 @@ function TransactionTemplate({
                   <div>
                     <i18n.Translate>
                       Follow this link to the{` `}
-                      <a rel="noreferrer" target="_bank" href={transaction.kycUrl}>KYC verifier</a>
+                      <a
+                        rel="noreferrer"
+                        target="_bank"
+                        href={transaction.kycUrl}
+                      >
+                        KYC verifier
+                      </a>
                     </i18n.Translate>
                   </div>
                 ) : (
@@ -279,7 +278,7 @@ function TransactionTemplate({
           <WarningBox>
             <div style={{ justifyContent: "center", lineHeight: "25px" }}>
               <i18n.Translate>This transaction is not completed</i18n.Translate>
-              <Link onClick={onRetry}>
+              <Link onClick={onRetry} style={{padding: 0}}>
                 <SvgIcon
                   title={i18n.str`Retry`}
                   dangerouslySetInnerHTML={{ __html: refreshIcon }}
@@ -444,6 +443,10 @@ export function TransactionView({
     transaction.type === TransactionType.Withdrawal ||
     transaction.type === TransactionType.InternalWithdrawal
   ) {
+    const conversion =
+      transaction.withdrawalDetails.type === WithdrawalType.ManualTransfer
+        ? transaction.withdrawalDetails.exchangeCreditAccountDetails ?? []
+        : [];
     return (
       <TransactionTemplate
         transaction={transaction}
@@ -464,21 +467,42 @@ export function TransactionView({
         </Header>
 
         {transaction.txState.major !==
-          TransactionMajorState.Pending ? undefined :
-          transaction.txState.minor === TransactionMinorState.KycRequired ||
-            transaction.txState.minor === TransactionMinorState.AmlRequired ? undefined :
-            transaction
-              .withdrawalDetails.type === WithdrawalType.ManualTransfer ? (
-              //manual withdrawal
-              <BankDetailsByPaytoType
-                amount={raw}
-                exchangeBaseUrl={transaction.exchangeBaseUrl}
-                subject={transaction.withdrawalDetails.reservePub}
-              />
-            ) : (
-              //integrated bank withdrawal
-              <ShowWithdrawalDetailForBankIntegrated transaction={transaction} />
-            )}
+          TransactionMajorState.Pending ? undefined : transaction.txState
+            .minor === TransactionMinorState.KycRequired ||
+            transaction.txState.minor ===
+            TransactionMinorState.AmlRequired ? undefined : transaction
+              .withdrawalDetails.type === WithdrawalType.ManualTransfer
+              && transaction.withdrawalDetails.exchangeCreditAccountDetails ? (
+          <Fragment>
+            <InfoBox>
+              {transaction.withdrawalDetails.exchangeCreditAccountDetails.length > 1 ?
+                <span>
+                  <i18n.Translate>
+                    Now the payment service provider is waiting for <Amount value={raw} /> to
+                    be transferred. Select one of the accounts and use the information below
+                    to complete the operation by making a wire transfer from your bank account.
+                  </i18n.Translate>
+                </span>
+                :
+                <span><i18n.Translate>
+                  Now the payment service provider is waiting for <Amount value={raw} /> to 
+                  be transferred. Use the information below to complete the operation
+                  by making a wire transfer from your bank account.
+                </i18n.Translate></span>}
+
+            </InfoBox>
+            <BankDetailsByPaytoType
+              amount={raw}
+              accounts={
+                transaction.withdrawalDetails.exchangeCreditAccountDetails ?? []
+              }
+              subject={transaction.withdrawalDetails.reservePub}
+            />
+          </Fragment>
+        ) : (
+          //integrated bank withdrawal
+          <ShowWithdrawalDetailForBankIntegrated transaction={transaction} />
+        )}
         <Part
           title={i18n.str`Details`}
           text={
@@ -748,40 +772,6 @@ export function TransactionView({
     );
   }
 
-  if (transaction.type === TransactionType.Reward) {
-    return (
-      <TransactionTemplate
-        transaction={transaction}
-        onDelete={onDelete}
-        onRetry={onRetry}
-        onAbort={onAbort}
-        onResume={onResume}
-        onSuspend={onSuspend}
-        onCancel={onCancel}
-      >
-        <Header
-          timestamp={transaction.timestamp}
-          type={i18n.str`Tip`}
-          total={effective}
-          kind="positive"
-        >
-          {transaction.merchantBaseUrl}
-        </Header>
-        {/* <Part
-          title={i18n.str`Merchant`}
-          text={<MerchantDetails merchant={transaction.merchant} />}
-          kind="neutral"
-        /> */}
-        <Part
-          title={i18n.str`Details`}
-          text={
-            <TipDetails amount={getAmountWithFee(effective, raw, "credit")} />
-          }
-        />
-      </TransactionTemplate>
-    );
-  }
-
   if (transaction.type === TransactionType.Refund) {
     return (
       <TransactionTemplate
@@ -876,6 +866,7 @@ export function TransactionView({
         />
         {transaction.txState.major === TransactionMajorState.Pending &&
           transaction.txState.minor === TransactionMinorState.Ready &&
+          transaction.talerUri &&
           !transaction.error && (
             <Part
               title={i18n.str`URI`}
@@ -973,12 +964,12 @@ export function TransactionView({
           kind="neutral"
         />
         {transaction.talerUri && (
-            <Part
-              title={i18n.str`URI`}
-              text={<ShowQrWithCopy text={transaction.talerUri} />}
-              kind="neutral"
-            />
-          )}
+          <Part
+            title={i18n.str`URI`}
+            text={<ShowQrWithCopy text={transaction.talerUri} />}
+            kind="neutral"
+          />
+        )}
         <Part
           title={i18n.str`Details`}
           text={
@@ -1034,6 +1025,13 @@ export function TransactionView({
       </TransactionTemplate>
     );
   }
+
+  if (transaction.type === TransactionType.Recoup) {
+    throw Error("recoup transaction not implemented");
+  }
+  if (transaction.type === TransactionType.Reward) {
+    throw Error("recoup transaction not implemented");
+  }
   assertUnreachable(transaction);
 }
 
@@ -1417,7 +1415,13 @@ export function TransferPickupDetails({
     </PurchaseDetailsTable>
   );
 }
-export function WithdrawDetails({ amount }: { amount: AmountWithFee }): VNode {
+export function WithdrawDetails({
+  conversion,
+  amount,
+}: {
+  conversion?: AmountJson;
+  amount: AmountWithFee;
+}): VNode {
   const { i18n } = useTranslationContext();
 
   const maxFrac = [amount.fee, amount.fee]
@@ -1428,15 +1432,38 @@ export function WithdrawDetails({ amount }: { amount: AmountWithFee }): VNode {
 
   return (
     <PurchaseDetailsTable>
-      <tr>
-        <td>
-          <i18n.Translate>Transfer</i18n.Translate>
-        </td>
-        <td>
-          <Amount value={amount.value} maxFracSize={amount.maxFrac} />
-        </td>
-      </tr>
-
+      {conversion ? (
+        <Fragment>
+          <tr>
+            <td>
+              <i18n.Translate>Transfer</i18n.Translate>
+            </td>
+            <td>
+              <Amount value={conversion} maxFracSize={amount.maxFrac} />
+            </td>
+          </tr>
+          {conversion.fraction === amount.value.fraction &&
+            conversion.value === amount.value.value ? undefined : (
+            <tr>
+              <td>
+                <i18n.Translate>Converted</i18n.Translate>
+              </td>
+              <td>
+                <Amount value={amount.value} maxFracSize={amount.maxFrac} />
+              </td>
+            </tr>
+          )}
+        </Fragment>
+      ) : (
+        <tr>
+          <td>
+            <i18n.Translate>Transfer</i18n.Translate>
+          </td>
+          <td>
+            <Amount value={amount.value} maxFracSize={amount.maxFrac} />
+          </td>
+        </tr>
+      )}
       {Amounts.isNonZero(amount.fee) && (
         <tr>
           <td>
@@ -1668,19 +1695,22 @@ function calculateAmountByWireTransfer(
   const allTracking = Object.values(state ?? {});
 
   //group tracking by wtid, sum amounts
-  const trackByWtid = allTracking.reduce((prev, cur) => {
-    const fee = Amounts.parseOrThrow(cur.wireFee);
-    const raw = Amounts.parseOrThrow(cur.amountRaw);
-    const total = !prev[cur.wireTransferId]
-      ? raw
-      : Amounts.add(prev[cur.wireTransferId].total, raw).amount;
+  const trackByWtid = allTracking.reduce(
+    (prev, cur) => {
+      const fee = Amounts.parseOrThrow(cur.wireFee);
+      const raw = Amounts.parseOrThrow(cur.amountRaw);
+      const total = !prev[cur.wireTransferId]
+        ? raw
+        : Amounts.add(prev[cur.wireTransferId].total, raw).amount;
 
-    prev[cur.wireTransferId] = {
-      total,
-      fee,
-    };
-    return prev;
-  }, {} as Record<string, { total: AmountJson; fee: AmountJson }>);
+      prev[cur.wireTransferId] = {
+        total,
+        fee,
+      };
+      return prev;
+    },
+    {} as Record<string, { total: AmountJson; fee: AmountJson }>,
+  );
 
   //remove wire fee from total amount
   return Object.entries(trackByWtid).map(([id, info]) => ({
@@ -1799,47 +1829,6 @@ function RefreshDetails({ amount }: { amount: AmountWithFee }): VNode {
   );
 }
 
-function TipDetails({ amount }: { amount: AmountWithFee }): VNode {
-  const { i18n } = useTranslationContext();
-
-  return (
-    <PurchaseDetailsTable>
-      <tr>
-        <td>
-          <i18n.Translate>Tip</i18n.Translate>
-        </td>
-        <td>
-          <Amount value={amount.value} maxFracSize={amount.maxFrac} />
-        </td>
-      </tr>
-
-      {Amounts.isNonZero(amount.fee) && (
-        <tr>
-          <td>
-            <i18n.Translate>Fees</i18n.Translate>
-          </td>
-          <td>
-            <Amount value={amount.fee} maxFracSize={amount.maxFrac} />
-          </td>
-        </tr>
-      )}
-      <tr>
-        <td colSpan={2}>
-          <hr />
-        </td>
-      </tr>
-      <tr>
-        <td>
-          <i18n.Translate>Total</i18n.Translate>
-        </td>
-        <td>
-          <Amount value={amount.total} maxFracSize={amount.maxFrac} />
-        </td>
-      </tr>
-    </PurchaseDetailsTable>
-  );
-}
-
 function Header({
   timestamp,
   total,
@@ -1992,7 +1981,7 @@ function ShowWithdrawalDetailForBankIntegrated({
   const raw = Amounts.parseOrThrow(transaction.amountRaw);
   return (
     <Fragment>
-      <EnabledBySettings name="advanceMode">
+      <EnabledBySettings name="advancedMode">
         <a
           href="#"
           onClick={(e) => {
@@ -2007,7 +1996,9 @@ function ShowWithdrawalDetailForBankIntegrated({
       {showDetails && (
         <BankDetailsByPaytoType
           amount={raw}
-          exchangeBaseUrl={transaction.exchangeBaseUrl}
+          accounts={
+            transaction.withdrawalDetails.exchangeCreditAccountDetails ?? []
+          }
           subject={transaction.withdrawalDetails.reservePub}
         />
       )}
