@@ -32,6 +32,7 @@ import {
   TalerErrorCode,
   TalerErrorDetail,
   WalletDiagnostics,
+  WalletNotification,
 } from "@gnu-taler/taler-util";
 import {
   WalletCoreApiClient,
@@ -40,6 +41,7 @@ import {
   WalletCoreResponseType,
 } from "@gnu-taler/taler-wallet-core";
 import {
+  ExtensionNotification,
   MessageFromBackend,
   MessageFromFrontendBackground,
   MessageFromFrontendWallet,
@@ -60,14 +62,6 @@ export interface ExtendedPermissionsResponse {
 }
 
 export interface BackgroundOperations {
-  freeze: {
-    request: number;
-    response: void;
-  };
-  sum: {
-    request: number[];
-    response: number;
-  };
   resetDb: {
     request: void;
     response: void;
@@ -76,6 +70,18 @@ export interface BackgroundOperations {
     request: void;
     response: void;
   };
+  reinitWallet: {
+    request: void;
+    response: void;
+  };
+  getNotifications: {
+    request: void;
+    response: WalletEvent[];
+  };
+  clearNotifications: {
+    request: void;
+    response: void;
+  };
   setLoggingLevel: {
     request: {
       tag?: string;
@@ -83,16 +89,10 @@ export interface BackgroundOperations {
     };
     response: void;
   };
-  // containsHeaderListener: {
-  //   request: void;
-  //   response: ExtendedPermissionsResponse;
-  // };
-  // toggleHeaderListener: {
-  //   request: boolean;
-  //   response: ExtendedPermissionsResponse;
-  // };
 }
 
+export type WalletEvent = { notification: WalletNotification, when: AbsoluteTime }
+
 export interface BackgroundApiClient {
   call<Op extends keyof BackgroundOperations>(
     operation: Op,
@@ -101,11 +101,13 @@ export interface BackgroundApiClient {
 }
 
 export class BackgroundError<T = any> extends Error {
-  public errorDetail: TalerErrorDetail & T;
+  public readonly errorDetail: TalerErrorDetail & T;
+  public readonly cause: Error;
 
-  constructor(title: string, e: TalerErrorDetail & T) {
+  constructor(title: string, e: TalerErrorDetail & T, cause: Error) {
     super(title);
     this.errorDetail = e;
+    this.cause = cause;
   }
 
   hasErrorCode<C extends keyof DetailsMap>(
@@ -138,7 +140,7 @@ class BackgroundApiClientImpl implements BackgroundApiClient {
         throw new BackgroundError(operation, {
           code: TalerErrorCode.GENERIC_UNEXPECTED_REQUEST_ERROR,
           when: AbsoluteTime.now(),
-        });
+        }, error);
       }
       throw error;
     }
@@ -146,6 +148,7 @@ class BackgroundApiClientImpl implements BackgroundApiClient {
       throw new BackgroundError(
         `Background operation "${operation}" failed`,
         response.error,
+        TalerError.fromUncheckedDetail(response.error),
       );
     }
     logger.trace("response", response);
@@ -169,14 +172,20 @@ class WalletApiClientImpl implements WalletCoreApiClient {
         payload,
       };
       response = await platform.sendMessageToBackground(message);
-    } catch (e) {
-      logger.error("Error calling backend", e);
-      throw new Error(`Error contacting backend: ${e}`);
+    } catch (error) {
+      if (error instanceof Error) {
+        throw new BackgroundError(operation, {
+          code: TalerErrorCode.GENERIC_UNEXPECTED_REQUEST_ERROR,
+          when: AbsoluteTime.now(),
+        }, error);
+      }
+      throw error;
     }
     if (response.type === "error") {
       throw new BackgroundError(
         `Wallet operation "${operation}" failed`,
         response.error,
+        TalerError.fromUncheckedDetail(response.error)
       );
     }
     logger.trace("got response", response);
@@ -186,7 +195,7 @@ class WalletApiClientImpl implements WalletCoreApiClient {
 
 function onUpdateNotification(
   messageTypes: Array<NotificationType>,
-  doCallback: undefined | (() => void),
+  doCallback: undefined | ((n: WalletNotification) => void),
 ): () => void {
   //if no callback, then ignore
   if (!doCallback)
@@ -194,9 +203,9 @@ function onUpdateNotification(
       return;
     };
   const onNewMessage = (message: MessageFromBackend): void => {
-    const shouldNotify = messageTypes.includes(message.type);
+    const shouldNotify = message.type === "wallet" && messageTypes.includes(message.notification.type);
     if (shouldNotify) {
-      doCallback();
+      doCallback(message.notification);
     }
   };
   return platform.listenToWalletBackground(onNewMessage);
@@ -206,14 +215,23 @@ export type WxApiType = {
   wallet: WalletCoreApiClient;
   background: BackgroundApiClient;
   listener: {
+    trigger: (d: ExtensionNotification) => void;
     onUpdateNotification: typeof onUpdateNotification;
   };
 };
 
+function trigger(w: ExtensionNotification) {
+  platform.triggerWalletEvent({
+    type: "web-extension",
+    notification: w,
+  })
+}
+
 export const wxApi = {
   wallet: new WalletApiClientImpl(),
   background: new BackgroundApiClientImpl(),
   listener: {
+    trigger,
     onUpdateNotification,
   },
 };
