@@ -24,19 +24,23 @@
  * Imports.
  */
 import {
+  AbsoluteTime,
   LogLevel,
   Logger,
+  OpenedPromise,
+  SetTimeoutTimerAPI,
+  TalerError,
   TalerErrorCode,
+  TalerErrorDetail,
   getErrorDetailFromException,
   makeErrorDetail,
+  openPromise,
   setGlobalLogLevelFromString,
   setLogLevelFromString
 } from "@gnu-taler/taler-util";
 import { HttpRequestLibrary } from "@gnu-taler/taler-util/http";
 import {
   DbAccess,
-  OpenedPromise,
-  SetTimeoutTimerAPI,
   SynchronousCryptoWorkerFactoryPlain,
   Wallet,
   WalletOperations,
@@ -44,7 +48,6 @@ import {
   deleteTalerDatabase,
   exportDb,
   importDb,
-  openPromise,
 } from "@gnu-taler/taler-wallet-core";
 import {
   BrowserHttpLib,
@@ -53,7 +56,7 @@ import {
 import { MessageFromFrontend, MessageResponse } from "./platform/api.js";
 import { platform } from "./platform/background.js";
 import { ExtensionOperations } from "./taler-wallet-interaction-loader.js";
-import { BackgroundOperations } from "./wxApi.js";
+import { BackgroundOperations, WalletEvent } from "./wxApi.js";
 
 /**
  * Currently active wallet instance.  Might be unloaded and
@@ -91,6 +94,17 @@ async function resetDb(): Promise<void> {
   await reinitWallet();
 }
 
+//FIXME: maybe circular buffer
+const notifications: WalletEvent[] = []
+async function getNotifications(): Promise<WalletEvent[]> {
+  return notifications
+}
+
+async function clearNotifications(): Promise<void> {
+  notifications.splice(0,notifications.length)
+}
+
+
 async function runGarbageCollector(): Promise<void> {
   const dbBeforeGc = currentDatabase;
   if (!dbBeforeGc) {
@@ -111,44 +125,30 @@ async function runGarbageCollector(): Promise<void> {
   logger.info("imported");
 }
 
-function freeze(time: number): Promise<void> {
-  return new Promise((res, rej) => {
-    setTimeout(res, time);
-  });
-}
-
-async function sum(ns: Array<number>): Promise<number> {
-  return ns.reduce((prev, cur) => prev + cur, 0);
-}
-
 const extensionHandlers: ExtensionHandlerType = {
-  isInjectionEnabled,
   isAutoOpenEnabled,
+  isDomainTrusted,
 };
 
-async function isInjectionEnabled(): Promise<boolean> {
-  const settings = await platform.getSettingsFromStorage();
-  return settings.injectTalerSupport === true;
-}
-
 async function isAutoOpenEnabled(): Promise<boolean> {
   const settings = await platform.getSettingsFromStorage();
   return settings.autoOpen === true;
 }
 
+async function isDomainTrusted(): Promise<boolean> {
+  const settings = await platform.getSettingsFromStorage();
+  return settings.injectTalerSupport === true;
+}
+
 const backendHandlers: BackendHandlerType = {
-  freeze,
-  sum,
   resetDb,
   runGarbageCollector,
+  getNotifications,
+  clearNotifications,
+  reinitWallet,
   setLoggingLevel,
 };
 
-// async function containsHeaderListener(): Promise<ExtendedPermissionsResponse> {
-//   const result = await platform.containsTalerHeaderListener();
-//   return { newValue: result };
-// }
-
 async function setLoggingLevel({
   tag,
   level,
@@ -163,10 +163,13 @@ async function setLoggingLevel({
     setLogLevelFromString(tag, level);
   }
 }
+let nextMessageIndex = 0;
 
 async function dispatch<
   Op extends WalletOperations | BackgroundOperations | ExtensionOperations,
 >(req: MessageFromFrontend<Op> & { id: string }): Promise<MessageResponse> {
+  nextMessageIndex = (nextMessageIndex + 1) % (Number.MAX_SAFE_INTEGER - 100);
+
   switch (req.channel) {
     case "background": {
       const handler = backendHandlers[req.operation] as (req: any) => any;
@@ -229,19 +232,26 @@ async function dispatch<
     case "wallet": {
       const w = currentWallet;
       if (!w) {
+        const lastError: TalerErrorDetail = walletInit.lastError instanceof TalerError ?
+          walletInit.lastError.errorDetail : undefined
+
         return {
           type: "error",
           id: req.id,
           operation: req.operation,
           error: makeErrorDetail(
             TalerErrorCode.WALLET_CORE_NOT_AVAILABLE,
-            {},
-            "wallet core not available",
+            { lastError },
+            `wallet core not available${!lastError ? "": `,last error: ${lastError.hint}`}`,
           ),
         };
       }
-
-      return await w.handleCoreApiRequest(req.operation, req.id, req.payload);
+      //multiple client can create the same id, send the wallet an unique key
+      const newId = `${req.id}_${nextMessageIndex}`
+      const resp = await w.handleCoreApiRequest(req.operation, newId, req.payload);
+      //return to the client the original id
+      resp.id = req.id
+      return resp
     }
   }
 
@@ -265,16 +275,25 @@ async function reinitWallet(): Promise<void> {
   }
   currentDatabase = undefined;
   // setBadgeText({ text: "" });
-  let httpLib: HttpRequestLibrary;
   let cryptoWorker;
   let timer;
 
+  const httpFactory = (): HttpRequestLibrary => {
+    if (platform.useServiceWorkerAsBackgroundProcess()) {
+      return new ServiceWorkerHttpLib({
+        // enableThrottling: false,
+      });
+    } else {
+      return new BrowserHttpLib({
+        // enableThrottling: false,
+      });
+    }
+  };
+
   if (platform.useServiceWorkerAsBackgroundProcess()) {
-    httpLib = new ServiceWorkerHttpLib();
     cryptoWorker = new SynchronousCryptoWorkerFactoryPlain();
     timer = new SetTimeoutTimerAPI();
   } else {
-    httpLib = new BrowserHttpLib();
     // We could (should?) use the BrowserCryptoWorkerFactory here,
     // but right now we don't, to have less platform differences.
     // cryptoWorker = new BrowserCryptoWorkerFactory();
@@ -286,25 +305,38 @@ async function reinitWallet(): Promise<void> {
   logger.info("Setting up wallet");
   const wallet = await Wallet.create(
     indexedDB as any,
-    httpLib as any,
+    httpFactory as any,
     timer,
     cryptoWorker,
-    {
-      features: {
-        allowHttp: settings.walletAllowHttp,
-      },
-    },
   );
   try {
-    await wallet.handleCoreApiRequest("initWallet", "native-init", {});
+    await wallet.handleCoreApiRequest("initWallet", "native-init", {
+      config: {
+        testing: {
+          emitObservabilityEvents: settings.showWalletActivity,
+        },
+        features: {
+          allowHttp: settings.walletAllowHttp,
+        },
+      }
+    });
   } catch (e) {
     logger.error("could not initialize wallet", e);
     walletInit.reject(e);
     return;
   }
   wallet.addNotificationListener((message) => {
-    logger.info("wallet -> ui", message);
-    platform.sendMessageToAllChannels(message);
+    if (settings.showWalletActivity) {
+      notifications.push({
+        notification: message,
+        when: AbsoluteTime.now()
+      })
+    }
+
+    platform.sendMessageToAllChannels({
+      type: "wallet",
+      notification: message,
+    });
   });
 
   platform.keepAlive(() => {
@@ -329,10 +361,12 @@ export async function wxMain(): Promise<void> {
   logger.trace("starting");
   const afterWalletIsInitialized = reinitWallet();
 
+  logger.trace("reload on new version");
   platform.registerReloadOnNewVersion();
 
   // Handlers for messages coming directly from the content
   // script on the page
+  logger.trace("listen all channels");
   platform.listenToAllChannels(async (message) => {
     //wait until wallet is initialized
     await afterWalletIsInitialized;
@@ -340,64 +374,15 @@ export async function wxMain(): Promise<void> {
     return result;
   });
 
+  logger.trace("register all incoming connections");
   platform.registerAllIncomingConnections();
 
+  logger.trace("redirect if first start");
   try {
-    await platform.registerOnInstalled(() => {
+    platform.registerOnInstalled(() => {
       platform.openWalletPage("/welcome");
     });
   } catch (e) {
     console.error(e);
   }
-
-  // platform.registerDeclarativeRedirect();
-  // if (false) {
-  /**
-   * this is not working reliable on chrome, just
-   * intercepts queries after the user clicks the popups
-   * which doesn't make sense, keeping it to make more tests 
-   */
-
-  // if (await isHeaderListenerEnabled()) {
-  //   if (await platform.getPermissionsApi().containsHostPermissions()) {
-  //     try {
-  //       platform.registerTalerHeaderListener();
-  //     } catch (e) {
-  //       logger.error("could not register header listener", e);
-  //     }
-  //   } else {
-  //     await platform.getPermissionsApi().requestHostPermissions()
-  //   }
-  // }
-
-  // On platforms that support it, also listen to external
-  // modification of permissions.
-  // platform.getPermissionsApi().addPermissionsListener((perm, lastError) => {
-  //   logger.info(`permission added: ${perm}`,)
-  //   if (lastError) {
-  //     logger.error(
-  //       `there was a problem trying to get permission ${perm}`,
-  //       lastError,
-  //     );
-  //     return;
-  //   }
-  //   platform.registerTalerHeaderListener();
-  // });
-
-  // }
 }
-
-
-// async function toggleHeaderListener(
-//   newVal: boolean,
-// ): Promise<ExtendedPermissionsResponse> {
-//   logger.trace("new extended permissions value", newVal);
-//   if (newVal) {
-//     platform.registerTalerHeaderListener();
-//     return { newValue: true };
-//   }
-
-//   const rem = await platform.getPermissionsApi().removeHostPermissions();
-//   logger.trace("permissions removed:", rem);
-//   return { newValue: false };
-// }
